;;; LCD Archive Entry:
;;; w3-mode|William M. Perry|wmperry@indiana.edu|
;;; Major mode for browsing World Wide Web nodes|
;;; 94-2-11|2.0.7|Location undetermined
;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 1993, 1994 by William M. Perry (wmperry@indiana.edu)
;;;
;;; This file is not part of GNU Emacs, but the same permissions apply.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; This is a major mode for browsing documents written in Hypertext Markup ;;;
;;; Language (HTML).  These documents are typicallly part of the World Wide ;;;
;;; Web (WWW), a project to create a global information net in hypertext    ;;;
;;; format.				                                    ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 1993, 1994 by William M. Perry (wmperry@indiana.edu)	    ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(if (not noninteractive) (progn
			   (or (featurep 'efs)
			       (featurep 'efs-auto)
			       (require 'ange-ftp))))

(if (not (fboundp 'gopher-dispatch-object))
    (autoload 'gopher-dispatch-object "gopher" "Get a gopher doc." t))
(if (not (fboundp 'html-mode))
    (autoload 'html-mode "html-mode" "Edit html documents." t))

(require 'w3-vars)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions that might not exist in old versions of emacs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(or (fboundp 'add-hook)
    (defun add-hook (hook-var function &optional at-end)
      "Add a function to a hook.
First argument HOOK-VAR (a symbol) is the name of a hook, second
 argument FUNCTION is the function to add.
Third (optional) argument AT-END means to add the function at the end
 of the hook list instead of the beginning.  If the function is already
 present, this has no effect.
Returns nil if FUNCTION was already present in HOOK-VAR, else new
 value of HOOK-VAR."
      (if (not (boundp hook-var)) (set hook-var nil))
      (let ((old (symbol-value hook-var)))
	(if (or (not (listp old)) (eq (car old) 'lambda))
	    (setq old (list old)))
	(if (w3-member function old)
	    nil
	  (set hook-var
	       (if at-end
		   (append old (list function)) ; don't nconc
		 (cons function old)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Access authorization functions for the w3 browser
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Base 64 encoding functions
;;; This code was converted to lisp code by me from the C code in
;;; ftp://cs.utk.edu/pub/MIME/b64encode.c
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defvar w3-b64-encoding
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
 "The string to use to encode with base 64.")

(defun b0 (x) (aref w3-b64-encoding (lsh x -18)))
(defun b1 (x) (aref w3-b64-encoding (logand (lsh x -12) 63)))
(defun b2 (x) (aref w3-b64-encoding (logand (lsh x -6) 63)))
(defun b3 (x) (aref w3-b64-encoding (logand x 63)))

(defun b64-encode (str)
  "Do base64 encoding on string STR and return the encoded string.
This code was converted to lisp code by me from the C code in
ftp://cs.utk.edu/pub/MIME/b64encode.c."
  (let (
	(word 0)			; The word to translate
	w1 w2 w3
	)
    (cond
     ((> (length str) 3)
      (format "%s%s"
	      (b64-encode (substring str 0 3))
	      (b64-encode (substring str 3 nil))))
     ((= (length str) 3)
      (setq w1 (aref str 0)
	    w2 (aref str 1)
	    w3 (aref str 2)
	    word (logior
		  (lsh (logand w1 255) 16)
		  (lsh (logand w2 255) 8)
		  (logand w3 255)))
      (format "%c%c%c%c" (b0 word) (b1 word) (b2 word) (b3 word)))
     ((= (length str) 2)
      (setq w1 (aref str 0)
	    w2 (aref str 1)
	    word (logior
		  (lsh (logand w1 255) 16)
		  (lsh (logand w2 255) 8)
		  0))
      (format "%c%c%c=" (b0 word) (b1 word) (b2 word)))
     ((= (length str) 1)
      (setq w1 (aref str 0)
	    word (logior
		  (lsh (logand w1 255) 16)
		  0))
      (format "%c%c==" (b0 word) (b1 word)))
     (t ""))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Decoding PGP/PEM responses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-decode-pgp/pem (arg)
  "Decode a pgp/pem message and parse it"
  (mc-decrypt-message)
  (w3-parse-mime-headers))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; The authorization code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defvar w3-basic-auth-storage nil
  "Where usernames and passwords are stored.  Its value is an assoc list of
assoc lists.  The first assoc list is keyed by the server name.  The cdr of
this is an assoc list based on the 'directory' specified by the url we are
looking up.")

(defun w3-public-key-exists (entity scheme)
  "Return t iff a key for ENTITY exists using SCHEME"
  (let (retval)
    (save-excursion
      (cond
       ((eq 'pgp scheme)			; PGP encryption
	(set-buffer (get-buffer-create " *keytmp*"))
	(erase-buffer)
	(call-process mc-pgp-path nil t nil "+batchmode" "-kxaf" entity)
	(goto-char (point-min))
	(setq retval (search-forward mc-pgp-key-begin-line nil t)))
       ((eq 'pem scheme)			; PEM encryption
	(set-buffer (find-file-noselect mc-ripem-pubkeyfile))
	(goto-char (point-min))
	(setq retval (search-forward entity nil t))))
      (kill-buffer (current-buffer)))
    retval))

(defun w3-get-server-keys (entity &optional scheme)
  "Make sure the key for ENTITY exists using SCHEME.  Find the "
  (or scheme (setq scheme mc-default-scheme))
  (save-excursion
    (cond
     ((w3-public-key-exists entity scheme) nil)
     (t
      (string-match "\\([^@]+\\)@\\(.*\\)" entity)
      (let ((w3-working-buffer " *w3-get-keys*"))
	(w3-retrieve (format "gopher://%s:79/0%s/w" (w3-match entity 1)
			     (w3-match entity 2)))
	(mc-snarf-keys)
	(kill-buffer w3-working-buffer))))))
   
(defun w3-fetch-with-pgp (url recipient type)
  "Retrieve a document with PGP authentication"
  (or noninteractive (require 'mailcrypt))
  (let ((request (w3-create-mime-request url ""))
	(w3-request-data nil)
	(w3-request-extra-headers nil))
    (save-excursion
      (w3-get-server-keys recipient type)
      (set-buffer (get-buffer-create " *w3-encryption*"))
      (erase-buffer)
      (insert "\n\n" mail-header-separator "\n" request)
      (mc-encrypt-message recipient type)
      (goto-char (point-min))
      (if (re-search-forward (concat "\n" mail-header-separator "\n") nil t)
	  (delete-region (point-min) (point)))
      (setq w3-request-data (buffer-string)
	    w3-request-extra-headers
	    (list (cons "Authorized" (format "%s entity=\"%s\""
					     (cond
					      ((eq type 'pgp) "PGP")
					      ((eq type 'pem) "PEM"))
					     w3-pgp/pem-entity))
		  (cons "Content-type" (format "application/x-www-%s-reply"
					       (cond
						((eq type 'pgp) "pgp")
						((eq type 'pem) "pem")))))))
    (kill-buffer " *w3-encryption*")
    (w3-retrieve (w3-parse-relative-link "/"))))
     
(defun w3-basic-auth (url &optional prompt overwrite)
  "Get the username/password for the specified URL.
If optional argument PROMPT is non-nil, ask for the username/password
to use for the url and its descendants.  If optional third argument
OVERWRITE is non-nil, overwrite the old username/password pair if it
is found in the assoc list."
  (let (server path user pass byserv retval)
    (if (string-match ":/*\\([^/]+\\)/+\\(.*\\)" url)
	(setq server (substring url (match-beginning 1) (match-end 1))
	      path (substring url (match-beginning 2) (match-end 2))))
    (setq byserv (cdr-safe (assoc server w3-basic-auth-storage)))
    (if (not byserv)			; Server not found
	(if prompt
	    (progn
	      (setq user (read-string "Username: " (user-real-login-name))
		    pass (funcall w3-passwd-entry-func "Password: "))
	      (setq w3-basic-auth-storage
		    (cons (list server
				(cons path
				      (setq retval
					    (b64-encode (format "%s:%s"
								user pass)))))
			  w3-basic-auth-storage)))
	  (setq retval nil))
      (progn				; Found the server
	(setq retval (cdr-safe (assoc path byserv)))
	(if (not retval)		; No exact match, check directories
	    (while (and byserv (not retval))
	      (if (string-match (concat (regexp-quote (car (car byserv)))
					"/*[^/]+") path)
		  (setq retval (cdr (car byserv)))
		(setq byserv (cdr byserv)))))
	(if (or (and (not retval) prompt) overwrite)
	    (progn
	      (setq user (read-string "Username: " (user-real-login-name))
		    pass (funcall w3-passwd-entry-func "Password: ")
		    retval (b64-encode (format "%s:%s" user pass))
		    byserv (assoc server w3-basic-auth-storage))
	      (setcdr byserv
		      (cons (cons path retval) (cdr byserv)))))))
    retval))

(defun w3-pubkey-auth (url &optional prompt overwrite)
  "Same as w3-basic-auth, but for public key encryption.  For right now,
this function always returns nil.  Need to read up on public key
authorization schemes in general before I do."
  nil)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; FORMS processing for html+
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(or (boundp 'MULE) (fset 'string-width 'length))

(defun w3-truncate-string (str len &optional pad)
  "Truncate string STR so that string-width of STR is not greater than LEN.
If width of the truncated string is less than LEN, and if a character PAD is
defined, add padding end of it."
  (if (boundp 'MULE)
      (let ((cl (string-to-char-list str)) (n 0) (sw 0))
	(if (<= (string-width str) len) str
	  (while (<= (setq sw (+ (char-width (nth n cl)) sw)) len)
	    (setq n (1+ n)))
	  (string-match (make-string n ?.) str)
	  (setq str (substring str 0 (match-end 0))))
	(if pad (concat str (make-string (- len (string-width str)) pad)) str))
    (format "%s%s"
	    (if (> (length str) len) (substring str 0 len) str)
	    (if (or (null pad) (> (length str) len))
		""
	      (make-string (- len (length str)) pad)))))

(defun w3-split (str del)
  "Split the string STR, with DEL (a regular expression) as the delimiter.
Returns an assoc list that you can use with completing-read."
  (let (x y)
    (while (string-match del str)
      (setq y (substring str 0 (match-beginning 0))
	    str (substring str (match-end 0) nil))
      (if (not (string-match "^[ \\\t]+$" y))
	  (setq x (cons (list y y) x))))
    (if (not (equal str ""))
	(setq x (cons (list str str) x)))
    x))

(defun w3-handle-form (num)
  "Parse a form, expecting the region to be narrowed between the <FORM>
and </FORM> tags."
  (goto-char (point-min))
  (let ((x (if (re-search-forward "<FORM\\([^>]*\\)>" nil t)
	       (buffer-substring (match-beginning 1) (match-end 1)) ""))
	action method ;enctype
	st nd input type name default value checked size maxlength prompt
	options formatfun)
    (if (string-match "METHOD *=+ *\"*[ \\\t]*\\([^>\" ]*\\)\"*" x)
	(setq method (w3-match x 1)) (setq method "GET"))
    (if (string-match "ACTION *=+ *\"*[ \\\t]*\\([^>\" ]*\\)\"*" x)
	(setq action (w3-match x 1)) (setq action (w3-view-url t)))
    (setq action (cons method action))
    (while (re-search-forward "<INPUT" nil t)
      (setq st (match-beginning 0)
	    nd (progn
		 (goto-char st)
		 (if (re-search-forward ">" nil t)
		     (match-end 0)
		   (progn (end-of-line) (point))))
	    input (buffer-substring st nd))
      (delete-region st nd)
      (if (string-match "TYPE[ \\\t]*=+[ \\\t]*\"*\\([^ >\"]+\\)\"*" input)
	  (setq type (upcase
		      (substring input (match-beginning 1) (match-end 1))))
	(setq type ""))
      (if (string-match "NAME[ \\\t]*=+[ \\\t]*\"+\\([^ \"]*\\)\"+" input)
	  (setq name (substring input (match-beginning 1) (match-end 1)))
	(setq name ""))
      (if (string-match "VALUE[ \\\t]*=+[ \\\t]*\"\\([^\"]*\\)\"" input)
	  (setq value (substring input (match-beginning 1) (match-end 1)))
	(setq value ""))
      (if (string-match "SIZE[ \\\t]*=+[ \\\t]*\"*\\([^ \"]+\\)\"*" input)
	  (setq size (string-to-int
		      (substring input (match-beginning 1) (match-end 1))))
	(setq size 20))
      (if (string-match "MAXLENGTH[ \\\t]*=+[ \\\t]*\"*\\([^ \"]+\\)\"*" input)
	  (setq maxlength (string-to-int
			   (substring input (match-beginning 1)
				      (match-end 1))))
	(setq maxlength 10000))
      (setq default value)
      (if (string-match "CHECKED" input)
	  (setq checked t)
	(setq checked nil))
      (if (equal type "CHECKBOX")
	  (setq default checked))
      (if (and (equal type "CHECKBOX")
	       (equal "" value))
	  (setq value "on"))
      (setq formatfun (intern (concat "w3-form-format-" (downcase type))))
      (if (not (fboundp formatfun))
	  (setq formatfun 'w3-form-format-unknown))
      (setq prompt (funcall formatfun value size checked))
      (goto-char st)
      (w3-insert prompt)
      (w3-add-zone st (point) w3-node-style
		   (list 'w3form
			 action type name default value
			 checked size maxlength num options) t)
      (w3-insert (if (w3-member type '("CHECKBOX" "RADIO")) "" " ")))))

(defun w3-form-format-int (&rest args)
  "Format an integer entry field"
  (w3-truncate-string (or (nth 0 args) "") (nth 1 args) ?_))

(fset 'w3-form-format-url 'w3-form-format-int)
(fset 'w3-form-format-float 'w3-form-format-int)
(fset 'w3-form-format-date 'w3-form-format-int)

(defun w3-form-format-reset (&rest args)
  "Format a reset button"
  (if (string= (nth 0 args) "") "Reset fields" (nth 0 args)))

(defun w3-form-format-password (&rest args)
  "Format a password entry field"
  (let ((value (or (nth 0 args) ""))
	(size (nth 1 args)))
    (format "%s%s"
	    (if (>= (length value) size) (make-string size ?*)
	      (make-string (length value) ?*))
	    (if (>= (length value) size) ""
	      (make-string (- size (length value)) ?.)))))

(defun w3-form-format-checkbox (&rest args)
  "Format a checkbox entry"
  (let ((checked (nth 2 args)))
    (format "[%s]" (if checked "X" " "))))

(fset 'w3-form-format-radio 'w3-form-format-checkbox)

(defun w3-form-format-submit (&rest args)
  "Format a form submit button"
  (if (string= (nth 0 args) "") "Submit this form" (nth 0 args)))

(defun w3-form-format-text (&rest args)
  "Format a text field"
  (w3-truncate-string (nth 0 args) (nth 1 args) ?_))

(defun w3-form-format-textarea (&rest args)
  "Format a multiline text box"
  "Multiline text entry")

(fset 'w3-form-format- 'w3-form-format-text)
(fset 'w3-form-format-unknown 'w3-form-format-text)

(defun w3-handle-textareas (num action)
  "Handle &lt;SELECT&gt; tags in a form"
  (let (
	(type "TEXTAREA")
	name
	default
	value
	checked
	size
	(maxlength 100)
	options
	st
	nd
	input)
    (goto-char (point-min))
    (while (re-search-forward "<TEXTAREA\\([^>]*\\)>\\\n*" nil t)
      (setq input (buffer-substring (match-beginning 1) (match-end 1))
	    st (prog1 (match-beginning 0)
		 (replace-match ""))
	    nd (if (re-search-forward "\\n*</TEXTAREA[^>]*>\\\n*" nil t)
		   (progn
		     (replace-match "")
		     (match-beginning 0))
		 (progn
		   (end-of-line)
		   (point)))
	    value (buffer-substring st nd)
	    options value
	    default value)
      (if (string-match "NAME[ \\\t]*=+[ \\\t]*\"*\\([^ \"]+\\)\"*" input)
	  (setq name (substring input (match-beginning 1) (match-end 1)))
	(setq name ""))
      (delete-region st nd)
      (w3-insert "Multiline Text Entry Area")
      (w3-add-zone st (point) w3-node-style
		   (list 'w3form
			 action type name default value checked size
			 maxlength num options) t))))

(defun w3-handle-selections (num action)
  "Handle &lt;SELECT&gt; tags in a form"
  (let (
	(type "OPTION")
	name
	default
	value
	checked
	size
	(maxlength 100)
	options
	parm
	st
	nd
	input
	mult				; Multiple input?
	(longest 0)			; Longest selection?
	sel)
    (goto-char (point-min))
    (while (re-search-forward "<SELECT\\([^>]*\\)>\\\n*" nil t)
      (setq input (buffer-substring (match-beginning 1) (match-end 1)))
      (setq options nil
	    mult nil
	    value nil
	    default nil)
      (replace-match "")
      (setq st (match-beginning 0))
      (setq nd
	    (if (re-search-forward "\\n*</SELECT[^>]*>\\\n*" nil t)
		(progn
		  (replace-match "")
		  (match-beginning 0))
	      (progn
		(end-of-line)
		(point))))
      (goto-char st)
      (while (re-search-forward "[\\\n ]*<OPTION\\([^>]*\\)> *\\([^<]*\\)"
				nd t)
	(setq parm (buffer-substring (match-beginning 1) (match-end 1))
	      sel (w3-eat-trailing-space
		   (buffer-substring (match-beginning 2) (match-end 2)))
	      options (cons (cons sel sel) options))
	(if (> (string-width sel) longest) (setq longest (string-width sel)))
	(if (string-match "selected" parm) (setq default sel)))
      (setq longest (+ 5 longest))
      (delete-region st nd)
      (if (string-match "NAME[ \\\t]*=+[ \\\t]*\"*\\([^ \"]+\\)\"*" input)
	  (setq name (substring input (match-beginning 1) (match-end 1)))
	(setq name ""))
      (if (string-match "SIZE[ \\\t]*=+[ \\\t]*\"*\\([^ \"]+\\)\"*" input)
	  (setq size (string-to-int (substring input (match-beginning 1)
					       (match-end 1))))
	(setq size 20))
      (setq mult (string-match "SEVERAL\\|MULTIPLE" input))
      (cond
       (mult
;	(let ((x 1) (z (/ (window-width) longest)))
	  (goto-char st)
	  (w3-insert "<UL>\n")
	  (while options
	    (w3-insert
	     (format
	      "<LI><INPUT TYPE=\"CHECKBOX\" NAME=\"%s\" VALUE=\"%s\" %s>%s\n"
	      name (car (car options))
	      (if (equal default (car (car options)))
		  "CHECKED" "") (car (car options))))
	    (setq options (cdr options)))
	  (w3-insert "</UL>\n"));)
       (t
	(if (not default)
	    (setq value (car (nth (1- (length options)) options)))
	  (setq value default))
	(setq default value)
	(goto-char st)
	(setq value (w3-truncate-string value maxlength 32))
	(w3-insert (w3-form-format-text value size nil))
	(w3-add-zone st (point) w3-node-style
		     (list 'w3form
			   action type name default value checked size
			   maxlength num options) t))))))

(defun w3-handle-forms ()
  "Take care of parsing an entire buffer for <FORM> tags."
  (set-buffer w3-working-buffer)
  (let ((num 1)
	x y z)
    (goto-char (point-min))
    (while (re-search-forward "<FORM\\([^>]*\\)>" nil t)
      (setq x (buffer-substring (match-beginning 1) (match-end 1)))
      (narrow-to-region (match-beginning 0)
			(if (re-search-forward "</FORM>" nil t) (match-end 0)
			  (point-max)))
      (if (string-match "METHOD *=+ *\"*[ \\\t]*\\([^>\" ]*\\)\"*" x)
	  (setq y (w3-match x 1)) (setq y "GET"))
      (if (string-match "ACTION *=+ *\"*[ \\\t]*\\([^>\" ]*\\)\"*" x)
	  (setq z (w3-match x 1)) (setq z (w3-view-url t)))
      (w3-handle-selections num (cons y z))
      (w3-handle-textareas num (cons y z))
      (w3-handle-form num)
      (setq num (1+ num))
      (w3-replace-regexp "</*FORM[^>]*>" "")
      (widen))))

(defun w3-do-text-entry (formobj zone)
  "Read in a multiline text entry area."
  (let ((data (list formobj zone (current-buffer)))
	(buff (get-buffer-create (format "%d:%s" (nth 9 formobj)
					 (nth 3 formobj)))))
    (switch-to-buffer-other-window buff)
    (indented-text-mode)
    (erase-buffer)
    (w3-insert (nth 5 formobj))
    (setq w3-current-last-buffer data)
    (message "Press C-c C-c when finished with text entry.")
    (local-set-key "\C-c\C-c" 'w3-finish-text-entry)))

(defun w3-finish-text-entry ()
  "Finish a text entry area"
  (interactive)
  (if w3-current-last-buffer
      (let* ((formobj (nth 0 w3-current-last-buffer))
	     (zone (nth 1 w3-current-last-buffer))
	     (buff (nth 2 w3-current-last-buffer))
	     (actn (nth 1 formobj))
	     (type (nth 2 formobj))
	     (name (nth 3 formobj))
	     (deft (nth 4 formobj))
	     (valu (buffer-string))
	     (chkd (nth 6 formobj))
	     (size (nth 7 formobj))
	     (maxl (nth 8 formobj))
	     (ident (nth 9 formobj))
	     (options (nth 10 formobj))
	     (st nil)
	     (nd nil))
	(local-set-key "\C-c\C-c" 'undefined)
	(kill-buffer (current-buffer))
	(delete-window)
	(if (not (and buff (bufferp buff) (buffer-name buff)))
	    (message "Could not find the form buffer for this text!")
	  (switch-to-buffer buff)
	  (if buffer-read-only (toggle-read-only))
	  (setq st (w3-zone-start zone)
		nd (w3-zone-end zone))
	  (w3-delete-zone zone)
	  (w3-add-zone st nd w3-node-style
		       (list 'w3form actn type name deft valu chkd
			     size maxl ident options) t)))
    (if (not buffer-read-only) (toggle-read-only))
    nil))

(defun w3-do-form-entry (formobj zone)
  "Read in a data entry field defined by FORMOBJ, covered by zone ZONE."
  (let* ((actn (nth 1 formobj))
	 (type (nth 2 formobj))
	 (name (nth 3 formobj))
	 (deft (nth 4 formobj))
	 (valu (nth 5 formobj))
	 (chkd (nth 6 formobj))
	 (size (nth 7 formobj))
	 (maxl (nth 8 formobj))
	 (ident (nth 9 formobj))
	 (options (nth 10 formobj))
	 (st (w3-zone-start zone))
	 (nd (w3-zone-end zone))
	 (submit-it nil)
	 (formatfun (intern (concat "w3-form-format-" (downcase type)))))
    (if (not (equal "SUBMIT" type))
	(save-excursion
	  (if (not (fboundp formatfun))
	      (setq formatfun 'w3-form-format-unknown))
	  (if buffer-read-only (toggle-read-only))
	  (cond
	   ((equal "CHECKBOX" type) (setq chkd (not chkd)))
	   ((equal "RADIO" type) nil)
	   ((equal "TEXTAREA" type) nil)
	   ((equal "RESET" type) (w3-revert-form ident))
	   (t (setq valu
		    (w3-read-correct-format type name options ident valu))))
	  (cond
	   ((equal "RESET" type) nil)
	   ((equal "RADIO" type) (w3-set-radio-button zone))
	   ((equal "TEXTAREA" type)
	    (if (not buffer-read-only) (toggle-read-only))
	    (w3-do-text-entry formobj zone))
	   (t
	    (w3-delete-zone zone)
	    (delete-region st nd)
	    (goto-char st)
	    (w3-insert (funcall formatfun valu size chkd))
	    (w3-add-zone st (point) w3-node-style
			 (list 'w3form actn type name deft valu chkd
			       size maxl ident options) t)
	    (if (not buffer-read-only) (toggle-read-only))
	    (if w3-running-FSF19 (setq w3-zones-list (w3-only-links)))
	    (if (boundp 'MULE)
		(w3-mule-attribute-zones w3-zones-list w3-mule-attribute))
	    ))
	  (cond
	   ((string-match "^isindex$" name) (setq submit-it t))
	   ((string-match "^internal-gopher$" name) (setq submit-it 'gopher))))
      (w3-submit-form ident nil))
    (if submit-it (w3-submit-form ident submit-it))))

(defun w3-zones-matching (actn)
  "Return a list of data entry zones in form ACTN"
  (let* ((big (w3-all-zones))
	 (data nil)
	 (result nil)
	 (cur nil))
    (while big
      (setq data (w3-zone-data (car big))
	    cur (car big)
	    big (cdr big))
      (if (and
	   (eq (nth 0 data) 'w3form)
	   (equal (nth 9 data) actn))
	  (setq result (cons cur result))))
    result))

(defun w3-revert-form (actn)
  "Revert all values for form ACTN to their defaults"
  (save-excursion
    (let* ((zones (w3-zones-matching actn))
	   actn data type name deft valu chkd size maxl idnt strt end cur
	   options formatfun
	   )
      (if buffer-read-only (toggle-read-only))
      (while zones
	(setq data (w3-zone-data (car zones))
	      actn (nth 1 data)
	      type (nth 2 data)
	      name (nth 3 data)
	      deft (nth 4 data)
	      valu (nth 5 data)
	      chkd (nth 6 data)
	      size (nth 7 data)
	      maxl (nth 8 data)
	      idnt (nth 9 data)
	      options (nth 10 data)
	      cur (car zones)
	      strt (w3-zone-start cur)
	      end  (w3-zone-end cur)
	      zones (cdr zones)
	      formatfun (intern (concat "w3-form-format-" (downcase type))))
	(if (not (fboundp formatfun)) (setq formatfun 'w3-form-format-unknown))
	(cond
	 ((or (w3-member type '("SUBMIT" "TEXTAREA" "RESET"))) nil)
	 (t
	  (if (w3-member type '("RADIO" "CHECKBOX"))
	      (setq chkd deft)
	    (setq valu deft))
	  (if w3-running-FSF19 (goto-char strt)
	    (w3-delete-zone cur))
	  (delete-region strt end)
	  (goto-char strt)
	  (w3-insert (funcall formatfun valu size chkd))
	  (w3-add-zone strt (point) w3-node-style
		       (list 'w3form actn type name deft valu chkd
			     size maxl idnt options) t))))
      (if (not buffer-read-only) (toggle-read-only)))
    (if w3-running-FSF19
	(setq w3-zones-list (w3-only-links)))
    (if (boundp 'MULE)
	(w3-mule-attribute-zones w3-zones-list w3-mule-attribute))
    ))

(defun w3-form-encode (result &optional isindex-query)
  "Create a string suitably enocoded for a URL request from the form
data in RESULT."
  (let ((query ""))
    (cond
     ((eq isindex-query 'gopher)	; Gopher searching by hypertext
      (concat "\t" (nth 5 (car result))))
     (isindex-query			; Isindex handling by hypertext
      (w3-hexify-string (nth 5 (car result))))
     (t					; Normal submission of form
      (while result			; This is a little convoluted, but
					; gets only checkboxes that are
					; and ignores submit & reset buttons
	(if (and (not
		  (and (w3-member (nth 2 (car result)) '("CHECKBOX" "RADIO"))
		       (not (nth 6 (car result)))))
		 (not (w3-member (nth 2 (car result)) '("SUBMIT" "RESET"))))
	    (setq query (format "%s=%s&%s" (nth 3 (car result))
				(w3-hexify-string (nth 5 (car result)))
				query)))
	(setq result (cdr result)))))
    query))

(defun w3-form-encode-ask-block (result)
  "Submit a gopher ask block to the server."
  (let ((query ""))
    ;;; This is different than the w3-form-encode function, because
    ;;; gopher+ will expect all the checkboxes/etc, even if they are
    ;;; not turned on.  Should still ignore RADIO boxes that are not
    ;;; active though.
  (while result	      
    (if (and (not (and (string= (nth 2 (car result)) "RADIO")
		       (not (nth 6 (car result)))))
	     (not (w3-member (nth 2 (car result)) '("SUBMIT" "RESET"))))
	(setq query (format "%s\r\n%s" query (nth 5 (car result)))))
    (setq result (cdr result)))
  (concat query "\r\n.\r\n")))

(defun w3-submit-form (actn isindex)
  "Submit form entry fields matching ACTN as their action identifier."
  (let* ((result (mapcar 'w3-zone-data (w3-zones-matching actn)))
	 (query (w3-form-encode result isindex))
	 (themeth (upcase (or (car (nth 1 (car result))) "GET")))
	 (theurl (cdr (nth 1 (car result)))))
    (cond
     ((eq isindex 'gopher) (w3-fetch (concat theurl query)))
     ((string= "GOPHER-ASK" themeth)
      (setq query (w3-form-encode-ask-block result))
      (w3-fetch (concat theurl (w3-hexify-string (concat "\t+\t1\n+-1\r\n"
							 query)))))
     ((string= "POST" themeth)
      (let ((w3-request-method themeth)
	    (w3-request-data query)
	    (w3-request-extra-headers
	     (list (cons "Content-type"
			 "application/x-www-form-urlencoded"))))
	(w3-maybe-relative theurl)))
     ((string= "GET" themeth)
      (w3-maybe-relative (format "%s?%s" theurl query)))
     (t (message "Unknown submit method: %s" themeth)))))

(defun w3-matching-radios (ext)
  "Return a list of all zones containing radio buttons with the same name
as that in EXT."
  (let* ((big (w3-all-zones))
	 (idnt (nth 9 (w3-zone-data ext)))
	 (name (nth 3 (w3-zone-data ext)))
	 data cur result)
    (while big
      (setq data (w3-zone-data (car big))
	    cur (car big)
	    big (cdr big))
      (if (and
	   (eq (nth 0 data) 'w3form)
	   (equal (nth 9 data) idnt)
	   (equal (nth 3 data) name))
	  (setq result (cons cur result))))
    result))
      
(defun w3-set-radio-button (ext)
  "Set the radio button at EXT to be on.  Will automatically
toggle other radio butons with the same name to be off."
  (save-excursion
    (let* ((result (w3-matching-radios ext))
	   (idnt (nth 9 (w3-zone-data ext)))
	   (name (nth 3 (w3-zone-data ext)))
	   actn type deft valu chkd size maxl strt end data options)
      (while result
	(setq data (w3-zone-data (car result))
	      actn (nth 1 data)
	      type (nth 2 data)
	      name (nth 3 data)
	      deft (nth 4 data)
	      valu (nth 5 data)
	      chkd (nth 6 data)
	      size (nth 7 data)
	      maxl (nth 8 data)
	      idnt (nth 9 data)
	      options (nth 10 data)
	      strt (w3-zone-start (car result))
	      end (w3-zone-end (car result)))
	(cond
	 ((and chkd (not (w3-zone-eq
			  ext (car result)))) ; Not supposed to be chkd
	  (w3-delete-zone (car result))	      ; but is.
	  (goto-char strt)
	  (delete-region strt end)
	  (setq chkd nil)
	  (w3-insert (funcall 'w3-form-format-radio valu size chkd))
	  (w3-add-zone strt (point) w3-node-style
		       (list 'w3form actn type name deft valu chkd size maxl
			     idnt options) t))
	 ((and (not chkd) (w3-zone-eq
			   ext (car result))) ; Supposed to be chkd
	  (w3-delete-zone (car result))       ; but isn't.
	  (goto-char strt)
	  (delete-region strt end)
	  (setq chkd t)
	  (w3-insert (funcall 'w3-form-format-radio valu size chkd))
	  (w3-add-zone strt (point) w3-node-style
		       (list 'w3form actn type name deft valu chkd size maxl
			     idnt options) t))
	 (t nil)) ; not supposed to be checked, and isn't
	(setq result (cdr result))))
    (if (not buffer-read-only) (toggle-read-only))
    (if w3-running-FSF19 (setq w3-zones-list (w3-only-links)))
    (if (boundp 'MULE)
	(w3-mule-attribute-zones w3-zones-list w3-mule-attribute))
    ))
			

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Type checking for FORMS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Date checking, taken from edb.el
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconst weekday-alist
 '(("Sunday" . 0) ("Monday" . 1) ("Tuesday" . 2) ("Wednesday" . 3)
   ("Thursday" . 4) ("Friday" . 5) ("Saturday" . 6)
   ("Tues" . 2) ("Thurs" . 4)
   ("Sun" . 0) ("Mon" . 1) ("Tue" . 2) ("Wed" . 3)
   ("Thu" . 4) ("Fri" . 5) ("Sat" . 6)))

(defconst full-monthname-alist
  '(("January" . 1) ("February" . 2) ("March" . 3) ("April" . 4)
    ("May" . 5) ("June" . 6) ("July" . 7) ("August" . 8)
    ("September" . 9) ("October" . 10) ("November" . 11) ("December" . 12)))


(defconst monthabbrev-alist
  '(("Jan" . 1) ("Feb" . 2) ("Mar" . 3) ("Apr" . 4) ("May" . 5) ("Jun" . 6)
    ("Jul" . 7) ("Aug" . 8) ("Sep" . 9) ("Oct" . 10) ("Nov" . 11) ("Dec" . 12)))

(defconst monthname-alist
  (append monthabbrev-alist
	  full-monthname-alist
	  '(("Sept" . 9))))

(defconst monthname-regexp
  (concat "\\("
	  (mapconcat (function car)
		     monthname-alist
		     "\\|")
	  "\\)\\.?"))

(defconst weekday-regexp
  (concat "\\("
	  (mapconcat (function car)
		     weekday-alist
		     "\\|")
	  "\\)\\.?"))

(defconst monthnumber-regexp "\\(0?[1-9]\\|1[0-2]\\)")
(defconst monthnumber-regexp-two-char "\\(0[1-9]\\|1[0-2]\\)")

(defconst monthday-regexp "\\(0?[1-9]\\|[12][0-9]\\|3[01]\\)")
(defconst monthday-regexp-two-char "\\([0-2][0-9]\\|3[01]\\)")

(defconst full-year-regexp "[0-2][0-9][0-9][0-9]")
(defconst short-year-regexp "[0-9][0-9]")

(defconst year-regexp (concat "\\(" full-year-regexp
			      "\\|" short-year-regexp "\\)"))

(defconst elt-separator-regexp "[ -.,/']+")

(defconst date-regexps
  (list
   ;; MMDDYY
   (cons (concat monthname-regexp
		 elt-separator-regexp
		 monthday-regexp
		 "\\("
		 elt-separator-regexp
		 year-regexp
		 "\\)?")
	 '(4 nil 1 2))
   (cons (concat monthnumber-regexp
		 elt-separator-regexp
		 monthday-regexp
		 "\\("
		 elt-separator-regexp
		 year-regexp
		 "\\)?")
	 '(4 1 nil 2))
   ;; DDMMYY
   (cons (concat monthday-regexp
		 elt-separator-regexp
		 monthname-regexp
		 "\\("
		 elt-separator-regexp
		 year-regexp
		 "\\)?")
	 '(4 nil 2 1))
   (cons (concat "\\("
		 monthday-regexp
		 elt-separator-regexp
		 "\\)?"
		 monthname-regexp
		 elt-separator-regexp
		 year-regexp)
	 '(4 nil 3 2))
   (cons (concat monthday-regexp
		 elt-separator-regexp
		 monthnumber-regexp
		 elt-separator-regexp
		 "\\(" full-year-regexp "\\)")
	 '(3 2 nil 1))
   ;; YYMMDD
   ;; Using year-regexp instead of full-year-regexp is ambiguous (consider
   ;; 11-11-11), but we already tried MMDDYY and it failed.
   (cons (concat year-regexp
		 elt-separator-regexp
		 monthname-regexp
		 elt-separator-regexp
		 monthday-regexp)
	 '(1 nil 2 3))
   (cons (concat year-regexp
		 elt-separator-regexp
		 monthnumber-regexp
		 elt-separator-regexp
		 monthday-regexp)
	 '(1 2 nil 3))
   ;; YYMMDD, no separators
   ;; This is ambiguous.
   (cons (concat year-regexp
		 monthnumber-regexp-two-char "?"
		 monthday-regexp-two-char "?")
	 '(1 2 nil 3))
   ;; WWMMDDYY
   (cons (concat weekday-regexp
		 elt-separator-regexp
		 monthname-regexp
		 elt-separator-regexp
		 monthday-regexp
		 "\\("
		 elt-separator-regexp
		 year-regexp
		 "\\)?")
	 '(5 nil 2 3))
   ;; WWDDMMYY
   (cons (concat weekday-regexp
		 elt-separator-regexp
		 monthday-regexp
		 elt-separator-regexp
		 monthname-regexp
		 "\\("
		 elt-separator-regexp
		 year-regexp
		 "\\)?")
	 '(5 nil 3 2))
   ;; ctime
   (cons (concat
	  weekday-regexp
	  " "
	  monthname-regexp
	  "  ?"
	  monthday-regexp
	  ;; time of day
	  " [0-9:]+ "
	  "\\(" full-year-regexp "\\)")
	 '(4 nil 2 3))
   )
  "Assoc list of regexps and match locators.
A match locator is a list of four numbers indicating which submatch of the
regexp contains the year, month number, month name, and day of the month.
The list elements may be nil if that information is not available.")

(defun w3-datep (date-string)
  "Parse DATE-STRING, and return a date object; err if the parse is invalid.
If DATE-STRING contains only whitespace, return a null date object.
If DATE-STRING is nil, use the result of `parse-date-default-function' instead."
  (let ((regexp-alist date-regexps)
	result)
    (if (zerop (length date-string))	;if empty string,
	(setq result t)			;empty date is kosher
      ;; regexp-alist is nulled if a match is found
      (progn
	(while regexp-alist
	  (if (string-match (concat "^" (car (car regexp-alist)) "$")
			    date-string)
	      (setq regexp-alist nil
		    result t)
	    ;; string-match failed
	    (setq regexp-alist (cdr regexp-alist))))))
    result))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Integer checking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-intp (str)
  "Integer checker"
  (string-match "^[0-9]+$" str))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Floating point checking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-floatp (str)
  "Floating point checker"
  (let (x y)
    (if (string-match "^\\([0-9]+\\)\\.\\([0-9]+\\)$" str)
	(progn
	  (setq x (substring str (match-beginning 1) (match-end 1))
		y (substring str (match-beginning 2) (match-end 2)))
	  (and (w3-intp x) (w3-intp y)))
      (w3-intp str))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; URL Checking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-urlp (str)
  "URL checker..."
  (string-match w3-nonrelative-link str))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Option list checking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-optionp (val)
  "Option list checker"
  (if (null val)
      (progn
	(message "Please make a selection from the menu")
	nil)
    t))

(defun w3-textp (str) t)		; don't care whats in a text field
(fset 'w3-p 'w3-textp)			; for default of "" to be text
(fset 'w3-passwordp 'w3-textp)		; don't care whats in a paswd either
(fset 'w3-textareap 'w3-textp)		; try this - might work

(defun w3-read-correct-format (type name options num value)
  "Read in a FORMS entry with type TYPE, and do typechecking"
  (let ((func (read (format "w3-%sp" (downcase type))))
	(valu value) exitp)
    (while (not exitp)
      (cond
       ((or (equal "TEXT" type)
	    (equal "" type))
	(setq valu (read-string "Enter text: " valu)))
       ((or (equal "FLOAT" type)
	    (equal "INT" type))
	(setq valu (read-string "Enter numeric value: " valu)))
       ((equal "PASSWORD" type)
	(setq valu (funcall w3-passwd-entry-func "Enter password:" valu)))
       ((equal "OPTION" type)
	(if (or (not window-system)
		(not (fboundp 'w3-x-popup-menu)))
	    (setq valu (completing-read "Please choose: " options nil t valu))
	  (setq valu (w3-x-popup-menu
		      (if (and (boundp 'last-input-event)
			       (listp last-input-event))
			  last-input-event
			(list (list (current-column) 1)
			      (selected-window)))
		      (list "WWW"
			    (cons "Select An Item" options)))))
	(if (consp valu) (setq valu (car valu))))
       ((equal "DATE" type)
	(setq valu (read-string "Enter date: " valu)))
       ((equal "URL" type)
	(setq valu (read-string "Enter valid URL: " valu))))
      (if (not (fboundp func)) (setq func 'w3-textp))
      (if (funcall func valu)
	  (setq exitp t)
	(progn
	  (message "Wrong format for type %s, try again." (downcase type))
	  (sit-for 2))))
    valu))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Gopher and Gopher+ support
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-convert-ask-to-form (ask)
  "Convert a Gopher+ ASK block into a form.  Returns a string to be inserted
into a buffer to create the form."
  (let ((form "<FORM METHOD=\"GOPHER-ASK\"><UL PLAIN>")
	(type "")
	(x 0)
	(parms ""))
    (while (string-match "^\\([^:]+\\): +\\(.*\\)" ask)
      (setq parms (w3-match ask 2)
	    type (w3-strip-leading-spaces (downcase (w3-match ask 1)))
	    x (1+ x)
	    ask (substring ask (if (= (length ask) (match-end 0))
				   (match-end 0) (1+ (match-end 0))) nil))
      (cond
       ((string= "note" type) (setq form (concat form parms)))
       ((or (string= "ask" type)
	    (string= "askf" type)
	    (string= "choosef" type))
	(setq parms (mapcar 'car (nreverse (w3-split parms "\\\t")))
	      form (format "%s\n<LI>%s<INPUT name=\"%d\" DEFAULT=\"%s\">"
			   form (or (nth 0 parms) "Text:")
			   x (or (nth 1 parms) ""))))
       ((string= "askp" type)
	(setq parms (mapcar 'car (nreverse (w3-split parms "\\\t")))
	      form (format
		    "%s\n<LI>%s<INPUT name=\"%d\" TYPE=\"PASSWORD\" DEFAULT=\"%s\">"
		    form			   ; Earlier string
		    (or (nth 0 parms) "Password:") ; Prompt
		    x				   ; Name
		    (or (nth 1 parms) "") 	   ; Default value
		    )))
       ((string= "askl" type)
	(setq parms (mapcar 'car (nreverse (w3-split parms "\\\t")))
	      form (format "%s\n<LI>%s<TEXTAREA NAME=\"%d\">%s</TEXTAREA>"
			   form			 ; Earlier string
			   (or (nth 0 parms) "") ; Prompt string
			   x			 ; Name
			   (or (nth 1 parms) "") ; Default value
			   )))
       ((or (string= "select" type)
	    (string= "choose" type))
	(setq parms (mapcar 'car (nreverse (w3-split parms "\\\t")))
	      form (format "%s\n<LI>%s<SELECT NAME=\"%d\">" form (car parms) x)
	      parms (cdr parms))
	(if (null parms) (setq parms (list "Yes" "No")))
	(while parms
	  (setq form (concat form "<OPTION>" (car parms) "\n")
		parms (cdr parms)))
	(setq form (concat form "</SELECT>")))))
    (concat form "\n<LI><INPUT TYPE=\"SUBMIT\""
	    " VALUE=\"Submit Gopher+ Ask Block\"></UL></FORM>")))

(defun w3-grok-gopher-link (str)
  "Return a list of link attributes from a gopher string.  Order is:
title, type, selector string, server, port, gopher-plus?"
  (let ((sterm (concat "^"
		       (mapconcat (function (lambda (x) "\\([^\\\t\\\n]*\\)"))
				  (make-list 4 'x) "\\\t")
		       "\\\t*\\([^\\\r\\\n]*\\)\\\r*$")))
    (if (string-match sterm str)
	(list (w3-match str 1)
	      (concat (char-to-string (aref (w3-match str 1) 0))
		      (w3-match str 2))
	      (w3-match str 3) (w3-match str 4) (w3-match str 5))
      (make-list 5 nil))))

(defun w3-is-gopher-directory-listing (&optional buffer)
  "Return t iff buffer BUFFER contains a gopher directory listing.
BUFFER defaults to w3-working-buffer."
  (let ((sterm (concat "^"
		       (mapconcat (function (lambda (x) "\\([^\\\t\\\n]*\\)"))
				  (make-list 4 'x) "\\\t")
		       "\\\t*\\([^\\\r\\\n]*\\)\\\r*$")))
    (save-excursion
      (set-buffer (or buffer w3-working-buffer))
      (goto-char (point-min))
      (while (looking-at "\\\n") (delete-char 1))
      (re-search-forward sterm nil t))))

(defun w3-format-gopher-link (gophobj)
  "Insert a gopher link as an <A> tag"
  (let ((title (nth 0 gophobj))
	(ref   (nth 1 gophobj))
	(type  (if (> (length (nth 0 gophobj)) 0)
		   (substring (nth 0 gophobj) 0 1) ""))
	(serv  (nth 2 gophobj))
	(port  (nth 3 gophobj))
	(plus  (nth 4 gophobj))
	(desc  nil))
    (if (and (equal type "")
	     (> (length title) 0))
	(setq type (substring title 0 1)))
    (setq title (and title (substring title 1 nil)))
    (setq desc (or (cdr (assoc type w3-gopher-labels)) "(UNK)"))
    (setq desc 
	  (format
	   "<IMG SRC=\"%sw3-gopher-%s.xbm\" ALT=\"%s\">"
	   w3-icon-directory type desc))
    (cond
     ((null ref) "")
     ((equal type "8")
      (format "<LI> %s <A HREF=\"telnet://%s:%s/%s\">%s</A>\n"
	      desc serv (concat port plus) ref title))
     ((equal type "T")
      (format "<LI> %s <A HREF=\"tn3270://%s:%s/%s\">%s</A>\n"
	      desc serv (concat port plus) ref title))
     (t (format "<LI> %s <A METHODS=%s HREF=\"gopher://%s:%s/%s\">%s</A>\n"
		desc type serv (concat port plus)
		(w3-hexify-string ref) title)))))

(defun w3-parse-gopher (&optional buffer)
  "Parse out a gopher response"
  (save-excursion
    (set-buffer (or buffer w3-working-buffer))
    (w3-replace-regexp (regexp-quote "&") "&amp;")
    (w3-replace-regexp (regexp-quote ">") "&gt;")
    (w3-replace-regexp (regexp-quote "<") "&lt;")
    (w3-replace-regexp "\\\n*\\.\\\n*\\'" "\n")
    (goto-char (point-max)) (w3-insert "\n")
    (goto-char (point-min))
    (let ((objs nil))
        (goto-char (point-min))
	(while (re-search-forward "^\\(.*\\)\\\r*\\\n" nil t)
	  (if (string= "" (buffer-substring (match-beginning 1) (match-end 1)))
	      nil
	    (setq objs (cons
			(w3-grok-gopher-link
			 (prog1
			     (buffer-substring (match-beginning 1)
					       (match-end 1))
			   (replace-match ""))) objs))))
	(setq objs (nreverse objs))
	(goto-char (point-min))
	(w3-insert "<TITLE>"
		(cond
		 ((string= "" w3-current-file) "Gopher root")
		 ((string= "1/" w3-current-file) "Gopher root")
		 ((string= "1" w3-current-file) "Gopher root")
		 ((string-match (format "^[%s]+/" w3-gopher-types)
				w3-current-file)
		  (substring w3-current-file 2 nil))
		 (t w3-current-file))
		"</TITLE><OL>")
	(while objs
	  (w3-insert (w3-format-gopher-link (car objs)))
	  (setq objs (cdr objs)))
	(w3-insert "</OL>"))))	

(defun w3-gopher-retrieve (host port selector &optional wait-for)
  "Fetch a gopher object and don't mess with it at all"
  (let ((proc (w3-open-stream "*gopher*" w3-working-buffer
			      host (if (stringp port) (string-to-int port)
				     port)))
	(len nil)
	(parsed nil)
	(buff (current-buffer)))
    (w3-clear-tmp-buffer)
    (setq w3-current-file selector
	  w3-current-port port
	  w3-current-server host
	  w3-current-last-buffer buff
	  w3-current-type "gopher")
    (if (> (length selector) 0)
	(setq selector (substring selector 1 nil)))
    (if (stringp proc)
	(message proc)
      (save-excursion
	(process-send-string proc (concat selector "\n"))
	(while (and (or (not wait-for)
			(progn
			  (goto-char (point-min))
			  (not (re-search-forward wait-for nil t))))
		    (memq (process-status proc) '(run open)))
	  (if (not parsed)
	      (cond
	       ((and (eq ?+ (char-after 1))
		     (memq (char-after 2)
			   (list ?0 ?1 ?2 ?3 ?4 ?5 ?6 ?7 ?8 ?9)))
		(setq parsed (copy-marker 2)
		      len (read parsed))
		(delete-region (point-min) parsed))
	       ((and (eq ?+ (char-after 1))
		     (eq ?- (char-after 2)))
		(setq len nil
		      parsed t)
		(goto-char (point-min))
		(delete-region (point-min) (progn
					     (end-of-line)
					     (point))))
	       ((and (eq ?- (char-after 1))
		     (eq ?- (char-after 2)))
		(setq parsed t
		      len nil)
		(goto-char (point-min))
		(delete-region (point-min) (progn
					     (end-of-line)
					     (point))))))
	  (if len (w3-lazy-message "Read %d of %d bytes (%d%%)" (point-max) len
				   (/ (* (point-max) 100) len))
	    (w3-lazy-message "Read %d bytes." (point-max)))
	  (accept-process-output))
	(condition-case ()
	    (delete-process proc)
	  (error nil))
	(w3-replace-regexp (regexp-quote "\r") "")
	(w3-replace-regexp "^\\\.\\\n" "")
	(w3-replace-regexp "^\\\.\r*$\\\n*" "")
	(w3-replace-regexp "\\\n*Connection closed.*\\\n*" "")
	(w3-replace-regexp "\\\n*Process .*gopher.*\\\n*" "")))))

(defun w3-do-gopher-cso-search (descr)
  "Do a gopher CSO search and return a plaintext document"
  (let ((host (nth 0 descr))
	(port (nth 1 descr))
	(file (nth 2 descr))
	search-type search-term)
    (string-match "search-by=\\([^&]+\\)" file)
    (setq search-type (w3-match file 1))
    (string-match "search-term=\\([^&]+\\)" file)
    (setq search-term (w3-match file 1))
    (w3-gopher-retrieve host port (format "2query %s=%s"
					  search-type search-term) "^[2-9]")
    (goto-char (point-min))
    (w3-replace-regexp "^-[0-9][0-9][0-9]:[0-9]*:" "")
    (w3-replace-regexp "^[^15][0-9][0-9]:.*" "")
    (w3-replace-regexp "^[15][0-9][0-9]:\\(.*\\)" "<H1>\\1</H1>&ensp;<PRE>")
    (goto-char (point-min))
    (w3-insert "<TITLE>Results of CSO search</TITLE>\n"
	    "<H1>" search-type " = " search-term "</H1>\n")
    (goto-char (point-max))
    (w3-insert "</PRE>")))
	    

(defun w3-do-gopher (descr)
  "Fetch a gopher object"
  (let ((host (nth 0 descr))
	(port (nth 1 descr))
	(file (nth 2 descr))
	(type (nth 3 descr))
	(extr (nth 4 descr))
	(buff (current-buffer))
	parse-gopher)
    (cond
     ((and				; Gopher CSO search
       (equal type "www/gopher-cso-search")
       (string-match "search-by=" file)) ; With a search term in it
      (w3-do-gopher-cso-search descr)
      (setq type "text/html"))
     ((equal type "www/gopher-cso-search") ; Blank CSO search
      (w3-clear-tmp-buffer)
      (w3-insert "<TITLE> CSO SEARCH </TITLE>\n"
	      "<H1> This is a CSO search </H1>\n"
	      "<HR>\n"
	      "<FORM><LI> Search by: <SELECT NAME=\"search-by\">\n"
	      "<OPTION>Name<OPTION>Phone<OPTION>Email<OPTION>Address"
	      "</SELECT>\n<LI> Search for: <INPUT NAME=\"search-term\">\n"
	      "<LI>&ensp;<INPUT TYPE=\"submit\" VALUE=\"Submit query\">\n"
	      "</UL></FORM><HR>")
      (setq type "text/html"
	    parse-gopher t))
     ((and
       (equal type "www/gopher-search")	; Ack!  Mosaic-style search href
       (string-match "\\\t" file))	; and its got a search term in it!
      (w3-gopher-retrieve host port file)
      (setq type "www/gopher"
	    parse-gopher t))
     ((and
       (equal type "www/gopher-search")	; Ack!  Mosaic-style search href
       (string-match "\\?" file))	; and its got a search term in it!
      (setq file (concat (substring file 0 (match-beginning 0)) "\t"
			 (substring file (match-end 0) nil)))
      (w3-gopher-retrieve host port file)
      (setq type "www/gopher"
	    parse-gopher t))
     ((equal type "www/gopher-search")	; Ack!  Mosaic-style search href
      (setq type "text/html"
	    parse-gopher t)
      (w3-clear-tmp-buffer)
      (w3-insert "<TITLE>Gopher Server</TITLE>\n")
      (w3-insert "<H1>Searchable Gopher Index</H1>")
      (w3-insert "<HR>Enter the search keywords below<P>")
      (w3-insert "<FORM><INPUT NAME=\"internal-gopher\">&ensp;</FORM><HR>"))
     ((null extr)			; Normal Gopher link
      (w3-gopher-retrieve host port file)
      (setq parse-gopher t))
     ((eq extr 'gopher+)		; A gopher+ link
      (w3-gopher-retrieve host port (concat file "\t+"))
      (setq parse-gopher t))
     ((eq extr 'ask-block)		; A gopher+ interactive query
      (w3-gopher-retrieve host port (concat file "\t!")) ; Fetch the info
      (goto-char (point-min))
      (cond
       ((re-search-forward "^\\+ASK:[ \\\t\\\r]*" nil t) ; There is an ASK
	(let ((x (buffer-substring (1+ (point))
				   (or (re-search-forward "^\\+[^:]+:" nil t)
				       (point-max)))))
	  (erase-buffer)
	  (w3-insert (w3-convert-ask-to-form x))
	  (setq type "text/html" parse-gopher t)))
       (t (setq parse-gopher t)))))
    (if (and parse-gopher (or (equal type "www/gopher")
			      (equal file "")))
;			      (w3-is-gopher-directory-listing)))
	(progn
	  (funcall (w3-mime-viewer "www/gopher"))
	  (setq type "text/html"
		w3-current-mime-viewer (w3-mime-viewer type))))
    (setq w3-current-mime-type (or type "text/plain")
	  w3-current-mime-viewer (w3-mime-viewer type)
	  w3-current-file file
	  w3-current-port port
	  w3-current-server host
	  w3-current-last-buffer buff
	  w3-current-type "gopher")))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Code for printing out roman numerals
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-decimal-to-roman (n)
  "Convert from decimal to roman numerals"
  (let ((curmod 1000)
	(str "")
	(j 7)
	i2 k curcnt)
    (while (>= curmod 1)
      (if (>= n curmod)
	  (progn
	    (setq curcnt (/ n curmod)
		  n (- n (* curcnt curmod)))
	    (if (= 4 (% curcnt 5))
		(setq i2 (+ j (if (> curcnt 5) 1 0))
		      str (format "%s%c%c" str
				  (aref w3-roman-characters (1- j))
				  (aref w3-roman-characters i2)))
	      (progn
		(if (>= curcnt 5)
		    (setq str (format "%s%c" str (aref w3-roman-characters j))
			  curcnt (- curcnt 5)))
		(setq k 0)
		(while (< k curcnt)
		  (setq str (format "%s%c" str
				    (aref w3-roman-characters (1- j)))
			k (1+ k)))))))
      (setq curmod (/ curmod 10)
	    j (- j 2)))
    str))
		     

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions for formatting nested lists in html
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-build-table (indent-level)
  "Build a Definition List"
  (set-buffer w3-working-buffer)
  (let ((x (make-string indent-level 9))
	(y (or (nth (1- indent-level) (cdr (assoc "DL" w3-list-chars-assoc)))
	       "*")))
    (goto-char (point-min))
    (w3-replace-regexp "[ \\\t\\\n]*<DD[^>]*>[ \\\t\\\n]*"
		       (concat "\n" x "  "))
    (w3-replace-regexp "[ \\\t\\\n]*<DT[^>]*>[ \\\t\\\n]*"
		       (concat "\n" x y " "))
    (w3-replace-regexp "</*DL[^>]*>" "\n")))

(defun w3-build-ordered-list (indent-level &optional roman)
  "Builds ordered lists"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let ((x 1) y
	(z (or (nth (1- indent-level) (cdr (assoc "OL" w3-list-chars-assoc)))
	       "."))
	parm url alt)
    (goto-char (point-min))
    (while (re-search-forward "<LI\\([^>]*\\)>[ \\\t]*" nil t)
      (setq parm (buffer-substring (match-beginning 1) (match-end 1)))
      (replace-match "")
      (if (string-match "SRC[ \\\t]*=+[ \\\t]*\"\\([^\"]*\\)\"" parm)
	  (setq url (substring parm (match-beginning 1) (match-end 1)))
	(setq url nil))
      (if (string-match "ALT[ \\\t]*=+[ \\\t]*\"\\([^\"]*\\)\"" parm)
	  (setq alt (substring parm (match-beginning 1) (match-end 1)))
	(setq alt nil))
      (setq y (format "\n%s%3s%s "
		      (make-string indent-level 9)
		      (if roman (w3-decimal-to-roman x)
			(format "%d" x)) z))
      (cond
       ((and (null alt) (null url)	; No alt/img and not in emacs 18
	     (or w3-running-FSF19 w3-running-epoch
		 w3-running-lemacs w3-running-old-lemacs))
	(w3-insert y))
       ((and (null alt) (null url))	; No alt/img and in emacs 18
	(insert-before-markers y))
       ((and url (fboundp 'w3-insert-graphic))
	(w3-insert "^")
	(w3-insert-graphic
	 url (1- (point)) 'center
	 (or alt (nth (1- indent-level) (cdr (assoc "OL" w3-list-chars-assoc)))
	     ".")))
       (alt (w3-insert alt)))
      (setq x (1+ x))))
  (goto-char (point-min))
  (w3-replace-regexp "</*OL[^>]*>" "\n"))

(defun w3-build-unordered-list (indent-level)
  "Build unordered lists"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let ((x (format "\n%s%s " (make-string indent-level 9)
		   (or (nth (1- indent-level)
			    (cdr (assoc "UL" w3-list-chars-assoc))) "*")))
	parm url alt)
    (while (re-search-forward "<LI\\([^>]*\\)>" nil t)
      (setq parm (buffer-substring (match-beginning 1) (match-end 1)))
      (replace-match "")
      (if (string-match "SRC[ \\\t]*=+[ \\\t]*\"\\([^\"]*\\)\"" parm)
	  (setq url (substring parm (match-beginning 1) (match-end 1)))
	(setq url nil))
      (if (string-match "ALT[ \\\t]*=+[ \\\t]*\"\\([^\"]*\\)\"" parm)
	  (setq alt (substring parm (match-beginning 1) (match-end 1)))
	(setq alt nil))
      (cond
       ((and (null alt) (null url)	; No alt/img and not in emacs 18
	     (or w3-running-FSF19 w3-running-epoch
		 w3-running-lemacs w3-running-old-lemacs))
	(w3-insert x))
       ((and (null alt) (null url))	; No alt/img and in emacs 18
	(insert-before-markers x))
       ((and url (fboundp 'w3-insert-graphic))
	(w3-insert "^")
	(w3-insert-graphic
	 url (1- (point)) 'center
	 (or alt (nth (1- indent-level) (cdr (assoc "UL" w3-list-chars-assoc)))
	     "*")))
       (alt (w3-insert alt)))))
  (goto-char (point-min))
  (w3-replace-regexp "</*\\(UL\\|DIR\\|MENU\\)[^>]*>" "\n"))

(defun w3-handle-lists (indent-level)
  "Handle building of lists - INDENT-LEVEL is how many tabs to use
to indent from the left margin."
  (let ((type (upcase (buffer-substring (match-beginning 1) (match-end 1))))
	(parm (upcase (buffer-substring (match-beginning 2) (match-end 2))))
	(pos nil))
    (while (setq pos (w3-sublists-exist type))
      (goto-char pos)
      (setq indent-level (1+ indent-level)
	    type (upcase (buffer-substring
			  (match-beginning 1) (match-end 1)))))
    (narrow-to-region (- (point) (+ 2 (length type) (length parm)))
		      (progn
			(re-search-forward (format "</%s>" type) nil t)
			(point)))
    (cond
     ((equal "OL" type) (w3-build-ordered-list indent-level
					       (string-match "ROMAN" parm)))
     ((equal "DL" type) (w3-build-table indent-level))
     (t (w3-build-unordered-list indent-level)))
    (w3-fill-paragraphs-in-list indent-level type)
    (widen)))

(defun w3-fill-paragraphs-in-list (indent-level type)
  "This will fill all the paragraphs within the current list.  INDENT-LEVEL
is the number of tabs to use as the leading fill."
  (w3-replace-regexp "\\\n\\\n+" "\n")
  (goto-char (point-min))
  (let ((fill-prefix
	 (format "%s%s" (make-string indent-level 9)
		 (if (equal type "OL") "     " "  "))) st nd ptag)
    (while (re-search-forward "^[^\\\n]" nil t)
      (setq st (progn (beginning-of-line) (point))
	    nd (progn (end-of-line) (point)))
      (save-restriction
	(narrow-to-region st nd)
	(goto-char (point-min))
	(while (re-search-forward " *<P\\([^>]*\\)> *" nil t)
	  (setq ptag (buffer-substring (match-beginning 1) (match-end 1)))
	  (setq st (match-beginning 0))
	  (replace-match "\n\n")
	  (if (string-match "ID=\"\\([^\"]+\\)\"" ptag)
	      (w3-add-zone st (progn (end-of-line) (point))
			   w3-default-style
			   (cons 'w3par
				 (list (substring ptag (match-beginning 1)
						  (match-end 1))
				       nil nil nil nil nil nil nil)))))
	(while (re-search-forward "^[^\\\t]" nil t)
		(beginning-of-line)
		(insert-before-markers fill-prefix))
	(fill-region (point-min) (point-max))))))

(defun w3-sublists-exist (type)
  "Figure out if there are sublists in the current list.  Expects point to
be _AFTER_ the current list tag, and type to be bound to what sort of
list it is (OL, UL, DL, MENU, etc)"
  (save-excursion
    (let* ((thestart  (point))
	   (newend (progn
		     (re-search-forward (format "</%s>" type) nil t)
		     (point))))
      (goto-char thestart)
      (if (re-search-forward "<\\(DL\\|OL\\|UL\\|DIR\\|MENU\\)\\([^>]*\\)>"
			     newend t)
	  (point)
	nil))))

(defun w3-do-lists ()
  (let ((tmp 0)
	(last (point-min)))
    (while (progn
	     (goto-char last)
	     (re-search-forward "<\\(DL\\|OL\\|UL\\|DIR\\|MENU\\)\\([^>]*\\)>"
				nil t))
      (setq last (match-beginning 0))
      (w3-handle-lists 1)
      (setq tmp (1+ tmp))
      (w3-lazy-message "Building lists...%s" (make-string tmp ?.)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for HTTP/1.0 MIME messages                                      ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-parse-mime-types (&optional fname)
  "Parse out the ~/.mime-types files"
  (let (mtype extns x)
    (setq fname (or fname (expand-file-name "~/.mime-types")))
    (and fname (not (and (file-exists-p fname)
			 (file-readable-p fname)))
	 (error "%s is non-existant or unreadable." fname))
    (save-excursion
      (set-buffer (get-buffer-create " *mime-types*"))
      (erase-buffer)
      (insert-file-contents fname)
      (w3-replace-regexp "#.*" "")
      (w3-replace-regexp "\\\n+" "\n")
      (w3-replace-regexp "[ \\\t]+$" "")
      (goto-char (point-min))
      (while (re-search-forward "^\\([^ \\\t\\\n]+\\)[ \\\t]+\\(.*\\)[ \\\t]*\\\n+"
				nil t)
	(setq mtype (buffer-substring (match-beginning 1) (match-end 1))
	      extns (buffer-substring (match-beginning 2) (match-end 2)))
	(replace-match "")
	(setq extns (mapcar (function (lambda (x) (concat "." (car x))))
			    (w3-split extns "[ \\\t\\.]+")))
	(while extns
	  (setq x (w3-extension-to-mime (car extns)))
	  (if (or (not x) w3-mime-mimetypes-overrides)
	      (setq w3-mime-extensions (cons (cons (car extns) mtype)
					     w3-mime-extensions)))
	  (setq extns (cdr extns))))
      (kill-buffer (current-buffer)))))

(defun w3-parse-mailcap (&optional fname)
  "Parse out the ~/.mailcap file"
  (let (x y z mjr)
    (if (not fname)
	(if (and (file-exists-p  (expand-file-name "~/.mailcap"))
		 (file-readable-p (expand-file-name "~/.mailcap")))
	    (setq fname (expand-file-name "~/.mailcap"))
	  (error "%s is non-existant or unreadable"
		 (expand-file-name "~/.mailcap"0))))
    (save-excursion
      (set-buffer (get-buffer-create " *mailcap*"))
      (erase-buffer)
      (insert-file-contents fname)
      (w3-replace-regexp "#.*" "")
      (w3-replace-regexp "\\\n+" "\\\n")
      (w3-replace-regexp "^$\\\n" "")
      (w3-replace-regexp "\\\\[ \\\t\\\n]+" " ")
      (sort-lines nil (point-min) (point-max))
      (while (re-search-forward
	      "\\(^[^\\\n/]+\\)/\\([^;]+\\);[ \\\t]*\\([^;\\\n]+\\).*"
	      nil t)
	(setq x (w3-strip-leading-spaces
		 (w3-eat-trailing-space
		  (buffer-substring (match-beginning 1) (match-end 1))))
	      y (w3-strip-leading-spaces
		 (w3-eat-trailing-space
		  (buffer-substring (match-beginning 2) (match-end 2))))
	      z (w3-strip-leading-spaces
		 (w3-eat-trailing-space
		  (buffer-substring (match-beginning 3) (match-end 3))))
	      mjr (assoc x w3-mime-viewers))
	(if (= ?' (string-to-char z))
	    (setq z (read (substring z 1 nil))))
	(if (null mjr)			; new major area
	    (progn
	      (setq w3-mime-viewers
		    (cons (cons x (list (cons y z))) w3-mime-viewers)))
	  (if (not (w3-in-assoc y (cdr mjr))) ; not in minor area either
	      (setcdr mjr (cons (cons y z) (cdr mjr)))
	    (if w3-mime-mailcap-overrides
		(let ((tmp (w3-in-assoc y (cdr mjr))))
		  (setcar tmp y)
		  (setcdr tmp z)))))))))

(defun w3-mime-viewer (encoding)
  "Get the mime viewer command for ENCODING, return nil if none found.
Expects an argument like text/html, or application/dvi"
  (if (not encoding) (setq encoding "/"))
  (string-match "/" encoding)
  (let* ((major (substring encoding 0 (match-beginning 0)))
	 (minor (substring encoding (1+ (match-beginning 0)) nil))
	 (alist (cdr-safe (assoc major w3-mime-viewers)))
	 (viewr nil))
    (if alist
	(setq viewr (cdr-safe (w3-in-assoc minor alist))))
    viewr))

(defun w3-parse-viewer-types ()
  "Create a string usable for an Accept: header from w3-mime-viewere"
  (let ((tmp w3-mime-viewers)
	mjr mnr (str ""))
    (while tmp
      (setq mnr (cdr (car tmp))
	    mjr (car (car tmp))
	    tmp (cdr tmp))
      (while mnr
	(if (> (+ (% (length str) 60)
		  (length (concat ", " mjr "/" (car (car mnr))))) 60)
	    (setq str (format "%s\nAccept: %s/%s" str mjr
			      (if (string= ".*" (car (car mnr))) "*"
				(car (car mnr)))))
	  (setq str (format "%s, %s/%s" str mjr
			    (if (string= ".*" (car (car mnr))) "*"
			      (car (car mnr))))))
	(setq mnr (cdr mnr))))
    (substring str 2 nil)))
              
(defun w3-create-mime-request (url ref-url)
  "Create a MIME request for URL, referred to by REF-URL, using METH as the
method."
  (if (and w3-use-http2
	   (not (w3-member w3-current-server w3-bad-server-list)))
      (let* ((tmp (w3-view-url t))
	     (tmp1 "")
	     (request nil))
	(while w3-request-extra-headers
	  (setq tmp1 (format
		      "%s%s: %s\n"
		      tmp1
		      (car (car w3-request-extra-headers))
		      (cdr (car w3-request-extra-headers)))
		w3-request-extra-headers (cdr w3-request-extra-headers)))
	(setq request
	      (format
	       (concat
		"%s %s HTTP/1.0\n"			; The request
		"From: %s@%s\n"				; Who its from
		"Accept-encoding: x-compress; x-gzip\n"	; Encoding
		"%s%s"					; Authentication
		"Accept: %s\n"				; Accept-string
		"User-Agent: Emacs-W3/%s\n"		; User agent
		"%s"					; Any extra headers
		"%s"					; Any data
		"\n\n")					; End request
	       (or w3-request-method "GET")
	       url
	       (user-real-login-name)
	       (system-name)
	      (if (w3-basic-auth tmp)
		  (format "Authorization: Basic %s\n" (w3-basic-auth tmp))
		"")
	      (if (w3-pubkey-auth tmp)
		  (format "Authorization: PubKey %s\n" (w3-pubkey-auth tmp))
		"")
	      w3-mime-accept-string
	      w3-version-number
	      (if tmp1 tmp1 "")
	      (if w3-request-data
		  (format "Content-length: %d\n\n%s"
			  (length w3-request-data) w3-request-data) "")))
	request)
    (format "GET %s\n" url)))

(defun w3-parse-mime-headers ()
  "Parse mime headers and remove them from the html"
  (set-buffer w3-working-buffer)
  (let* ((st (point-min))
	 (nd (progn
	       (goto-char (point-min))
	       (re-search-forward "^\r*$" nil t)
	       (1+ (point))))
	 (hdrs (buffer-substring st nd)))
    (delete-region st nd)
    (let (
	  status			; status field (200-500)
	  result			; Assoc-List of MIME headers
	  )
      (if (string-match "^HTTP.+ \\([0-9]+\\) *\\([^\\\n\\\r]*\\)\\\r*$" hdrs)
	  (setq status (string-to-int
			(substring hdrs (match-beginning 1) (match-end 1)))))
      (while (string-match "^\\([^\\\n\\\r:]+\\): *\\([^\\\n\\\r]*\\)\\\r*\\\n"
			   hdrs)
	(setq result (cons
		      (cons (downcase
			     (substring hdrs (match-beginning 1)
					(match-end 1)))
			    (substring hdrs (match-beginning 2) (match-end 2)))
		      result)
	      hdrs (substring hdrs (match-end 0) nil)))
      (setq w3-current-mime-type (cdr (assoc "content-type" result))
	    w3-current-mime-encoding (cdr (assoc "content-encoding" result))
	    w3-current-mime-viewer (w3-mime-viewer w3-current-mime-type)
	    w3-current-mime-headers result)
      (cond
       ((= status 500) nil)		; Internal server error
       ((= status 501) nil)		; Facility not supported
       ((= status 400) nil)		; Bad request - syntax
       ((= status 401)			; Unauthorized access, retry w/auth.
	(let* ((y (cdr (assoc "www-authenticate" result)))
	       (type (downcase (if (string-match "[ \\\t]" y)
				   (substring y 0 (match-beginning 0))
				 y)))
	       (x (intern (concat "w3-" type "-auth"))))
	  (cond
	   ((or (equal "pem" type) (equal "pgp" type))
	    (string-match "entity=\"\\([^\"]+\\)\"" y)
	    (w3-fetch-with-pgp w3-current-file (w3-match y 1) (intern type)))
	   ((fboundp x)
	    (funcall x (w3-view-url t) t (funcall x (w3-view-url t)))
	    (w3-retrieve (w3-view-url t)))
	   (t
	    (goto-char (point-max))
	    (w3-insert "<HR>Sorry, but I do not know how to handle" y
		       " authentication.  If you'd like to write it,"
		       " send it to wmperry@indiana.edu.<HR>")))))
       ((= status 402) nil)		; Payment required, retry w/Chargeto:
       ((= status 403) nil)		; Access is forbidden
       ((= status 404) nil)		; Not found...
       ((or (= status 301)		; Moved - retry with Location: header
	    (= status 302)		; Found - retry with Location: header
	    (= status 303))		; Method - retry with location/method
	(let ((x (w3-view-url t))
	      (redir (or (cdr (assoc "uri" result))
			 (cdr (assoc "location" result))))
	      (redirmeth (or (cdr (assoc "method" result)) "GET")))
	  (if (not (equal x redir))
	      (let ((w3-request-method redirmeth))
		(w3-maybe-relative redir))
	    (progn
	      (goto-char (point-max))
	      (w3-insert "<HR>Error!  This URL tried to redirect me to itself!<P>"
		      "Please notify the server maintainer.")))))
       ((= status 204)			; No response - leave old document
	(kill-buffer w3-working-buffer))
       (t nil))				; All others indicate success
      hdrs)))

(defun w3-is-mime-response ()
  "Determine if the current buffer is a MIME response"
  (set-buffer w3-working-buffer)
  (if (and w3-use-telnet (equal w3-current-type "http"))
      (progn
	(w3-replace-regexp "^\\\r" "")
	(goto-char (point-min))
	(if (re-search-forward
	     (regexp-quote (w3-create-mime-request w3-current-file "")) nil t)
	    (progn
	      (replace-match "")
	      (goto-char (point-min))
	      (kill-line 1)))))
  (let* ((st (point-min))
	 (nd (progn
	       (goto-char (point-min))
	       (re-search-forward "^\r*$" nil t)
	       (if (not (eobp)) (1+ (point)) (point))))
	 (hdrs (buffer-substring st nd)))
    (and (string-match "^HTTP/.+" hdrs)
	 (equal w3-current-type "http")
	 (string-match "^MIME-version:.*" hdrs))))    


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions for compatibility with XMosaic
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for global history file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-write-global-history (&optional fname)
  "Write the global history file into w3-global-history-file"
  (interactive)
  (if (not fname) (setq fname w3-global-history-file))
  (if (not (file-exists-p w3-global-history-file))
      (progn
	(message "Creating history file %s." w3-global-history-file)
	(set-buffer (get-buffer-create " *W3HIST*"))
	(erase-buffer)
	(w3-insert "ncsa-mosaic-history-format-1\nGlobal\n"))
    (progn
      (set-buffer (get-buffer-create " *W3HIST*"))
      (erase-buffer)
      (insert-file-contents w3-global-history-file)))
  (let ((tmp w3-history-list)
	url)
    (while tmp
      (setq url (car (car tmp)))
      (if url
	  (progn
	    (goto-char (point-min))
	    (if (not (re-search-forward (regexp-quote url) nil t))
		(progn
		  (goto-char (point-max))
		  (w3-insert (format "%s %s\n" url (current-time-string)))))))
      (setq tmp (cdr tmp))))
  (write-file w3-global-history-file)
  (kill-buffer (current-buffer)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Hotlist Handling Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-remove-from-hotlist ()
  "Deletes a document from your hotlist file"
  (interactive)
  (save-excursion
    (if (not w3-hotlist) (message "No hotlist in memory!")
      (if (not (file-exists-p w3-hotlist-file))
	  (message "Hotlist file %s does not exist." w3-hotlist-file)
	(let* ((title (car (assoc (completing-read "Delete Document: "
						   w3-hotlist nil t)
				  w3-hotlist)))
	       (buffer (get-buffer-create " *HOTW3*")))
	  (set-buffer buffer)
	  (erase-buffer)
	  (insert-file-contents w3-hotlist-file)
	  (if (re-search-forward (regexp-quote title) nil t)
	      (progn
		(previous-line 1)
		(beginning-of-line)
		(kill-line 2)
		(write-file w3-hotlist-file)
		(setq w3-hotlist (w3-delete-from-alist title w3-hotlist))
		(kill-buffer (current-buffer)))
	    (message "%s was not found in %s" title w3-hotlist-file))))))
  (if (and w3-running-FSF19 (eq window-system 'x))
      (progn
	(delete-menu-item '("Navigate"))
	(w3-build-FSF19-menu))))

(defun w3-rename-hotlist-entry (title)
  "Rename a personal annotation"
  (interactive (list (completing-read "Rename entry: " w3-hotlist nil t)))
  (cond					; Do the error handling first
   ((not w3-hotlist) (error "No hotlist in memory!"))
   ((not (file-exists-p (expand-file-name w3-hotlist-file)))
    (error "Hotlist file %s does not exist." w3-hotlist-file))
   ((not (file-readable-p (expand-file-name w3-hotlist-file)))
    (error "Hotlist file %s exists, but is unreadable." w3-hotlist-file)))
  (save-excursion
    (let ((obj (assoc title w3-hotlist))
	  (used (mapcar 'car w3-hotlist))
	  (buff (get-buffer-create " *HOTW3*"))
	  (new nil)
	  )
      (while (or (null new) (w3-member new used))
	(setq new (read-string "New name: ")))
      (set-buffer buff)
      (erase-buffer)
      (insert-file-contents (expand-file-name w3-hotlist-file))
      (goto-char (point-min))
      (if (re-search-forward (regexp-quote title) nil t)
	  (progn
	    (previous-line 1)
	    (beginning-of-line)
	    (kill-line 2)
	    (w3-insert (format "%s %s\n%s\n" (nth 1 obj) (current-time-string)
			    new))
	    (setq w3-hotlist (cons (list new (nth 1 obj))
				   (w3-delete-from-alist title w3-hotlist)))
	    (write-file w3-hotlist-file)
	    (kill-buffer (current-buffer))
	    (if (and w3-running-FSF19 window-system)
		(progn
		  (delete-menu-item '("Navigate"))
		  (w3-build-FSF19-menu))))
	(message "%s was not found in %s" title w3-hotlist-file)))))    
       
(defun w3-append-hotlist (fname)
  "Append a hotlist to the one in memory"
  (interactive "fAppend hotlist file: ")
  (let ((x w3-hotlist))
    (w3-parse-hotlist fname)
    (setq w3-hotlist (nconc x w3-hotlist))))

(defun w3-parse-hotlist (&optional fname)
  "Read in the hotlist specified by FNAME"
  (if (not fname) (setq fname w3-hotlist-file))
  (setq w3-hotlist nil)
  (if (not (file-exists-p fname))
      (message "%s does not exist!" fname)
    (let* ((old-buffer (current-buffer)) 
	   (buffer (get-buffer-create " *HOTW3*"))
	   cur-link
	   cur-alias)
      (set-buffer buffer)
      (erase-buffer)
      (insert-file-contents fname)
      (goto-char (point-min))
      (while (re-search-forward "^\n" nil t) (replace-match ""))
      (goto-line 3)
      (while (not (equal (point) (point-max)))
	(re-search-forward "^[^ ]*" nil t)
	(setq cur-link (buffer-substring (match-beginning 0) (match-end 0)))
	(setq cur-alias (buffer-substring (progn
					    (forward-line 1)
					    (beginning-of-line)
					    (point))
					  (progn
					    (end-of-line)
					    (point))))
	(if (not (equal cur-alias ""))
	    (setq w3-hotlist (cons (list cur-alias cur-link) w3-hotlist))))
      (kill-buffer buffer)
      (set-buffer old-buffer))))

;;;###autoload
(defun w3-use-hotlist ()
  "Possibly go to a link in your W3/Mosaic hotlist.
This is part of the emacs World Wide Web browser.  It will prompt for
one of the items in your 'hotlist'.  A hotlist is a list of often
visited or interesting items you have found on the World Wide Web."
  (interactive)
  (if (not w3-setup-done) (w3-do-setup))
  (if (not w3-hotlist) (message "No hotlist in memory!")
    (let* ((url (car (cdr (assoc
			   (completing-read "Goto Document: " w3-hotlist nil t)
			   w3-hotlist)))))
      (w3-fetch url))))

(defun w3-add-document-to-hotlist ()
  "Add this documents url to the hotlist"
  (interactive)
  (save-excursion
    (let* ((buffer (get-buffer-create " *HOTW3*"))
	   (title (buffer-name))
	   (url (w3-view-url t)))
      (if (w3-rassoc (list url) w3-hotlist)
	  (error "That item already in hotlist, use w3-rename-hotlist-entry."))
      (set-buffer buffer)
      (erase-buffer)
      (setq w3-hotlist (cons (list title url) w3-hotlist))
      (if (not (file-exists-p w3-hotlist-file))
	  (progn
	    (message "Creating hotlist file %s" w3-hotlist-file)
	    (w3-insert "ncsa-xmosaic-hotlist-format-1\nDefault\n\n")
	    (backward-char 1))
	(progn
	  (insert-file-contents w3-hotlist-file)
	  (goto-char (point-max))
	  (backward-char 1)))
      (w3-insert "\n" url " " (current-time-string) "\n" title)
      (write-file w3-hotlist-file)
      (kill-buffer (current-buffer))))
  (if (and w3-running-FSF19 (eq window-system 'x))
      (progn
	(delete-menu-item '("Navigate" ))
	(w3-build-FSF19-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Annotation server handling
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-fetch-annotations ()
  "Fetch the annotations for the current document"
  (if (not w3-group-annotation-server)
      (error "No group annotation server defined!")
    (let ((cmd (format "ANN_GET /url=\"%s\";=\n" (w3-view-url t)))
	  (proc nil)
	  (tmp 0)
	  (tmp2 0))
      (save-excursion
	(set-buffer (get-buffer-create w3-working-buffer))
	(goto-char (point-max))
	(w3-insert "\n"))
      (setq proc (w3-open-stream "*anno*" (get-buffer-create w3-working-buffer)
				 w3-group-annotation-server
				 w3-group-annotation-port))
      (message "Fetching annotations...")
      (if (processp proc)
	  (progn
	    (process-send-string proc cmd)
	    (while (memq (process-status proc) '(run open))
	      (if (= 0 (setq tmp2 (% (1+ tmp2) 200)))
		  (message "Fetching annotations..%s" (make-string
						       (setq tmp (% (1+ tmp) 50))
						       ?.)))
	      (accept-process-output)))
	(message proc))
      
      (condition-case ()
	  (delete-process proc);; make sure its dead
	(error nil))
      (w3-replace-regexp (regexp-quote (substring cmd 0 -1)) "")
      (w3-replace-regexp
       "Process .*anno.* \\(exit\\|kill\\|finish\\).*" ""))))

(defun w3-is-annotation ()
  "Is this a group annotation?"
  (and (equal w3-current-server w3-group-annotation-server)
       (= (string-to-int w3-current-port) w3-group-annotation-port)))

(defun w3-delete-group-annotation ()
  "Delete this group annotation"
  (interactive)
  (if (not w3-group-annotation-server)
      (error "No group annotation server defined!")
    (if (w3-is-annotation)
	(let ((cmd (format "ANN_DELETE /url=\"%s\";=\n" (w3-view-url t)))
	      (proc nil)
	      (parseit nil)
	      (buff (current-buffer)))
	  (save-excursion
	    (set-buffer (get-buffer-create w3-working-buffer))
	    (erase-buffer)
	    (setq proc (w3-open-stream "*anno*" (get-buffer-create w3-working-buffer)
				       w3-group-annotation-server
				       w3-group-annotation-port))
	    (if (processp proc)
		(process-send-string proc cmd))
	    (while (memq (process-status proc) '(run open))
	      (accept-process-output))
	    (condition-case ()
		(delete-process proc);; make sure its dead
	      (error nil))
	    (w3-replace-regexp (regexp-quote cmd) "")
	    (w3-replace-regexp "Process .*anno.* exit.*" "")
	    (goto-char (point-min))
	    (setq w3-current-type nil
		  w3-current-file "historylist"
		  w3-current-last-buffer buff)
	    (if (and (re-search-forward "[^ \t\n]+" nil t)
		     (not
		      (progn
			(goto-char (point-min))
			(re-search-forward "success!" nil t))))
		(setq parseit t)
	      (setq parseit nil)))
	  (if parseit
	      (w3-sentinel nil nil)
	    (message "Deleted...")))
      (message "This is not an annotation."))))

(defun w3-add-group-annotation ()
  "Add an annotation to the current url"
  (interactive)
  (let ((url (w3-view-url t))
	(buf (get-buffer-create "*Annotation*")))
    (if w3-mutable-windows (pop-to-buffer buf) (switch-to-buffer buf))
    (set-buffer buf)
    (erase-buffer)
    (w3-insert "</PRE>\n\n")
    (html-mode)
    (setq w3-current-annotation url)
    (define-key html-mode-map "\C-c\C-c" 'w3-do-group-annotation)
    (message "Hit C-cC-c to send this annotation.")))	

(defun w3-do-group-annotation ()
  "Finish adding an annotation"
  (interactive)
  (let* ((bufsize (buffer-size))
	 (bufstr  (buffer-string))
	 (cmd (format "ANN_%s /url=\"%s\";title=\"%s\";user=\"%s\";date=%s;length=%d;=%s\n"
		      (if w3-editing-annotation "CHANGE" "SET")
		      w3-current-annotation
		      (read-string "Title: "
				   (format "Annotation by %s (%s@%s)"
					   (user-full-name)
					   (user-real-login-name)
					   (system-name)))
		      (format "%s@%s" (user-real-login-name)
			      (system-name))
		      (current-time-string)
		      bufsize bufstr))
	 (proc nil)
	 (parseit nil))
    (save-excursion
      (not-modified)
      (kill-buffer (current-buffer))
      (set-buffer (get-buffer-create w3-working-buffer))
      (erase-buffer)
      (setq proc (w3-open-stream "*anno*" (get-buffer-create w3-working-buffer)
				 w3-group-annotation-server
				 w3-group-annotation-port))
      (process-send-string proc cmd)
      (while (memq (process-status proc) '(run open))
	(accept-process-output))
      (condition-case ()
	  (delete-process proc);; make sure its dead
	(error nil))
      (w3-replace-regexp (regexp-quote cmd) "")
      (w3-replace-regexp "Process .*anno.* exit.*" "")
      (goto-char (point-min))
      (setq w3-current-type nil
	    w3-current-file "historylist")
      (if (re-search-forward "[^ \t\n]+" nil t)
	  (setq parseit t)
	(setq parseit nil)))
    (if parseit
	(w3-sentinel nil nil)
      (message "Annotation delivered..."))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Private annotation support
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-parse-personal-annotations ()
  "Read in personal annotation file"
  (if (and 
       (file-exists-p (format "%s/LOG" w3-personal-annotation-directory))
       (file-readable-p (format "%s/LOG" w3-personal-annotation-directory)))
      (save-excursion
	(setq w3-personal-annotations nil);; nuke the old list
	(let ((start nil)
	      (end nil)
	      (txt nil)
	      (url nil)
	      (num nil))
	  (set-buffer (get-buffer-create " *panno*"))
	  (erase-buffer)
	  (insert-file-contents
	   (format "%s/LOG" w3-personal-annotation-directory))
	  (goto-char (point-min))
	  (w3-replace-regexp "\\\n+" "\\\n")
	  (goto-char (point-min)) 
	  (kill-line 2);; nuke the header lines
	  (goto-char (point-min))
	  (cond
	   ((eobp) nil)			; Empty LOG file
	   (t
	    (if (/= (char-after (1- (point-max))) ?\n)
		(save-excursion
		  (goto-char (point-max))
		  (w3-insert "\n")))
	    (while (/= (point-max) (point))
	      (setq start (point)
		    end (prog2 (end-of-line) (point) (forward-char 1))
		    txt (buffer-substring start end)
		    url (substring txt 0 (string-match " " txt))
		    num (w3-split
			 (substring txt (1+ (string-match " " txt)) nil)
			 "[ \\\t]"))
	      (while num
		(setq w3-personal-annotations 
		      (cons
		       (list url
			     (list (car (car num))
				   (w3-grok-annotation-format
				    (car (car num)))))
		       w3-personal-annotations)
		      num (cdr num))))))
	  (kill-buffer " *panno*")))))

(defun w3-grok-annotation-format (anno)
  "Grab the title from an annotation"
  (save-excursion
    (set-buffer (get-buffer-create " *annotmp*"))
    (erase-buffer)
    (if (file-exists-p (format "%s/PAN-%s.html"
			       w3-personal-annotation-directory anno))
	(insert-file-contents (format "%s/PAN-%s.html"
				      w3-personal-annotation-directory anno)))
    (goto-char (point-min))
    (prog1
	(if (re-search-forward "<title>\\(.*\\)</title>" nil t)
	    (buffer-substring (match-beginning 1) (match-end 1))
	  "No title")
      (kill-buffer " *annotmp*"))))    

(defun w3-fetch-personal-annotations ()
  "Grab any personal annotations for the current url"
  (let ((url  (w3-view-url t))
	(anno w3-personal-annotations)
	(annolist nil))
    (if (assoc url anno)
	(while anno
	  (if (equal (car (car anno)) url)
	      (setq annolist
		    (cons
		     (format "<A HREF=\"file:%s%s/PAN-%s.html\">%s</A>"
			     (if (= ?/ (string-to-char
					w3-personal-annotation-directory)) ""
			       "/")
			     w3-personal-annotation-directory
			     (car (car (cdr (car anno))))
			     (car (cdr (car (cdr (car anno))))))
		     annolist)))
	  (setq anno (cdr anno))))
    annolist))

(defun w3-is-personal-annotation (url)
  "Is URL a personal annotation?"
  (string-match "file:/.*/PAN-.*\\.html" url))

(defun w3-delete-personal-annotation ()
  "Delete a personal annotation."
  (interactive)
  (if (w3-is-personal-annotation (w3-view-url t))
      (let ((num nil)
	    (annotated-url nil)
	    (anno w3-personal-annotations))
	(string-match "file:/.*/PAN-\\(.*\\)\\.html" (w3-view-url t))
	(setq num (substring (w3-view-url t) (match-beginning 1)
			     (match-end 1)))
	(while anno
	  (if (equal num (car (car (cdr (car anno)))))
	      (setq annotated-url (car (car anno))))
	  (setq anno (cdr anno)))
	(if annotated-url
	    (save-excursion
	      (set-buffer (get-buffer-create " *annotmp*"))
	      (erase-buffer)
	      (insert-file-contents (format "%s/LOG"
					    w3-personal-annotation-directory))
	      (replace-regexp (format "[ \\\t]+\\b%s\\b[ \\\t]*" num) " ")
	      (goto-char (point-min))
	      (delete-matching-lines (format "^%s +$" annotated-url))
	      (let ((make-backup-files nil)
		    (version-control nil)
		    (require-final-newline t))
		(write-region (point-min) (point-max)
			      (format "%s/LOG"
				      w3-personal-annotation-directory)))
	      (kill-buffer " *annotmp*")
	      (setq anno w3-personal-annotations
		    w3-personal-annotations nil)
	      (while anno
		(if (not (string= num (car (car (cdr (car anno))))))
		    (setq w3-personal-annotations
			  (cons (car anno) w3-personal-annotations)))
		(setq anno (cdr anno)))
	      (delete-file (format "%s/PAN-%s.html"
				   w3-personal-annotation-directory num)))
	  (message "Couldn't find url that this is annotating!")))
    (message "This isn't a personal annotation.")))  

(defun w3-add-personal-annotation ()
  "Add an annotation to this document."
  (interactive)
  (let ((url (w3-view-url t))
	(buf (get-buffer-create "*Personal Annotation*")))
    (set-buffer buf)
    (if w3-mutable-windows (pop-to-buffer buf) (switch-to-buffer buf))
    (erase-buffer)
    (w3-insert "</PRE>\n\n")
    (html-mode)
    (setq w3-current-annotation url)
    (define-key html-mode-map "\C-c\C-c" 'w3-do-personal-annotation)
    (message "Hit C-cC-c to send this annotation.")))

(defun w3-find-highest-annotation-number ()
  "Find the highest annotation number in this buffer"
  (let (x)
    (goto-char (point-min))
    (while (re-search-forward "[^ \\\t\\\n]*[ \\\t]\\(.*\\)" nil t)
      (setq x (nconc (mapcar (function (lambda (x) (string-to-int (car x))))
			     (w3-split (buffer-substring (match-beginning 1)
							 (match-end 1))
				       "[ \\\t]")) x)))
    (if (not x) (setq x '(0)))
    (1+ (car (sort x '>)))))

(defun w3-do-personal-annotation ()
  "Finish doing a personal annotation."
  (interactive)
  (if (or (not w3-personal-annotation-directory)
	  (not (file-exists-p w3-personal-annotation-directory))
	  (not (file-directory-p w3-personal-annotation-directory)))
      (error "No personal annotation directory!")
    (let ((url w3-current-annotation)
	  (txt (buffer-string))
	  (title (read-string "Title: "
			      (format "Annotation by %s on %s"
				      (user-real-login-name)
				      (current-time-string))))
	  (fname nil)
	  (num nil))
      (save-excursion
	(not-modified)
	(kill-buffer (current-buffer))
	(set-buffer (get-buffer-create " *annotmp*"))
	(erase-buffer)
	(if (file-exists-p		; Insert current LOG file if
					; it exists.
	     (format "%s/LOG" w3-personal-annotation-directory))
	    (insert-file-contents
	     (format "%s/LOG" w3-personal-annotation-directory))
	  (progn			; Otherwise, create a file
	    (goto-char (point-min))	; that conforms to first
					; annotation format from NCSA
	    (w3-insert "ncsa-mosaic-personal-annotation-log-format-1\n")
	    (w3-insert "Personal\n")))
	(goto-char (point-min))
	(setq num (int-to-string (w3-find-highest-annotation-number))
	      fname (format "%s/PAN-%s.html"
			    w3-personal-annotation-directory num))
	(goto-char (point-min))
	(if (re-search-forward (regexp-quote url) nil t)
	    (progn
	      (end-of-line)
	      (w3-insert " "))
	  (goto-char (point-max))
	  (w3-insert "\n" url " "))
	(w3-insert num)
	(let ((make-backup-files nil)
	      (version-control nil)
	      (require-final-newline t))
	  (write-region (point-min) (point-max)
			(format "%s/LOG" w3-personal-annotation-directory))
	  (erase-buffer)
	  (w3-insert (format "%s\n<title>%s</title>\n<h1>%s</h1>\n"
			  w3-annotation-marker
			  title title))
	  (w3-insert
	   (format "<address>%s (%s@%s)</address>\n<address>%s</address>\n"
		   (user-full-name)
		   (user-real-login-name)
		   (system-name)
		   (current-time-string)))
	  (w3-insert "<HR>\n<pre>" txt)
	  (write-region (point-min) (point-max) fname))
	(setq w3-personal-annotations
	      (cons (list url (list num title)) w3-personal-annotations))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for printing
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-print-this-url (&optional url)
  "Print out the current document (in LaTeX format)"
  (interactive)
  (if (not url) (setq url (w3-view-url t)))
  (let ((format (completing-read
		 "Format: "
		 '(("HTML Source") ("Formatted Text") ("LaTeX'd"))
		 nil t)))
    (save-excursion
      (cond
       ((equal "HTML Source" format)
	(w3-retrieve url)
	(lpr-buffer))
       ((equal "Formatted Text" format)
	(lpr-buffer))
       ((equal "LaTeX'd" format)
	(w3-retrieve url)
	(w3-convert-html-to-latex)
	(save-window-excursion
	  (write-region (point-min) (point-max) "/tmp/w3-tmp.latex" nil 5)
	  (shell-command
	   (format
	    "cd /tmp/ ; latex w3-tmp.latex ; %s w3-tmp.dvi ; rm -f w3-tmp*"
	    w3-print-command))
	  (kill-buffer "*Shell Command Output*")))))))

(defun w3-print-url-under-point ()
  "Print out the url under point (in LaTeX format)"
  (interactive)
  (w3-print-this-url (w3-view-this-url t)))

(defun w3-convert-html-to-latex ()
  "Convert an html document into LaTeX - this is pretty much the same as the
sed scripts from info.cern.ch"
  (interactive)
  (set-buffer w3-working-buffer)
  (if w3-use-html2latex
      (shell-command-on-region (point-min) (point-max)
			       (format "%s %s" w3-html2latex-prog
				       w3-html2latex-args) t)
    (progn
      (goto-char (point-min))
      (w3-replace-regexp "\\\\" "\\\\backslash ")
      (w3-replace-regexp "{" "\\\\{")
      (w3-replace-regexp "}" "\\\\}")
      (goto-char (point-min))
      (w3-insert "\\documentstyle{article}\n\\begin{document}\n")
      (goto-char (point-max))
      (w3-insert "\\end{document}")
      (w3-replace-regexp "<\\(XMP\\|LISTING\\)>" "\\\\begin{verbatim}")
      (w3-replace-regexp "</\\(XMP\\|LISTING\\)>" "\\\\end{verbatim}")
      (w3-replace-regexp "<\\(ISINDEX\\|NEXTID\\)[^>]*>" "")
      (w3-replace-regexp (regexp-quote "$") "\\\\$")
      (w3-replace-regexp (regexp-quote "&gt;") "$>$")
      (w3-replace-regexp (regexp-quote "&lt;") "$<$")
      (w3-replace-regexp (regexp-quote "&amp;") " and ")
      (w3-replace-regexp "%" "\\\\%")
      (w3-replace-regexp "#" "\\\\#")
      (w3-replace-regexp "_" "\\\\_")
      (w3-replace-regexp "~" "\\\\~")
      (w3-replace-regexp "<LI> *" "\\\\item ")
      (w3-replace-regexp (regexp-quote "^") "\\\\^")
      (w3-replace-regexp "<P>" "\\\\par")
      (w3-replace-regexp "<TITLE>\\([^<]*\\)</TITLE>" "\\\\section{\\1}")
      (w3-replace-regexp "<H1>" "\\\\section{")
      (w3-replace-regexp "<H2>" "\\\\subsection{")
      (w3-replace-regexp "<H3>" "\\\\subsubsection{")
      (w3-replace-regexp "<H4>" "\\\\subsubsection{")
      (w3-replace-regexp "<H5>" "\\\\paragraph{")
      (w3-replace-regexp "<H6>" "\\\\subparagraph{")
      (w3-replace-regexp "</H[0-9]*>" "}")
      (w3-replace-regexp "<\\(UL\\|DIR\\|MENU\\)>" "\\\\begin{itemize}")
      (w3-replace-regexp "</\\(UL\\|DIR\\|MENU\\)>" "\\\\end{itemize}")
      (w3-replace-regexp "<OL>" "\\\\begin{enumerate}")
      (w3-replace-regexp "</OL>" "\\\\end{enumerate}")
      (w3-replace-regexp "<DL>" "\\\\begin{description}")
      (w3-replace-regexp "</DL>" "\\\\end{description}")
      (w3-replace-regexp "<DT>\\([^<]*$\\)" "\\\\item[\\1]")
      (w3-replace-regexp "<DD>" "")
      (w3-replace-regexp "<A[ \t\n]+[^>]*>" "")   ;; get rid of anchors
      (w3-replace-regexp "</A>" "")
      (w3-replace-regexp
       "<\\(EM\\|B\\|STRONG\\|DFN\\)>\\([^<]*\\)</\\(EM\\|B\\|STRONG\\|DFN\\)>"
       "{\\\\bf \\2}")
      (w3-replace-regexp
       "<\\(CODE\\|SAMP\\|TT\\|KBD\\|VAR\\)>\\([^<]*\\)</\\(CODE\\|SAMP\\|TT\\|KBD\\|VAR\\)>"
       "{\\\\tt \\2}")
      (w3-replace-regexp
       "<\\(CITE\\|U\\)>\\([^<]*\\)</\\(CITE\\|U\\)>" "{\\\\underline \\2}")
      (w3-replace-regexp
       "<\\(I\\|ADDRESS\\)>\\([^<]*\\)</\\(I\\|ADDRESS\\)>" "{\\\\it \\2}")
      (w3-replace-regexp "<IMG[^>]*>" ""))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions to pass files off to external viewers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-start-viewer (fname cmd &optional view)
  "Start a subprocess, named FNAME, executing CMD
If third arg VIEW is non-nil, show the output in a buffer when
the subprocess exits."
  (if view (save-excursion
	     (set-buffer (get-buffer-create view))
	     (erase-buffer)))
  (let ((proc
	 (start-process fname view (or (getenv "ESHELL")
				       (getenv "SHELL")
				       "/bin/sh") "-c" cmd)))
    proc))

(defun w3-pass-to-viewer ()
  "Pass a w3 buffer to a viewer based on file extension."
  (set-buffer w3-working-buffer)
  (let* ((view w3-current-mime-viewer))
    (if (null view)
	(setq view 'indented-text-mode))
    (cond
     ((symbolp view)
      (if (not (memq view '(w3-prepare-buffer w3-print w3-source)))
	  (progn
	    (rename-buffer (w3-generate-new-buffer-name
			    (file-name-nondirectory w3-current-file)))
	    (set-buffer-modified-p nil)
	    (if w3-mutable-windows
		(pop-to-buffer (file-name-nondirectory w3-current-file))
	      (switch-to-buffer (file-name-nondirectory w3-current-file)))
	    (buffer-enable-undo)
	    (funcall view))
	(funcall view)))
     ((stringp view)
      (let ((fname (w3-generate-unique-filename))
	    (show (cond
		   ((null w3-always-show-output) nil)
		   ((eq w3-always-show-output t) t)
		   (t (funcall w3-confirmation-func
			       "View process output?")))))
	(if (boundp 'MULE)
	    (write-region (point-min) (point-max) fname nil nil *noconv*)
	  (write-region (point-min) (point-max) fname))
	(kill-buffer w3-working-buffer)
	(message "Passing to viewer %s" view)
	(set-process-sentinel
	 (w3-start-viewer fname (format view fname)
			  (if show (w3-generate-new-buffer-name
				    (prin1-to-string (read view)))
			    nil))
	 'w3-viewer-sentinel)))
     ((listp view)
      (set-buffer-modified-p nil)
      (buffer-enable-undo)
      (eval view))
     (t
      (message "Unknown viewer specified: %S" view)
      (switch-to-buffer w3-working-buffer)))))

(defun w3-save-binary-file ()
  (interactive)
  (let ((x (read-file-name "Filename to save as: "
			   (expand-file-name "~/") "")))
    (save-excursion
      (if (boundp 'MULE)
	  (write-region (point-min) (point-max) x nil nil *noconv*)
	(write-region (point-min) (point-max) x))
      (kill-buffer (current-buffer)))))

(defun w3-viewer-sentinel (proc string)
  "Delete any temp files left from a viewer process."
  (let ((fname (process-name proc))
	(buffr (process-buffer proc)))
    (if (and (file-exists-p fname)
	     (file-writable-p fname))
	(delete-file fname))
    (if buffr
	(if w3-mutable-windows
	    (pop-to-buffer buffr)
	  (switch-to-buffer buffr)))))

(defun w3-generate-new-buffer-name (start)
  "Create a new buffer name based on START."
  (let ((x 1)
	name)
    (if (not (get-buffer start))
	start
      (progn
	(setq name (format "%s<%d>" start x))
	(while (get-buffer name)
	  (setq x (1+ x)
		name (format "%s<%d>" start x)))
	name))))

(defun w3-generate-unique-filename ()
  "Generate a unique filename in /tmp"
  (let ((base (format "/tmp/w3-tmp.%d" (user-real-uid)))
	(fname "")
	(x 0))
    (setq fname (format "%s%d" base x))
    (while (file-exists-p fname)
      (setq x (1+ x)
	    fname (format "%s%d" base x)))
    fname))

(defun w3-extension-to-mime (extn)
  "This will convert a file extensions (EXTN) to a mime-type, using
the variable w3-mime-extensions"
  (cdr (assoc (downcase extn) w3-mime-extensions)))


(defvar w3-lazy-message-time 0)

(defun w3-lazy-message-1 (&rest args)
  "Just like `message', but is a no-op if called more than once a second.
Will not do anything if w3-show-status is nil."
  (if (or (null w3-show-status)
	  (= w3-lazy-message-time
	     (setq w3-lazy-message-time (nth 1 (current-time)))))
      nil
    (apply 'message args)))

(defun w3-lazy-message-2 (&rest args)
  "Just like `message', but will not do anything if w3-show-transfer-status
is nil."
  (if w3-show-status
      (apply 'message args)
    nil))

(if (fboundp 'current-time)
    (fset 'w3-lazy-message 'w3-lazy-message-1)
  (fset 'w3-lazy-message 'w3-lazy-message-2))

(defun w3-build-url (protocol)
  "Build a url for PROTOCOL, return it as a string"
  (interactive (list (cdr (assoc (completing-read
				  "Protocol: "
				  w3-acceptable-protocols-alist nil t)
				 w3-acceptable-protocols-alist))))
  (let (user host port file)
    (cond
     ((null protocol) (error "Protocol is unknown to me!"))
     ((string= protocol "news")
      (setq host (read-string "Enter news server name, or blank for default: ")
	    port (read-string "Enter port number, or blank for default: ")
	    file (read-string "Newgroup name or Message-ID: ")))
     ((string= protocol "mailto") (setq file (read-string "E-mail address: ")))
     ((string= protocol "http")
      (setq host (read-string "Enter server name: ")
	    port (read-string "Enter port number, or blank for default: ")
	    file (read-string "Remote file: "))
      (and (string= "" port) (setq port nil))
      (and (string= "" host) (error "Must specify a remote machine!")))
     ((string= protocol "file")
      (if (funcall w3-confirmation-func "Local file?")
	  (setq file (read-file-name "Local File: " nil nil t))
	(setq user (read-string "Login as user (blank=anonymous): ")
	      host (read-string "Remote machine name: "))
	(and (string= user "") (setq user "anonymous"))
	(and (string= host "") (error "Must specify a remote machine!"))
	(setq file (read-file-name "File: " (format "/%s@%s:" user host)
				   nil t)
	      file (substring file (length (format "/%s@%s:" user host))))))
     ((or (string= protocol "telnet")
	  (string= protocol "tn3270"))
      (setq user (read-string "Login as user (blank=none): ")
	    host (read-string "Remote machine name: ")
	    port (read-string "Port number (blank=23): "))
      (and (string= "" port) (setq port nil))
      (and (string= "" user) (setq user nil))
      (and (string= "" host) (error "Must specify a host machine!")))
     ((string= protocol "gopher")
      (setq host (read-string "Enter server name: ")
	    port (read-string "Enter port number, or blank for default: ")
	    file (read-string "Remote file: "))
      (and (string= "" port) (setq port nil))
      (and (string= "" host) (error "Must specify a remote machine!"))))
    (message "%s:%s%s"
	     protocol
	     (if (null host) "" (concat "//" host
					(if (null port) "" (concat ":" port))))
	     (if (= ?/ (string-to-char file)) file (concat "/" file)))))
      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for parsing different types of HREF's
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-grok-http-href (url)
  "Return a list of server, port, file, dest from URL"
  (let ((x w3-current-server)
	(y w3-current-port))
    (if (string-match "http:/\\([^/].*\\)" url)	; Weird URL
	(setq url (format "http://%s:%s/%s"
			  x y (substring url
					 (match-beginning 1)
					 (match-end 1)))))
    (if (string-match "http:\\([^/].*\\)" url)	; Another weird URL
	(setq url (w3-parse-relative-link (w3-match url 1))))
    (string-match "http:+/*\\([^:/]*\\):*\\([^/]*\\)/*\\(/.*\\)" url)
    (let* ((server (w3-match url 1))
	   (port   (w3-match url 2))
	   (file   (w3-match url 3))
	   (dest   (if (string-match "#.+$" file)
		       (prog1
			   (substring file (1+ (match-beginning 0))
				      (match-end 0))
			 (setq file (substring file 0 (match-beginning 0))))
		     nil)))
      (if (and (string= server "")
	       (string= port ""))
	  (progn
	    (string-match "/*\\([^:]+\\):*\\([0-9]*\\)" file)
	    (setq server (w3-match file 1)
		  port (w3-match file 2)
		  file "/")))
      (if (string= port "")
	  (setq port "80"))
      (list server port file dest))))

(defun w3-grok-file-href (url)
  "Return a list of username, server, file, destination out of URL"
  (let (user server file dest)
    (cond
     ((string-match "//" url)		; Remote file
      (string-match "^\\(file\\|ftp\\):/*\\([^/]*\\)/*\\(/.*\\)" url)
      (setq server (w3-match url 2)
	    file (w3-match url 3)
	    user "anonymous"
	    dest (if (string-match "#.+$" file)
		     (prog1
			 (substring file (1+ (match-beginning 0))
				    (match-end 0))
		       (setq file (substring file 0 (match-beginning 0))))
		   nil))
      (if (string-match "@" server)
	  (setq user (substring server 0 (match-beginning 0))
		server (substring server (1+ (match-beginning 0)) nil)))
      (if (equal server "localhost")
	  (setq server nil)))
     (t
      (setq dest (if (string-match "#\\(.+\\)$" url)
		     (prog1
			 (w3-match url 1)
		       (setq url (substring url 0 (match-beginning 0))))
		   nil)
	    file url)
      (if (string-match "file:\\(.*\\)" file)
	  (setq file (w3-match file 1)))))
    (list user server file dest)))

(defun w3-grok-news-href (url)
  "Parse out a news url"
  (string-match "/*\\([^/:]*\\):*\\([0-9]*\\)/*\\(.*\\)" url)
  (let (
	(host (substring url (match-beginning 1) (match-end 1)))
	(port (substring url (match-beginning 2) (match-end 2)))
	(art  (substring url (match-beginning 3) (match-end 3))))
    (if (equal port "") (setq port "119"))
    (if (equal host "") (setq host w3-news-server))
    (if (equal art "") (setq art host
			     host w3-news-server))
    (list host port art)))

(defun w3-grok-gopher-href (url)
  "Return a list of attributes from a gopher url.  List is of the
type: host port selector-string MIME-type extra-info"
  (let (host				; host name
	port				; Port #
	selector			; String to send to gopher host
	type				; MIME type
	extra				; Extra information
	x				; Temporary storage for host/port
	y				; Temporary storage for selector
	)
    (or (string-match "gopher:/*\\([^/]+\\)/*\\(.*\\)" url)
	(error "Can't understand url %s" url))
    (setq x (w3-match url 1)		; The host (and possible port #)
	  y (w3-unhex-string
	     (w3-match url 2)))		; The selector (and possible type)

    ;First take care of the host/port/gopher+ information from the url
    ;A + after the port # (host:70+) specifies a gopher+ link
    ;A ? after the port # (host:70?) specifies a gopher+ ask block
    (if (string-match "^\\([^:]+\\):\\([0-9]+\\)\\([?+]*\\)" x)
	(setq host (w3-match x 1)
	      port (w3-match x 2)
	      extra (w3-match x 3))
      (setq host x
	    port "70"
	    extra nil))
    (cond
     ((equal extra "")  (setq extra nil))
     ((equal extra "?") (setq extra 'ask-block))
     ((equal extra "+") (setq extra 'gopher+)))

    ; Next, get the type/get rid of the Mosaic double-typing. Argh.
    (setq x (string-to-char y)		; Get gopher type
	  selector y			; Get the selector string
	  type (cdr (assoc x w3-gopher-to-mime)))
    
    (list host port (or selector "") type extra)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for the different types of urls
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-http (url)
  "Retrieve URL via http.  If SOURCE is non-nil, then don't parse the buffer."
  (let ((lb (current-buffer))
	(href (w3-grok-http-href url)))
    (w3-clear-tmp-buffer)
    (setq w3-current-type "http"
	  w3-current-last-buffer lb)
    (let* ((server (nth 0 href))
	   (port   (nth 1 href))
	   (file   (nth 2 href))
	   (dest   (nth 3 href))
	   (parsed-mime nil)
	   (target-size 0))
      (if (or (not (w3-member port w3-bad-port-list))
	      (funcall w3-confirmation-func
		       (format
			"Warning!  Trying to connect to port %s - continue? "
			port)))
	  (progn
	    (if (equal port "") (setq port "80"))
	    (if (equal file "") (setq file "/") )
	    (setq w3-current-server server
		  w3-current-port port
		  w3-current-file file
		  w3-find-this-link dest)
	    (w3-lazy-message "Fetching: %s %s %s" server port file)
	    (let ((process
		   (w3-open-stream "WWW" w3-working-buffer server
				   (string-to-int port))))
	      (if (stringp process)
		  (progn
		    (set-buffer w3-working-buffer)
		    (erase-buffer)
		    (setq w3-current-mime-type "text/html"
			  w3-current-mime-viewer (w3-mime-viewer "text/html"))
		    (w3-insert "<TITLE>ERROR</TITLE>\n"
			    "<H1>ERROR - Could not establish connection</H1>"
			    "<P>"
			    "The browser could not establish a connection "
			    (format "to %s:%s.<P>" server port)
			    "The server is either down, or the URL"
			    (format "(%s) is malformed.<P>" (w3-view-url t)))
		    (message process))
		(progn
		  (process-kill-without-query process)
		  (process-send-string process
				       (w3-create-mime-request file ""))
		  (if w3-be-asynchronous
		      (set-process-sentinel process 'w3-sentinel)
		    (save-excursion
		      (set-buffer w3-working-buffer)
		      (while (memq (process-status process) '(run open))
			(if (and w3-show-http2-transfer
				 (not parsed-mime)
				 (w3-is-mime-response))
			    (progn
			      (goto-char (point-min))
			      (if (re-search-forward
				   "^content-length:[ \\\t]*\\([0-9]+\\)"
				   nil t)
				  (setq target-size
					(string-to-int
					 (buffer-substring
					  (match-beginning 1)
					  (match-end 1)))))
			      (setq parsed-mime t)))
			(if (> target-size 0)
			    (w3-lazy-message "Read %d of %d bytes (%d%%)"
					     (point-max) target-size
					     (/ (* (point-max) 100)
						target-size))
			  (if (/= 1 (point-max))
			      (w3-lazy-message "Read %d bytes." (point-max))))
			(accept-process-output))
		      (condition-case ()
			  (delete-process process)
			(error nil))))))))
	(progn
	  (ding)
	  (message "Aborting connection to bad port..."))))))

(defun w3-file (url)
  "Find a link to an ftp site - simple transformation to ange-ftp format"
  (let* ((href (w3-grok-file-href url))
	 (user (nth 0 href))
	 (site (nth 1 href))
	 (file (nth 2 href))
	 (dest (nth 3 href))
	 (obuf (current-buffer)))
    (w3-clear-tmp-buffer)
    (setq w3-current-last-buffer obuf)
    (cond
     (site
      (let ((filename (concat "/" user "@" site ":" file)))
	(cond
	 ((file-directory-p filename)
	  (if (eq w3-directory-format 'hypertext)
	      (progn
		(setq w3-current-type "ftp"
		      w3-find-this-link dest
		      w3-current-last-buffer obuf
		      w3-current-user user
		      w3-current-server site
		      w3-current-file (format
				       "%s%s" file
				       (if (equal "/"
						  (substring file -1 nil))
					   "" "/")))
		(w3-format-directory filename))
	    (progn
	      (if (get-buffer w3-working-buffer)
		  (kill-buffer w3-working-buffer))
	      (find-file filename))))
	 (t
	  (set-buffer (get-buffer-create w3-working-buffer))
	  (setq w3-current-type "ftp")
	  (setq w3-current-server site)
	  (setq w3-current-file file)
	  (insert-file-contents filename nil)))))
     (t
      (setq file (expand-file-name (if (string-match "file:" file)
				       (substring file (match-end 0) nil)
				     file))
	    w3-current-type nil
	    w3-current-last-buffer obuf
	    w3-find-this-link dest
	    w3-current-file file)
      (cond
       ((file-directory-p file)
	(cond
	 ((eq w3-directory-format 'hypertext)
	  (w3-format-directory file)
	  (setq w3-current-file (format
				 "%s%s" file
				 (if (equal "/"
					    (substring file -1 nil))
				     "" "/"))))
	 (t
	  (if (get-buffer w3-working-buffer)
	      (kill-buffer w3-working-buffer))
	  (find-file file))))
       (t (insert-file-contents file)))))))
	  
(defun w3-news-get-header (header)
  "Get header information HEADER out of news article in nntp buffer"
  (save-excursion
    (set-buffer nntp-server-buffer)
    (goto-char (point-min))
    (if (re-search-forward (concat "^" header ": +\\(.*\\)") nil t)
	(buffer-substring (match-beginning 1) (match-end 1))
      "")))

(defun w3-news-get-body ()
  "Get body of article from the nntp buffer"
  (save-excursion
    (set-buffer nntp-server-buffer)
    (goto-char (point-min))
    (re-search-forward "\\\n\\\n")
    (buffer-substring (match-end 0) (point-max))))

(defun w3-format-news ()
  "Format a news buffer in html"
  (let ((from  (w3-news-get-header "From"))
	(subj  (w3-news-get-header "Subject"))
	(org   (w3-news-get-header "Organization"))
	(date  (w3-news-get-header "Date"))
	(body  (w3-news-get-body)))
    (w3-clear-tmp-buffer)
    (setq w3-current-file ""
	  w3-current-type "")
    (w3-insert
     (format "<TITLE>%s</TITLE>\n" subj)
     (format
      "<ADDRESS>%s</ADDRESS><P>\n<H1>%s</H1><P>\n%s %s<P><P><P>\n<PRE>%s</PRE>"
      from subj org date body))))

(defun w3-format-whole-newsgroup (newsgroup header-list)
  (w3-clear-tmp-buffer)
  (setq w3-current-file ""
	w3-current-type "")
  (w3-insert (format "<TITLE>%s</TITLE>\n<H1>%s</H1>\n<DL>\n" newsgroup
		  newsgroup))
  (while header-list
    (w3-insert
     (format "<DT>%s\n<DD><A HREF=\"news:%s\">%s</A>\n"
	     (nntp-header-from (car header-list))
	     (if (string-match "<\\(.*\\)>"
			       (nntp-header-id (car header-list)))
		 (substring (nntp-header-id (car header-list))
			    (match-beginning 1) (match-end 1)))
	     (nntp-header-subject (car header-list))))
    (setq header-list (cdr header-list))))

(defun w3-nntp-stat-newsgroup (newsgroup)
  "Select newsgroup NEWSGROUP and return a list of headers of the
remaining articles."
  (cond
   ((string-match "flee" nntp-version)
    (nntp/command "GROUP" newsgroup)
    (save-excursion
      (set-buffer nntp-server-buffer)
      (while (progn
	       (goto-char (point-min))
	       (not (re-search-forward
		     "[0-9]+[ \\\t]+[0-9]+[ \\\t]+\\([0-9]+\\)[ \\\t]+\\([0-9]+\\)" nil t)))
	(accept-process-output)))
    (let ((stat (save-excursion (set-buffer nntp-server-buffer)
				(buffer-string))))
      (string-match
       "[0-9]+[ \\\t]+[0-9]+[ \\\t]+\\([0-9]+\\)[ \\\t]+\\([0-9]+\\)" stat)
      (nntp-retrieve-headers (w3-make-sequence
			      (string-to-int
			       (substring stat (match-beginning 1)
					  (match-end 1)))
			      (string-to-int
			       (substring stat (match-beginning 2)
					  (match-end 2)))))))
   (t
    (nntp-request-group newsgroup)
    (let ((stat (nntp-status-message)))
      (string-match "[0-9]+[ \\\t]+\\([0-9]+\\)[ \\\t]+\\([0-9]+\\)" stat)
      (nntp-retrieve-headers (w3-make-sequence
			      (string-to-int
			       (substring stat (match-beginning 1)
					  (match-end 1)))
			      (string-to-int
			       (substring stat (match-beginning 2)
					  (match-end 2)))))))))

(defun w3-news (article)
  "Find a news reference"
  (or noninteractive (require 'nntp))
  (let ((buff (current-buffer)))
    (set-buffer (get-buffer-create w3-working-buffer))
    (setq w3-current-last-buffer buff)
    (set-buffer buff))
  (let* ((info (w3-grok-news-href article))
	 (host (nth 0 info))
	 (port (nth 1 info))
	 (article (nth 2 info)))
    (if (not (equal w3-current-nntp-server host))
	(nntp-close-server))
    (or (nntp-server-opened) (nntp-open-server host (string-to-int port)))
    (cond
     ((string-match "@" article)	; Its a specific article
      (if (not (equal ">" (substring article -1 nil)));; get in correct
	  (setq article (format "<%s>" article)));; format
      (if (not (nntp-server-opened))
	  (nntp-open-server host (string-to-int port)))
      (if (nntp-request-article article);; did we get it?
	  (w3-format-news);; yes
	(progn
	  (set-buffer w3-working-buffer)
	  (w3-insert "<TITLE>ERROR</TITLE>\n"
		  "<H1> Error requesting article... </H1>"
		  "The status message returned by the NNTP server was:<BR>"
		  (format "<PRE>%s</PRE><P>" (nntp-status-message))
		  "If you feel this is an error, <A HREF=\""
		  "mailto:wmperry@indiana.edu>send me mail</A>."))))
     ((string= article "")		; List all newsgroups
      (message "Not implemented yet."))
     (t					; Whole newsgroup
      (w3-format-whole-newsgroup article
				 (w3-nntp-stat-newsgroup article))))
    (setq w3-current-file article)))

;;;###autoload
(defun w3-open-local (fname)
  "Find a local file, and interpret it as a hypertext document.
This is part of the emacs World Wide Web browser.  It will prompt for
an existing file or directory, and retrieve it as a hypertext document.
If it is a directory, and w3-directory-format is 'hypertext, then an
HTML directory listing is created on the fly.  Otherwise, dired-mode is
used to visit the buffer."
  (interactive "FLocal file: ")
  (if (not w3-setup-done) (w3-do-setup))
  (w3-fetch (concat "file:" fname)))

(defun w3-format-directory (dir)
  "Format the files in DIR into hypertext"
  (let ((files (directory-files dir)) file
	div attr mod-time size)
    (save-excursion
      (setq div (1- (length files)))
      (set-buffer w3-working-buffer)
      (erase-buffer)
      (w3-insert (format "<TITLE>Index of %s</TITLE>\n" dir))
      (w3-insert (format "<H1> Directory listing of %s</H1>\n<P>" dir))
      (w3-insert "<DL>\n")
      (while files
	(w3-lazy-message "Building directory list... (%d%%)"
		 (/ (* 100 (- div (length files))) div))
	(setq file (if (equal "/" (substring dir -1 nil))
		       (format "%s%s" dir (car files))
		     (format "%s/%s" dir (car files))))
	(setq attr (file-attributes file)
	      mod-time (nth 5 attr)
	      size (nth 7 attr))
	(if (or (equal '(0 0) mod-time) ; Set to null if unknown or
                                        ; untranslateable
		(not (or w3-running-lemacs w3-running-old-lemacs
			 w3-running-FSF19)))
	    (setq mod-time nil)
	  (setq mod-time (current-time-string mod-time)))
	(if (equal size 0) (setq size nil) (setq size (int-to-string size)))
	(cond
	 ((equal "." (car files)) nil)
	 ((equal ".." (car files))
	  (w3-insert
	   (format
	    "<DT> <IMG SRC=\"%sw3-ftp-dir.xbm\" ALT=\"\"><A HREF=\"%s\">%s</A>"
	    w3-icon-directory (car files) "Parent Directory")))
	 ((stringp (nth 0 attr))	; Symbolic link handling
	  (w3-insert
	   (format
	    "<DT> <IMG SRC=\"%sw3-ftp-sym.xbm\" ALT=\"\"><A HREF=\"%s\">%s -&gt; %s</A>"
	    w3-icon-directory (car files) (car files) (nth 0 attr))))
	 ((nth 0 attr)			; Directory handling
	  (w3-insert
	   (format
	    "<DT> <IMG SRC=\"%sw3-ftp-dir.xbm\" ALT=\"\"> <A HREF=\"%s\">%s/</A>"
	    w3-icon-directory (car files) (car files))))
	 (t				; Normal file handlnig
	  (w3-insert
	   (format
	    "<DT> <IMG SRC=\"%sw3-ftp-dat.xbm\" ALT=\"\"><A HREF=\"%s\">%s</A>"
	    w3-icon-directory (car files) (car files)))))
	(cond
	 ((equal "." (car files)) nil)
	 (t 
	  (if (not (or mod-time size)) nil (w3-insert "\n<DD>"))
	  (if mod-time (w3-insert "Last Mod: " mod-time))
	  (if (and mod-time size) (w3-insert ", "))
	  (if size (w3-insert "Size: " size " bytes"))
	  (w3-insert "\n")))
	(setq files (cdr files)))
      (w3-insert "\n</DL>"))))

(defun w3-telnet (url)
  "Open up a telnet connection"
  (string-match "telnet:/*\\(.*@\\)*\\([^/]*\\)/*" url)
  (let* ((server (substring url (match-beginning 2) (match-end 2)))
	 (name (if (match-beginning 1)
		   (substring url (match-beginning 1) (1- (match-end 1)))
		 nil))
	 (title (format "%s%s" (if name (concat name "@") "") server))
	 (thebuf (string-match ":" server))
	 (port (if thebuf
		   (prog1
		       (substring server (1+ thebuf) nil)
		     (setq server (substring server 0 thebuf))) "23")))
    (cond
     (window-system
      (start-process "htmlsub" nil w3-xterm-command
		     "-title" title
		     "-ut" "-e" w3-telnet-prog server port)
      (if name (message "Please log in as %s" name)))
     (w3-use-transparent
      (require 'transparent)
      (if name (message "Please log in as %s" name))
      (sit-for 1)
      (transparent-window (get-buffer-create
			   (format "%s%s:%s" (if name (concat name "@") "")
				   server port))
			  w3-telnet-prog
			  (list server port) nil
			  "Press any key to return to emacs"))
     (t
      (terminal-emulator
       (get-buffer-create (format "%s%s:%s" (if name (concat name "@") "")
				  server port))
       w3-telnet-prog
       (list server port))
      (if name (message "Please log in as %s" name))))))

(defun w3-tn3270 (url)
  "Open up a tn3270 connection"
  (string-match "tn3270:/*\\(.*@\\)*\\([^/]*\\)/*" url)
  (let* ((server (substring url (match-beginning 2) (match-end 2)))
	 (name (if (match-beginning 1)
		   (substring url (match-beginning 1) (1- (match-end 1)))
		 nil))
	 (thebuf (string-match ":" server))
	 (title (format "%s%s" (if name (concat name "@") "") server))
	 (port (if thebuf
		   (prog1
		       (substring server (1+ thebuf) nil)
		     (setq server (substring server 0 thebuf))) "23")))
    (cond
     (window-system
      (start-process "htmlsub" nil w3-xterm-command
		     "-title" title
		     "-ut" "-e" w3-tn3270-emulator server port)
      (if name (message "Please log in as %s" name)))
     (w3-use-transparent
      (require 'transparent)
      (if name (message "Please log in as %s" name))
      (sit-for 1)
      (transparent-window (get-buffer-create
			   (format "%s%s:%s" (if name (concat name "@") "")
				   server port))
			  w3-tn3270-emulator
			  (list server port) nil
			  "Press any key to return to emacs"))
     (t
      (terminal-emulator
       (get-buffer-create (format "%s%s:%s" (if name (concat name "@") "")
				  server port))
       w3-tn3270-emulator
       (list server port))
      (if name (message "Please log in as %s" name))))))

(defun w3-mailto (url)
  "Send mail to someone"
  (string-match "mailto:/*\\(.*\\)" url)
  (let ((to (substring url (match-beginning 1) (match-end 1)))
	(url (w3-view-url t)))
    (cond
     ((and w3-mutable-windows (fboundp w3-mail-other-window-command))
      (funcall w3-mail-other-window-command))
     ((fboundp w3-mail-command)
      (funcall w3-mail-command))
     (w3-mutable-windows (mail-other-window))
     (t (mail)))
    (mail-to)
    (w3-insert (format "%s\nX-URL-From: %s" to url))
    (mail-subject)))

(defun w3-gopher (url)
  "Handle gopher URLs"
  (let ((descr (w3-grok-gopher-href url)))
    (cond
     ((or (not (w3-member (nth 1 descr) w3-bad-port-list))
	  (funcall
	   w3-confirmation-func
	   (format "Warning!  Trying to connect to port %s - continue? "
		   (nth 1 descr))))
      (if w3-use-hypertext-gopher
	  (w3-do-gopher descr)
	(gopher-dispatch-object (vector (if (= 0
					       (string-to-char (nth 2 descr)))
					    ?1
					  (string-to-char (nth 2 descr)))
					(nth 2 descr) (nth 2 descr)
					(nth 0 descr)
					(string-to-int (nth 1 descr)))
				(current-buffer))))
     (t (ding) (message "Aborting connection to bad port...")))))

;;;###autoload
(defun w3-retrieve (url)
  "Retrieve a document over the World Wide Web.
The document should be specified by its fully specified
Uniform Resource Locator.  No parsing is done, just return the
document as the server sent it.  The document is left in the
buffer specified by w3-working-buffer.  w3-working-buffer is killed
immediately before starting the transfer, so that no buffer-local
variables interfere with the retrieval.  HTTP/1.0 redirection will
be honored before this function exits."
  (if (get-buffer w3-working-buffer)
      (kill-buffer w3-working-buffer))
  (string-match "\\([^:]*\\):/*" url)
  (let* ((type (substring url (match-beginning 1) (match-end 1))))
    (cond
     ((equal type "mailto") (w3-mailto url))
     ((equal type "wais") (w3-fetch
			   (format "http://%s:%s/%s"
				   w3-wais-gateway-server
				   w3-wais-gateway-port
				   (substring url (match-end 0) nil))))
     ((equal type "news") (w3-news (substring url (match-end 0) nil)))
     ((equal type "ftp") (w3-file url))
     ((equal type "file") (w3-file url))
     ((equal type "http") (w3-http url))
     ((equal type "telnet") (w3-telnet url))
     ((equal type "gopher") (w3-gopher url))
     ((equal type "tn3270") (w3-tn3270 url))
     (t (progn
	   (set-buffer (get-buffer-create w3-working-buffer))
	   (erase-buffer)
	   (w3-insert "<TITLE> Link Error! </TITLE>\n"
		   "<H1> An error has occurred... </H1>\n"
		   (format "The link type <CODE>%s</CODE>" type)
		   " is unrecognized or unsupported at this time.<P>\n"
		   "If you feel this is an error, please"
		   "<A HREF=\"mailto://wmperry@indiana.edu\">send me mail.</A>"
		   "<P><ADDRESS>William Perry</ADDRESS>"
		   "<ADDRESS>wmperry@indiana.edu</ADDRESS>")
	   (setq w3-current-file "error.html"))))
    (if (boundp 'MULE) (w3-convert-code-for-mule type url))
    (cond
     ((equal type "wais") nil)
     ((and w3-be-asynchronous (equal type "http")) nil)
     ((not (get-buffer w3-working-buffer)) nil)
     ((w3-is-mime-response) (w3-parse-mime-headers))
     ((w3-member w3-current-server w3-bad-server-list) nil)
     (t
      (setq w3-bad-server-list (cons w3-current-server w3-bad-server-list))))))

;;;###autoload
(defun w3-fetch (&optional url)
  "Retrieve a document over the World Wide Web.
The World Wide Web is a global hypertext system started by CERN in
Switzerland in 1991.

The document should be specified by its fully specified
Uniform Resource Locator.  The document will be parsed, printed, or
passed to an external viewer as appropriate.  See the variable
w3-mime-viewers for how to specify a viewer for a file type."
  (interactive (list (read-string "URL: " (if (eq major-mode 'w3-mode)
					      (w3-view-url t) ""))))
  (let ((x (w3-view-url t))
	(y (current-buffer))
	(z (point))
	(bufs (buffer-list))
	(found nil))
    (if (not w3-setup-done) (w3-do-setup))
    (if (string= "file:nil" x) (setq x nil))
    (while (and bufs (not found))
      (save-excursion
	(set-buffer (car bufs))
	(setq found (if (and
			 (not (equal (buffer-name (car bufs))
				     w3-working-buffer))
			 (eq major-mode 'w3-mode)
			 (equal (w3-view-url t) url)) (car bufs) nil)
	      bufs (cdr bufs))))
    (if (or (not found)
	    (cond
	     ((eq w3-reuse-buffers 'no) t)
	     ((eq w3-reuse-buffers 'yes) nil)
	     (t
	      (if w3-reuse-buffers
		  (progn
		    (ding)
		    (message
		     "Warning: Invalid value for variable w3-reuse-buffers: %s"
		     (prin1-to-string w3-reuse-buffers))
		    (sit-for 2)))
	      (not (funcall w3-confirmation-func
			    (format "URL found in buffer %10s, reuse "
				    (buffer-name found)))))))
	(progn
	  (w3-retrieve url)
	  (if (get-buffer w3-working-buffer)
	      (cond
	       ((string-match "^wais:" url) nil)
	       ((and w3-be-asynchronous (string-match "^http:" url)) nil)
	       (t
		(setq w3-current-next-link (list url nil nil))
		(set-buffer w3-working-buffer)
		(setq w3-current-last-link (list x y z))
		(w3-sentinel nil nil)))))
      (switch-to-buffer found))))
    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Miscellaneous functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-use-starting-documents ()
  "Use the list of predefined starting documents from w3-starting-documents"
  (interactive)
  (let ((w3-hotlist w3-starting-documents))
    (w3-use-hotlist)))

(defun w3-show-starting-documents ()
  "Show the list of predefined starting documents from w3-starting-documents"
  (interactive)
  (let ((w3-hotlist w3-starting-documents))
    (w3-show-hotlist)))

(defun w3-insert-formatted-url (p)
  "Insert a formatted url into a buffer.  With prefix arg, insert the url
under point."
  (interactive "P")
  (let (buff str)
    (cond
     (p
      (setq p (w3-view-this-url t))
      (or p (error "No url under point"))
      (setq str (format "<A HREF=\"%s\">%s</A>" p
			(read-string "Link text: "
				     (nth 3 (w3-zone-data
					     (w3-zone-at (point))))))))
     (t
      (setq str (format "<A HREF=\"%s\">%s</A>" (w3-view-url t)
			(read-string "Link text: " (buffer-name))))))
    (setq buff (read-buffer "Insert into buffer: " nil t))
    (if buff
	(save-excursion
	  (set-buffer buff)
	  (w3-insert str))
      (message "Cancelled."))))

(defun w3-length-after-parsing (str)
  "Returns the length of a string after removing all text between
<>, and resolving all HTML entity references"
  (let ((tmp ""))
    (while (string-match "\\([^<]*\\)<[^>]+>" str)
      (setq tmp (concat tmp (w3-match str 1))
	    str (substring str (match-end 0) nil)))
    (setq tmp (concat tmp str))
    (setq tmp (w3-fix-entities-in-string tmp))
    (length tmp)))

(defun w3-first-n-items (l n)
  "Return the first N items from list L"
  (let ((x 0)
	y)
    (if (> n (length l))
	(setq y l)
      (while (< x n)
	(setq y (nconc y (list (nth x l)))
	      x (1+ x))))
    y))
	       
(defun w3-breakup-menu (menu-desc max-len)
  (if (> (length menu-desc) max-len)
      (cons (cons "More..." (w3-first-n-items menu-desc max-len))
	    (w3-breakup-menu (nthcdr max-len menu-desc) max-len))
    menu-desc))

(defun w3-get-url-at-point (&optional pt)
  "Get the URL closest to point, but don't change your
position. Has a preference for looking backward when not
directly on a symbol."
  ;; Not at all perfect - point must be right in the name.
  (save-excursion
    (if pt (goto-char pt))
    (let ((filename-chars ".a-zA-Z0-9---_/:~") start)
      (save-excursion
	;; first see if you're just past a filename
	(if (not (eobp))
	    (if (looking-at "[] \t\n[{}()]") ; whitespace or some parens
		(progn
		  (skip-chars-backward " \n\t\r({[]})")
		  (if (not (bobp))
		      (backward-char 1)))))
	(if (string-match (concat "[" filename-chars "]")
			  (char-to-string (following-char)))
	    (progn
	      (skip-chars-backward filename-chars)
	      (setq start (point))
	      (skip-chars-forward filename-chars))
	  (message "No URL found around point!")
	  (setq start (point)))
	(buffer-substring start (point))))))

(defun w3-follow-url-at-point (&optional pt)
  "Follow the URL under PT, defaults to link under (point)"
  (interactive "d")
  (w3-fetch (w3-get-url-at-point pt)))			     

;;;###autoload
(defun w3-batch-fetch ()
  "Fetch all the URLs on the command line and save them to files in
the current directory.  The first argument after the -f w3-batch-fetch
on the command line should be a string specifying how to save the
information retrieved.  If it is \"html\", then the page will be
unformatted when it is written to disk.  If it is \"text\", then the
page will be formatted before it is written to disk.  If it is
\"binary\" it will not mess with the file extensions, and just save
the data in raw binary format.  If none of those, the default is
\"text\", and the first argument is treated as a normal URL."  
  (if (not noninteractive)
      (error "`w3-batch-fetch' is to be used only with -batch"))
  (let ((fname "")
        (curname "")
	(x 0)
	(args (symbol-value 'command-line-args-left))
	(w3-strict-width 80)
	(w3-delimit-emphasis nil)
	(w3-delimit-links nil)
	(retrieval-function 'w3-fetch)
	(file-format "text")
	(header "")
	(file-extn ".txt"))
    (setq file-format (downcase (car args)))
    (cond
     ((string= file-format "html")
      (message "Saving all text as raw HTML...")
      (setq retrieval-function 'w3-retrieve
	    file-extn ".html"
	    header "<BASE HREF=\"%s\">"
	    args (cdr args)))
     ((string= file-format "binary")
      (message "Saving as raw binary...")
      (setq retrieval-function 'w3-retrieve
	    file-extn ""
	    args (cdr args)))
     ((string= file-format "text")
      (setq header "Text from: %s\n---------------\n")
      (message "Saving all text as formatted...")
      (setq args (cdr args)))
     (t
      (setq header "Text from: %s\n---------------\n")
      (message "Going with default, saving all text as formatted...")))
    (while args
      (funcall retrieval-function (car args))
      (goto-char (point-min))
      (if buffer-read-only (toggle-read-only))
      (insert (format header (car args)))
      (if (string= file-extn "") nil
	(setq fname (w3-file-extension (w3-basepath w3-current-file t) t)))
      (if (string= (w3-strip-leading-spaces fname) "")
	  (setq fname "root"))
      (setq curname fname)
      (while (file-exists-p (concat curname file-extn))
	(setq curname (concat fname x)
	      x (1+ x)))
      (setq fname (concat curname file-extn))
      (write-region (point-min) (point-max) fname)
      (setq args (cdr args)))))

(defun w3-eat-trailing-space (x)
  "Remove spaces/tabs at the end of a string"
  (let ((y (1- (length x)))
	(skip-chars (list ?  ?\t ?\n)))
    (while (and (>= y 0) (memq (aref x y) skip-chars))
      (setq y (1- y)))
    (substring x 0 (1+ y))))

(defun w3-strip-leading-spaces (x)
  "Remove spaces at the front of a string"
  (let ((y (1- (length x)))
	(z 0)
	(skip-chars (list ?  ?\t ?\n)))
    (while (and (<= z y) (memq (aref x z) skip-chars))
      (setq z (1+ z)))
    (substring x z nil)))

(defun w3-reload-all-files ()
  "Reload all w3 files"
  (interactive)
  (setq w3-setup-done nil
	w3-hotlist nil
	w3-mime-accept-string nil
	w3-style-regexp nil)
  (let ((x '(w3 w3-mule w3-emacs w3-emacs19 w3-epoch w3-lucid
		w3-new-lucid w3-next)))
    (while x
      (setq features (delq (car x) features)
	    x (cdr x)))
    (require 'w3))
  (w3-do-setup))

(defun w3-document-source ()
  "View this documents source"
  (interactive)
  (let ((w3-source t)
	(url (w3-view-url t)))
    (if url
	(progn (w3-retrieve (w3-view-url t))
	       (w3-sentinel nil nil))
      (message "Automatically generated hypertext - no source available."))))

(defun w3-mail-current-document ()
  "Mail the current-document to someone"
  (interactive)
  (let* ((format (completing-read
		  "Format: "
		  '(("HTML Source") ("Formatted Text") ("LaTeX Source"))
		  nil t))
	 (url (w3-view-url t))
	 (str 
	  (save-excursion
	    (cond
	     ((equal "HTML Source" format) (w3-retrieve url))
	     ((equal "Formatted Text" format) nil)
	     ((equal "LaTeX Source" format)
	      (w3-retrieve url)
	      (w3-convert-html-to-latex)))
	    (buffer-string))))
    (cond
     ((and w3-mutable-windows (fboundp w3-mail-other-window-command))
      (funcall w3-mail-other-window-command))
     ((fboundp w3-mail-command)
      (funcall w3-mail-command))
     (w3-mutable-windows (mail-other-window))
     (t (mail)))
    (mail-subject)
    (w3-insert (format "%s from URL %s" format url))
    (re-search-forward mail-header-separator nil)
    (forward-char 1)
    (while (< (current-column) 79) (w3-insert "-"))
    (w3-insert "\n" (if (equal "HTML Source" format)
			(format "<BASE HREF=\"%s\">" url) "") str "\n")
    (while (< (current-column) 79) (w3-insert "-"))
    (mail-to)))

(defun w3-parse-relative-link (url &optional method)
  "Try to resolve a link like \"library/io.html\""
  (let ((resolved (cond ((equal w3-current-type "http")
			 (concat "http://" w3-current-server ":" 
				 w3-current-port))
			((equal w3-current-type "gopher")
			 (concat "gopher://" w3-current-server ":"
				 w3-current-port "/"))
			((equal w3-current-type "ftp")
			 (concat "file://" w3-current-server "/"))
			(t "file:"))))
    (if (= ?# (string-to-char url))
	(setq resolved url)
      (progn
	(setq url (w3-remove-relative-links url))
	(cond
	 ((equal (string-to-char url) 47)
	  (setq resolved (concat resolved url)))
	 (t (setq resolved (concat resolved
				   (w3-basepath w3-current-file) "/" url))))))
    resolved))

(defun w3-internal-use-history (hist-item)
  "Go to the link in the history"
  (let ((url (nth 0 hist-item))
	(buf (nth 1 hist-item))
	(pnt (nth 2 hist-item)))
    (cond
     ((null buf)			; Find a buffer with same url
      (let ((x (buffer-list))
	    (found nil))
	(while (and x (not found))
	  (save-excursion
	    (set-buffer (car x))
	    (setq found (string= (w3-view-url t) url))
	    (if (not found) (setq x (cdr x)))))
	(cond
	 (found
	  (switch-to-buffer (car x))
	  (if (number-or-marker-p pnt) (goto-char pnt)))
	 (t
	  (w3-fetch url)))))
     ((buffer-name buf)			; Reuse the old buffer if possible
      (switch-to-buffer buf)
      (if (number-or-marker-p pnt) (goto-char pnt))
      (if (and url (= ?# (string-to-char url)))	; Destination link
	  (progn
	    (goto-char (point-min))
	    (w3-find-specific-link (substring url 1 nil)))))
     (url (w3-maybe-relative url))		; Get the link
     (t (message "Couldn't understand whats in the history.")))))

(defun w3-forward-in-history ()
  "Go tot the next link in the history"
  (interactive)
  (cond
   ((null w3-current-next-link) (message "No previous history item."))
   (t (w3-internal-use-history w3-current-next-link))))

(defun w3-backward-in-history ()
  "Go to the previous link in the history"
  (interactive)
  (cond
   ((null w3-current-last-link) (message "No previous history item."))
   (t (w3-internal-use-history w3-current-last-link))))

(defun w3-relative-link (url)
  (if (equal "#" (substring url 0 1))
      (progn
	(setq w3-current-last-link (list "" (current-buffer) (point)))
	(setq w3-current-next-link (list url nil nil))
	(goto-char (point-min))
	(w3-find-specific-link (substring url 1 nil)))
    (w3-fetch (w3-parse-relative-link url))))
  
(defun w3-maybe-eval ()
  "Maybe evaluate a buffer of emacs lisp code"
  (if (funcall w3-confirmation-func "This is emacs-lisp code, evaluate it?")
      (eval-current-buffer)
    (emacs-lisp-mode)))

(defun w3-build-continuation ()
  "Build a series of functions to be run on this file"
  (save-excursion
    (set-buffer w3-working-buffer)
    (let ((cont w3-default-continuation)
	  (extn (w3-file-extension w3-current-file)))
      (if (assoc extn w3-uncompressor-alist)
	  (setq extn (w3-file-extension
		      (substring w3-current-file 0 (- (length extn))))))
      (if w3-source (setq w3-current-mime-viewer
			  (w3-mime-viewer "www/source")))
      (if (not w3-current-mime-viewer)
	  (setq w3-current-mime-viewer
		(w3-mime-viewer (w3-extension-to-mime extn))))
      (if w3-current-mime-viewer
	  (setq cont (append cont '(w3-pass-to-viewer)))
	(setq cont (append cont (list w3-default-action))))
      cont)))

(defun w3-link-info (&optional url no-show)
  "Get more information on a link."
  (interactive)
  (if (not url) (setq url (w3-view-this-url t)))
  (if (not url) (setq url (w3-view-url t)))
  (let (ctype result serv fil type)
    (string-match "^\\([^:]+\\):/*\\([^/]+\\)/*\\(.*\\)" url)
    (setq type (w3-match url 1)
	  serv (w3-match url 2)
	  fil (w3-match url 3))
    (cond
     ((string= type "news")
      (setq ctype (if (string-match "@" serv) "news article"
		    (concat "newsgroup (" serv ")"))
	    type "NNTP site at %s"
	    serv w3-news-server))
     ((and (string= type "file")
	   (not (string-match "^file://" url)))
      (setq serv "local file system."
	    ctype (concat (cdr (assoc (w3-extension-to-mime
				       (w3-file-extension fil))
				      w3-mime-descriptions)) " file")
	    type "%s"))
     ((string= "gopher" type)
      (setq type (concat type " site at %s."))
      (if (= ?0 (string-to-char fil))
	  (setq ctype "gopher document")
	(setq ctype "gopher directory")))
     (t
      (if (string= type "file") (setq type "ftp"))
      (setq ctype (concat (cdr
			   (assoc (w3-extension-to-mime
				   (w3-file-extension fil))
				  w3-mime-descriptions)) " file")
	    type (concat type " site at %s."))))
    (setq result
	  (format "A %s on the %s" ctype (format type serv)))
    (if (not no-show) (message result) result)))

(defun w3-use-links ()
  "Select one of the &lt;LINK&gt; tags from this document and fetch it."
  (interactive)
  (if (not w3-current-links)
      (error "No <LINK> tags in this document.")
    (w3-fetch (cdr (assoc (completing-read "Link with REV="
					   w3-current-links nil t)
			  w3-current-links)))))

(defun w3-fix-fake-urls (st nd)
  "Fix fake urls into real \"<A HREF=...></A>\" type links"
  (interactive "r")
  (save-restriction
    (narrow-to-region st nd)
    (goto-char (point-min))
    (w3-replace-regexp
     (concat w3-nonrelative-link "\\(:[^ >]+\\)") " \\1\\2")
    (w3-replace-regexp
     (concat "[^=\"]+" w3-nonrelative-link "\\(:[^ >\\\n]+\\)")
     "<A HREF=\"\\1\\2\"> \\1\\2 </A>")))

(defun w3-update-source-files (w3-source-directory)
  "Retrieve the latest version of the w3 browser and copy it to`
w3-source-directory"
  (interactive "DWhere to store w3.tar.z? ")
  (let ((newfile (format "%s%sw3.tar.z" w3-source-directory
			 (if (equal "/" (substring w3-source-directory -1 nil))
			     "" "/")))
	(oldfile "/anonymous@cs.indiana.edu:/pub/elisp/w3/w3.tar.z"))
    (if (file-newer-than-file-p oldfile newfile)
	(copy-file oldfile newfile 3))
    (shell-command (format "gunzip %s" newfile))
    (shell-command (format "cd %s ; tar xf %s"
			   w3-source-directory
			   newfile))))

(defun w3-hexify-string (str)
  "Escape characters in a string"
  (let ((str2 "")
	(char 0))
    (if (boundp 'MULE)
	(setq str (code-convert-string 
		   str *internal* w3-mule-retrieval-coding-system)))
    (while (not (equal str ""))
      (setq char (string-to-char str)
	    str (substring str 1 nil)
	    str2 (format "%s%s" str2
			 (if (or (> char ?z)
				 (< char ?/)
				 (and (< char ?a)
				      (> char ?Z))
				 (and (< char ?A)
				      (> char ?9)))
			     (if (< char 16)
				 (upcase (format "%%0%x" char))
			       (upcase (format "%%%x" char)))
			   (format "%c" char)))))
    str2))

(defun w3-find-this-file ()
  "Do a find-file on the currently viewed html document if it is a file: or
ftp: reference"
  (interactive)
  (cond
   ((and (null w3-current-type)
	 (eq major-mode 'w3-mode))
    (if w3-mutable-windows
	(find-file-other-window w3-current-file)
      (find-file w3-current-file)))
   ((equal w3-current-type "ftp")
    (if w3-mutable-windows
	(find-file-other-window
	 (format "/anonymous@%s:%s" w3-current-server w3-current-file))
      (find-file 
       (format "/anonymous@%s:%s" w3-current-server w3-current-file))))
   (t (message "Sorry, I can't get that file so you can alter it."))))

(defun w3-delete-from-alist (x alist)
  "Remove X from ALIST, return new alist"
  (if (eq (assoc x alist) (car alist)) (cdr alist)
    (delq (assoc x alist) alist)))

(defun w3-count-occurences (regexp)
  "Count # of matches for REGEXP after point. Modified from the how-many
function of emacs19"
  (let ((n 0) opoint)
    (save-excursion
      (while (and (not (eobp))
		  (progn (setq opoint (point))
			 (re-search-forward regexp nil t)))
	(if (= opoint (point))
	    (forward-char 1)
	  (setq n (1+ n)))))
    n))      

(defun w3-insert-this-url (pref-arg)
  "Insert the current url in another buffer, with prefix ARG, insert URL under point"
  (interactive "P")
  (let ((thebuf (get-buffer (read-buffer "Insert into buffer: ")))
	(oldbuf (current-buffer))
	(url (if pref-arg (w3-view-this-url t) (w3-view-url t))))
    (if (not (equal "Not on a link!" url))
	(progn
	  (set-buffer thebuf)
	  (w3-insert url)
	  (set-buffer oldbuf)))))      

(defun w3-show-hotlist ()
  "View the hotlist in hypertext form"
  (interactive)
  (if (not w3-setup-done) (w3-do-setup))
  (if (not w3-hotlist)
      (error "Sorry, no hotlist is in memory.")
    (let ((tmp w3-hotlist)
	  (buff (current-buffer)))
      (w3-clear-tmp-buffer)
      (w3-insert "<TITLE> Hotlist </TITLE>\n")
      (w3-insert (format "<H1> Hotlist From %s </H1>\n" w3-hotlist-file))
      (w3-insert "<OL>\n")
      (while tmp
	(w3-insert (format "<LI> <A HREF=\"%s\">%s</A>\n"
			(car (cdr (car tmp)))
			(car (car tmp))))
	(setq tmp (cdr tmp)))
      (w3-insert "\n</OL>\n")
      (setq w3-continuation (append w3-continuation '(w3-prepare-buffer))
	    w3-current-type "www"
	    w3-current-file "hotlist.html"
	    w3-current-mime-viewer 'w3-prepare-buffer
	    w3-current-last-buffer buff)
      (w3-prepare-buffer))))

(defun w3-lookup-style (type)
  "Return the physical style of logical style <TYPE>"
  (let ((x (cdr (assoc type w3-style-assoc))))
    (if (symbolp x) (symbol-value x) x)))	

(defun w3-make-sequence (start end)
  "Make a sequence (list) of numbers from START to END"
  (let ((sqnc '()))
    (while (<= start end)
      (setq sqnc (cons end sqnc)
	    end (1- end)))
    sqnc))

(defun w3-maybe-relative (url)
  "Take a url and either fetch it, or resolve relative refs, then fetch it"
  (cond
   ((not
     (string-match w3-nonrelative-link url))
    (w3-relative-link url))
   (t (w3-fetch url))))
 
(defun w3-in-assoc (elt list)
  "Check to see if ELT matches any of the regexps in the car elements of LIST"
  (let (rslt)
    (while (and list (not rslt))
      (and (car (car list))
	   (string-match (car (car list)) elt)
	   (setq rslt (car list)))
      (setq list (cdr list)))
    rslt))

(defun w3-member (elt list)
  "Function defined so that we are sure member will always use equal, like
its supposed to.  This was pulled from Jamie Zawinskies byte compiler "
  (while (and list (not (equal elt (car list))))
    (setq list (cdr list)))
  list)

(defun w3-goto-last-buffer ()
  "Go to last WWW buffer visited"
  (interactive)
  (if w3-current-last-buffer
      (if w3-mutable-windows
	  (pop-to-buffer w3-current-last-buffer)
	(switch-to-buffer w3-current-last-buffer))
    (message "No previous buffer found.")))

(defun w3-file-extension (fname &optional x)
  "Return the filename extension of FNAME.  If optional variable X is t,
then return the basename of the file with the extension stripped off."
  (if (string-match "\\.[^\\.]+$" fname)
      (if x (substring fname 0 (match-beginning 0))
	(substring fname (match-beginning 0) nil))
    ""))

(defun w3-toggle-telnet ()
  "Toggle telnetting status"
  (interactive)
  (setq w3-use-telnet (not w3-use-telnet)))

(defun w3-basepath (file &optional x)
  "Return the base pathname of FILE, or the actual filename if X is true"
  (if (string-match "\\(.*\\)/\\([^/]*\\)" file)
      (if (not x) (substring file (match-beginning 1) (match-end 1))
	(substring file (match-beginning 2) (match-end 2)))
    file))

(defun w3-replace-regexp (regexp to-string)
  "Quiet replace-regexp."
  (goto-char (point-min))
  (while (re-search-forward regexp nil t)
    (replace-match to-string t nil)))

(defun w3-find-highest-link-num ()
  "Find highest NAMEd link, so we can number on from there."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let ((lnkctr 0))
    (while (re-search-forward "<A[ \t\n]+NAME=" (point-max) t)
      (let* ((start (match-end 0))
	     (end (save-excursion
		    (re-search-forward "[ \t\n>]" (point-max) t)
		    (match-beginning 0)))
	     (subst (buffer-substring start end)))
	(and subst
	     (> (string-to-int subst) lnkctr)
	     (setq lnkctr (string-to-int subst)))))
    lnkctr))

(defun w3-clear-tmp-buffer ()
  "Clear the temporary W3 buffer"
  (set-buffer (get-buffer-create w3-working-buffer))
  (if buffer-read-only (toggle-read-only))
  (erase-buffer))  

;;;###autoload
(defun w3-preview-this-buffer ()
  "See what this buffer will look like when its formatted as HTML.
HTML is the HyperText Markup Language used by the World Wide Web to
specify formatting for text.  More information on HTML can be found at
info.cern.ch:/pub/www/doc."
  (interactive)
  (let* ((thebuf (current-buffer)))
    (w3-clear-tmp-buffer)
    (insert-buffer thebuf)
    (setq w3-current-file (buffer-file-name thebuf)
	  w3-current-type nil)
    (w3-sentinel nil nil)))

(defun w3-open-stream (name buffer host service)
  "Open a stream to a host, using either telnet (if w3-use-telnet is t,
or a raw stream (if w3-use-telnet is nil)"
  (if w3-use-telnet
      (let ((proc
	     (start-process name buffer w3-telnet-prog host
			    (int-to-string service)))
	    (tmp nil))
	(if (boundp 'MULE) (w3-inhibit-code-conversion proc buffer))
	(save-excursion
	  (set-buffer buffer)
	  (setq tmp (point))
	  (while (not (progn
			(goto-char (point-min))
			(re-search-forward w3-telnet-ready-regexp nil t)))
	    (accept-process-output))
	  (delete-region tmp (point))
	  (goto-char (point-min))
	  (if (re-search-forward "connect:" nil t)
	      (progn
		(condition-case ()
		    (delete-process proc)
		  (error nil))
		(w3-replace-regexp ".*connect:.*" "")
		nil)
	    proc)))
    (let ((tries 1)
	  (x t)
	  (z nil))
      (while (and x
		  (<= tries w3-connection-retries))
	(condition-case ()
	    (setq z (open-network-stream name buffer host service)
		  x nil)
	  (error (progn
		   (message "Failed on connection try #%d" tries)
		   (sit-for 1)
		   (setq x t
			 tries (1+ tries))))))
      (if (> tries w3-connection-retries)
	  (format "Could not establish connection to %s:%d" host service)
	(if (boundp 'MULE) (w3-inhibit-code-conversion z buffer))
	z))))

(defun w3-unhex (x)
  (if (> x ?9)
      (if (>= x ?a)
	  (+ 10 (- x ?a))
	(+ 10 (- x ?A)))
    (- x ?0)))

(defun w3-unhex-string (str)
  "Remove %XXX embedded spaces, etc in a url"
  (let ((tmp ""))
    (while (string-match "%[0-9a-f][0-9a-f]" str)
      (let* ((start (match-beginning 0))
	     (ch1 (w3-unhex (elt str (+ start 1))))
	     (code (+ (* 16 ch1)
		      (w3-unhex (elt str (+ start 2))))))
	(setq tmp
	      (concat 
	       tmp (substring str 0 start)
	       (char-to-string code)))
	(setq str (substring str (match-end 0)))))
    (setq tmp (concat tmp str))
    tmp))

(defun w3-rassoc (key list)
  "An 'rassoc' function - don't want to bother with loading cl just
for this function"
  (let ((found nil))
    (while (and list (not found))
      (if (equal (cdr (car list)) key) (setq found (car list)))
      (setq list (cdr list)))
    found))

(defun w3-insert-entities-in-string (str)
  "Remove special characters in STR and replace them with HTML[+] entities"
  (let ((tmp "")
	(regexp "\\(&\\|<\\|>\\)")
	(temp '(("&lt;" . "<") ("&gt;" . ">") ("&amp;" . "&")))
	(x nil))
    (while (string-match regexp str)
      (setq x (car (w3-rassoc (w3-match str 1) temp))
	    tmp (format "%s%s%s" tmp (substring str 0 (match-beginning 0)) x)
	    str (substring str (match-end 0))))
    (concat tmp str)))

(defun w3-fix-entities-in-string (str)
  "Remove &xxx; entities in string STR"
  (let ((tmp "")
	(regexp (concat "\\(" (mapconcat (function (lambda (x) (car x)))
					 w3-html-entities "\\|") "\\)"))
	(x nil))
    (while (string-match regexp str)
      (setq x (cdr (assoc (w3-match str 1) w3-html-entities))
	    tmp (format "%s%s%s" tmp (substring str 0 (match-beginning 0)) x)
	    str (substring str (match-end 0))))
    (setq tmp (concat tmp str))
    tmp))    
       
(defun w3-edit-source ()
  "Edit the html document just retrieved"
  (set-buffer w3-working-buffer)
  (let ((ttl (format "Editing %s Annotation: %s"
		     (cond
		      ((eq w3-editing-annotation 'group) "Group")
		      ((eq w3-editing-annotation 'personal) "Personal")
		      (t "Unknown"))
		     (w3-basepath w3-current-file t)))
	(str (buffer-string)))
    (set-buffer (get-buffer-create ttl))
    (w3-insert str)
    (kill-buffer w3-working-buffer)))

(defun w3-clean-text ()
  "Clean up a buffer after telnet (trash at beginning, connection closed)"
  (set-buffer w3-working-buffer)
  (if (and w3-use-telnet (equal w3-current-type "http"))
      (progn
	(goto-char (point-min))
	(kill-line w3-telnet-header-length)
	(w3-replace-regexp "Connection.*" ""))))

(defun w3-source ()
  "Show the source of a file"
  (let ((tmp (buffer-name (generate-new-buffer "Document Source"))))
    (set-buffer w3-working-buffer)
    (kill-buffer tmp)
    (rename-buffer tmp)
    (set-buffer-modified-p nil)
    (buffer-enable-undo)
    (if w3-mutable-windows (pop-to-buffer tmp) (switch-to-buffer tmp))))

(defun w3-uncompress ()
  "Uncompress a file"
  (set-buffer w3-working-buffer)
  (let ((extn (w3-file-extension w3-current-file)))
    (if (assoc extn w3-uncompressor-alist)
	(progn
	  (message "Uncompressing")
	  (shell-command-on-region (point-min) (point-max)
				   (cdr (assoc extn w3-uncompressor-alist))
				   t)))))

(defun w3-sentinel (proc string)
  (set-buffer w3-working-buffer)
  (if w3-be-asynchronous
      (progn
	(if (boundp 'MULE) (w3-convert-code-for-mule w3-current-type
						     (w3-view-url t)))
	(cond
	 ((not (get-buffer w3-working-buffer)) nil)
	 ((w3-is-mime-response) (w3-parse-mime-headers))
	 ((w3-member w3-current-server w3-bad-server-list) nil)
	 (t
	  (setq w3-bad-server-list
		(cons w3-current-server w3-bad-server-list))))))
  (w3-replace-regexp "Process WWW.*" "")
  (let ((x (w3-build-continuation)))
    (while x
      (funcall (car x))
      (setq x (cdr x)))))

(defun w3-show-history-list ()
  "Format the w3-history-list prettily and show it to the user"
  (interactive)
  (if (not w3-history-list)
      (error "Sorry, no history list available now!")
    (let ((urls w3-history-list)
	  (buff (current-buffer)))
      (w3-clear-tmp-buffer)
      (w3-insert "<TITLE> History List For This Session of W3</TITLE>\n")
      (w3-insert "<H1>History List For This Session of W3</H1>\n")
      (w3-insert "<H2>(Oldest items last in list)</H2>\n")
      (w3-insert "<OL>\n")
      (while urls
	(w3-insert (format "<LI> <A HREF=\"%s\">%s</A>\n"
			(car (car urls)) (cdr (car urls))))
	(setq urls (cdr urls)))
      (w3-insert "\n</OL>\n")
      (setq w3-continuation (append w3-continuation '(w3-prepare-buffer))
	    w3-current-type "www"
	    w3-current-file "history.html"
	    w3-current-mime-viewer 'w3-prepare-buffer
	    w3-current-last-buffer buff)
      (w3-prepare-buffer))))

(defun w3-save-as ()
  "Save a document to the local disk"
  (interactive)
  (let ((format (completing-read
		 "Format: "
		 '(("HTML Source") ("Formatted Text") ("LaTeX Source"))
		 nil t))
	(fname (expand-file-name
		(read-file-name "File name: " default-directory)))
	(url (w3-view-url t)))
    (cond
     ((equal "HTML Source" format)
      (w3-retrieve url)			; Get the document
      (goto-char (point-min))
      (insert (format "<BASE HREF=\"%s\">\n" url)))
     ((equal "Formatted Text" format)
      nil)				; Do nothing - we have the text already
     ((equal "LaTeX Source" format)
      (w3-retrieve url)			; Get the file
      (w3-convert-html-to-latex)))	; Convert to LaTeX
    (write-region (point-min) (point-max) fname)))

(defun w3-upcase-region (st nd)
  "Uppercase a region of text, ignoring text within < and >"
  (save-excursion
    (goto-char st)
    (while (re-search-forward "\\(<[^>]+>\\)\\|\\(&[^;]+;\\)" nd t)
      (upcase-region st (match-beginning 0))
      (setq st (match-end 0)))
    (upcase-region st nd)))
      


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions to parse out <A> tags and replace it with a hyperlink zone
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-match (s x)
  "Return regexp match x in s."
  (substring s (match-beginning x) (match-end x)))

(defun w3-build-links-list ()
  "Build links out of url specs in the temporary buffer.  This function
looks in the buffer pointed to by w3-working-buffer.  The links will be
fully usable by w3-follow-link, etc."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (if buffer-read-only (toggle-read-only))
  (let* ((next-id (int-to-string (1+ (w3-find-highest-link-num))))
	 (cur-id "")
	 (cur-txt "")
	 (cur-urn "")
	 (cur-rel "")
	 (cur-href "")
	 (cur-rev "")
	 (cur-title "")
	 (cur-meth "")
	 )
    (goto-char (point-min))
    (while (re-search-forward w3-link-begin-regexp nil t)
      (let* ((start (match-beginning 0))
	     (cur-url (prog1
			  (buffer-substring (match-beginning 0)
					    (match-end 0))
			(replace-match "")))
	     (end   (if (re-search-forward w3-link-end-regexp nil t)
			(prog1
			    (match-beginning 0)
			  (replace-match ""))
		      (progn
			(end-of-line)
			(point)))))
	(save-excursion
	  (goto-char start)
	  (skip-chars-forward " \t")
	  (setq start (point))
	  (goto-char end)
	  (skip-chars-backward " \t")
	  (setq end (point)))
	(if (< end start) (setq cur-txt end
				end start
				start cur-txt))
	(if (string-match "NAME *= *\"*\\([^\" >]*\\)\"*" cur-url)
	    (setq cur-id (w3-match cur-url 1))
	  (setq cur-id next-id
		next-id (int-to-string (1+ (string-to-int next-id)))))
	(if (string-match "HREF *= *\"*[ \\\t]*\\([^>\" ]*\\)\"*" cur-url)
	    (setq cur-href (w3-match cur-url 1))
	  (setq cur-href nil))
	(if (string-match "[^H]REF *= *\"*\\([^\" ]*\\)\"*" cur-url)
	    (setq cur-rel (w3-match cur-url 1)))
	(if (string-match "REV *= *\"*\\([^\" ]*\\)\"*" cur-url)
	    (setq cur-rev (w3-match cur-url 1))
	  (setq cur-rev nil))
	(if (string-match "URN *= *\"*\\([^\" ]*\\)\"*" cur-url)
	    (setq cur-rev (w3-match cur-url 1))
	  (setq cur-rev nil))
	(if (string-match "TITLE *= *\"\\([^\" ]*\\)\"" cur-url)
	    (setq cur-title (w3-match cur-url 1))
	  (setq cur-title nil))
	(if (string-match "METHODS *= *\"*\\([^\" ]*\\)\"*" cur-url)
	    (setq cur-meth (w3-match cur-url 1))
	  (setq cur-meth nil))
	(setq cur-txt (w3-fix-entities-in-string (buffer-substring start end)))
	(if (and cur-href
		 (not (string-match w3-nonrelative-link cur-href)))
	    (setq cur-href (w3-parse-relative-link cur-href)))
	(cond
	 ((and (eq w3-delimit-links 'linkname) cur-href)
	  (goto-char end)
	  (w3-insert (format "[%s]" cur-id)))
	 ((and (not (null w3-delimit-links)) cur-href)
	  (goto-char start)
	  (w3-insert w3-link-start-delimiter)
	  (goto-char (+ end (length w3-link-start-delimiter)))
	  (w3-insert w3-link-end-delimiter)
	  (setq end (+ end (length w3-link-start-delimiter)
		       (length w3-link-end-delimiter)))))
	(if cur-href
	    (w3-add-zone start end w3-node-style
			 (list 'w3 cur-id cur-href cur-txt
			       cur-urn cur-rel cur-rev
			       cur-meth cur-title) t)
	  (w3-add-zone start end w3-default-style
		       (list 'w3 cur-id cur-href cur-txt
			     cur-urn cur-rel cur-rev
			     cur-meth cur-title)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions to handle LINK attributes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-handle-links ()
  "Parse out the LINK attributes.
This will take the <LINK> attributes out of the current w3-working-buffer
and return an assoc list of the form (Rel or rev tag . url)"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (if buffer-read-only (toggle-read-only))
  (let* ((cur-href "")
	 (cur-rev "")
	 (result nil))
    (goto-char (point-min))
    (while (re-search-forward "<LINK[ \\\t]+" nil t)
      (let* ((start (prog1 (match-beginning 0) (replace-match "")))
	     (end (prog2 (re-search-forward ">" nil t) (point)
		    (replace-match "")))
	     (cur-lnk (prog1 (buffer-substring start end)
			(delete-region start end))))
	(if (string-match "RE[LV][ \\\t]*=\"*\\([^ \\\t\"]*\\)\"*" cur-lnk)
	    (setq cur-rev 
		  (mapcar 'car (w3-split (downcase (w3-match cur-lnk 1))
					 ",[ \\\t]*")))
	  (setq cur-rev '("unkown")))
	(if (string-match "HREF *= *\"*[ \\\t]*\\([^>\"]*\\)\"*" cur-lnk)
	    (setq cur-href (w3-match cur-lnk 1))
	  (setq cur-href nil))
	(if (and cur-href (not (string-match w3-nonrelative-link cur-href)))
	    (setq cur-href (w3-parse-relative-link cur-href)))
	(while cur-rev
	  (setq result (cons (cons (car cur-rev) cur-href) result)
		cur-rev (cdr cur-rev)))))
    result))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Functions to handle formatting an html buffer
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-prepare-buffer ()
  "Function to prepare w3-buffer for processing.  This will completely
reformat a buffer - if you just want to parse out links, see the documentation
for w3-build-links-list."
  (w3-lazy-message "Parsing...")
  (set-buffer w3-working-buffer)
  (let ((annos (w3-fetch-personal-annotations)))
    (if annos
	(progn
	  (goto-char (point-max))
	  (w3-insert
	   "<P>--------------\n<H1>Personal Annotations</H1><P><UL>")
	  (while annos
	    (goto-char (point-max))
	    (w3-insert "\n<LI> " (car annos))
	    (setq annos (cdr annos)))
	  (w3-insert "</UL>"))))
  (let ((hdrs w3-show-headers)
	x y)
    (goto-char (setq y (point-max)))
    (while hdrs
      (if (setq x (assoc (car hdrs) w3-current-mime-headers))
	  (w3-insert "<LI> <B>" (car x) "</B> :" (w3-insert-entities-in-string
					       (cdr x))))
      (setq hdrs (cdr hdrs)))
    (if (= y (point-max))
	nil
      (w3-insert "</UL>")
      (goto-char y)
      (w3-insert "<HR><UL>")))    
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (run-hooks 'w3-file-prepare-hooks)
  (setq fill-column (- (or w3-strict-width (window-width)) w3-right-border))
  (w3-replace-regexp (format "[%c%c%c]" ?\r ? ?) "")
  (let ((case-fold-search t)
	(ttl "")
	(pltxt nil))
    (goto-char (point-min))
    (w3-replace-regexp "<!--[.\\\n]--[\\\t\\\n]>" "")
    (goto-char (point-min))
    (if (re-search-forward "<PLAINTEXT>" nil t)
	(progn
	  (replace-match "")
	  (setq pltxt (buffer-substring (point) (point-max)))
	  (delete-region (point) (point-max))))
    (w3-replace-regexp "<\\(TBL[^>]*\\)>" "<\\1>\n<PRE>")
    (w3-replace-regexp "</TBL>" "</TBL>\n</PRE>")
    (w3-replace-regexp "<TEXTAREA" "<PRE><TEXTAREA")
    (w3-replace-regexp "</TEXTAREA>" "</TEXTAREA></PRE>")
    (w3-replace-regexp "<LISTING>" "<PRE>")
    (w3-replace-regexp "</LISTING>" "</PRE>")
    (w3-replace-regexp "<DL[^>]*>" "<DL>")
    (goto-char (point-min))
    (w3-balance-pre)
    (w3-balance-xmp)
    (w3-lazy-message "Parsing...")
    (w3-handle-arbitrary-tags)
    (w3-check-index)
    (w3-replace-regexp "<LIT>" "<PRE>")
    (w3-replace-regexp "</LIT>" "</PRE>")
    (w3-fix-xmp)
    (w3-fix-pre)
    (w3-fix-render-hints)
    (w3-handle-footnotes)
    (w3-handle-notes)
    (setq w3-current-links (w3-handle-links))
    (w3-fix-extras)
    (goto-char (point-min))
    (w3-lazy-message "Parsing...")
    (w3-replace-regexp "[ \\\t]*<SP>[ \\\t]*" "<SP>")
    (w3-replace-regexp "[ \\\t]*&nbsp;[ \\\t]*" "&nbsp;")
    (w3-handle-whitespace)
    (w3-handle-headers)
    (w3-lazy-message "Parsing...")
    (w3-restore-pre)
    (goto-char (point-min))
    (let* ((x (w3-handle-base))
	   (w3-delay-image-loads t)
	   (w3-current-file (and (not (car x)) w3-current-file))
	   (w3-current-server (and (not (car x)) w3-current-server))
	   (w3-current-type (and (not (car x)) w3-current-type))
	   (w3-current-user (and (not (car x)) w3-current-user))
	   (w3-current-port (and (not (car x)) w3-current-port)))
      (cond
       ((car x)				; there was a <base> tag
	(cond
	 ((eq (car x) 'http)
	  (setq x (w3-grok-http-href (cdr x))
		w3-current-type "http"
		w3-current-server (nth 0 x)
		w3-current-port (nth 1 x)
		w3-current-file (nth 2 x)))
	 ((or (eq (car x) 'file)
	      (eq (car x) 'ftp))
	  (setq x (w3-grok-file-href (cdr x))
		w3-current-type (and (nth 0 x) "ftp")
		w3-current-user (nth 0 x)
		w3-current-server (nth 1 x)
		w3-current-file (nth 2 x)))
	 ((eq (car x) 'gopher)
	  (setq x (w3-grok-gopher-href (cdr x))
		w3-current-type "gopher"
		w3-current-server (nth 0 x)
		w3-current-port (nth 1 x)
		w3-current-file (nth 2 x)))))
       (t nil))
      (w3-handle-graphics)
      (w3-build-links-list))
    (w3-handle-forms)
    (w3-do-lists)
    (w3-replace-regexp "<LI>" "\n\t*")
    (w3-replace-regexp "<DT>" "\n<DT>")
    (w3-replace-regexp "<DD>" "\n\t*")
    (goto-char (point-min))
    (let ((st (if (re-search-forward "<TITLE>" nil t)
		  (prog1
		      (match-beginning 0)
		    (replace-match "")) nil))
	  (nd (if (re-search-forward "</TITLE>" nil t)
		  (prog1
		      (match-beginning 0)
		    (replace-match ""))
		nil)))
      (if st
	  (progn
	    (setq ttl (w3-fix-entities-in-string
		       (w3-strip-leading-spaces
			(w3-eat-trailing-space
			 (buffer-substring st nd)))))
	    (delete-region st nd))
	(setq ttl (w3-basepath w3-current-file t)))
      (if (> (length ttl) 50) (setq ttl (substring ttl 0 50)))
      (setq ttl (w3-generate-new-buffer-name ttl)))
    (w3-fix-paragraphs)
    (w3-replace-regexp "<SP>" " ")
    (w3-fix-unknown-tags)
    (w3-fix-ampersands)
    (w3-restore-xmp)
    (goto-char (point-min))
    (set-buffer w3-working-buffer)
    (w3-mode)
    (rename-buffer ttl)
    (if w3-mutable-windows
	(pop-to-buffer ttl)
      (switch-to-buffer ttl))
    (if pltxt (progn (goto-char (point-max)) (w3-insert "\n" pltxt)))
    (goto-char (point-min))
    (while (looking-at "[ \\\n]")
      (delete-char 1))
    (if (boundp 'MULE)
	(w3-mule-attribute-zones w3-zones-list w3-mule-attribute))
    (if (not buffer-read-only) (toggle-read-only))
    (w3-lazy-message "Done.")
    (w3-fix-extent-endpoints)
    (goto-char (point-min))
    (if w3-find-this-link
	(w3-find-specific-link w3-find-this-link))
    (run-hooks 'w3-file-done-hooks)
    (if (and (bufferp w3-current-last-buffer)
	     (save-excursion
	       (set-buffer w3-current-last-buffer)
	       (eq major-mode 'w3-mode))
	     (not w3-keep-old-buffers))
	(kill-buffer w3-current-last-buffer))
    (if w3-keep-history
	(let ((url (w3-view-url t)))
	  (if (and (not (assoc url w3-history-list))
		   (not (equal url "file:historylist")))
	      (setq w3-history-list
		    (cons (cons url ttl) w3-history-list)))))
    (w3-lazy-message "Done.")
    (cond
     ((not (fboundp 'w3-insert-graphic)) nil)	; No graphics abilities
     (w3-delay-image-loads nil)			; Wait for graphics
     (t
      (message "Processing images...")		; Grab the images
      (w3-load-delayed-images)))
    (if (get-buffer "Conversion errors")
	(switch-to-buffer-other-window "Conversion errors"))
    (if w3-running-epoch (set-variable 'buffer-style w3-default-style))
    (if w3-running-FSF19 (setq w3-zones-list (w3-only-links)))
    (sit-for 0)))

(defun w3-handle-base ()
  "Handle BASE tag"
  (let (base url)
    (goto-char (point-min))
    (if (re-search-forward "<BASE\\([^>]+\\)>" nil t)
	(progn
	  (setq base (buffer-substring (match-beginning 1) (match-end 1)))
	  (replace-match "")
	  (if (not (string-match "HREF[ \\\t]*=[ \\\t]*\"\\([^\"]+\\)\"" base))
	      (message "Malformed 'BASE' tag.")
	    (setq url (w3-match base 1)))))
    (if (stringp url)
	(if (string-match "^\\([^:]+\\):/+" url)
	    (setq base (intern (downcase (w3-match url 1))))))
    (cons base url)))

(defun w3-handle-notes ()
  "Handle NOTE tags, as per the HTML+ 'Notes and Admonishments' section."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let (role img x)
    (while (re-search-forward "<NOTE\\([^>]*\\)>" nil t)
      (setq x (buffer-substring (match-beginning 1) (match-end 1)))
      (replace-match "<HR><BR><B>")
      (if (string-match "ROLE=\"\\([^\"]+\\)\"" x)
	  (setq role (substring x (match-beginning 1) (match-end 1)))
	(setq role "NOTE"))
      (if (string-match "SRC=\"\\([^\"]+\\)\"" x)
	  (setq img (substring x (match-beginning 1) (match-end 1)))
	(setq img nil))
      (if img
	  (w3-insert (format "<IMG SRC=\"%s\" ALIGN=\"CENTER\">" img)))
      (w3-insert (format "<B>%s:</B>" role))))
  (w3-replace-regexp "</NOTE>" "<BR><HR>"))

(defun w3-handle-footnotes ()
  "Handle footnotes, margin notes, etc, from the HTML+ spec"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (if (re-search-forward "<FOOTNOTE>" nil t)
      (progn
	(goto-char (point-max))
	(w3-insert "<P>Footnotes<BR><HR>")
	(goto-char (point-min))))
  (let ((fcounter 1) st nd txt)
    (while (re-search-forward "<FOOTNOTE>" nil t)
      (setq st (prog1 (match-beginning 0) (replace-match ""))
	    nd (if (re-search-forward "</FOOTNOTE>" nil t)
		   (prog1
		       (match-beginning 0)
		     (replace-match ""))
		 (progn (end-of-line) (point)))
	    txt (buffer-substring st nd))
      (delete-region st nd)
      (goto-char st)
      (w3-insert (format "<A HREF=\"#w3-internal-footnote%d\">%d</A>"
		      fcounter fcounter))
      (goto-char (point-max))
      (w3-insert (format "<P ID=\"w3-internal-footnote%d\">%d. "
		      fcounter fcounter) txt)
      (setq fcounter (1+ fcounter))
      (goto-char (point-min)))
    (while (re-search-forward "<MARGIN>" nil t)
      (setq st (prog1 (match-beginning 0) (replace-match ""))
	    nd (if (re-search-forward "</MARGIN>" nil t)
		   (prog1
		       (match-beginning 0)
		     (replace-match ""))
		 (progn (end-of-line) (point)))
	    txt (buffer-substring st nd))
      (delete-region st nd)
      (goto-char st)
      (w3-insert (format "<A HREF=\"#w3-internal-footnote%d\">%d</A>"
		      fcounter fcounter))
      (goto-char (point-max))
      (w3-insert (format "<P ID=\"w3-internal-footnote%d\">%d. "
		      fcounter fcounter) txt)
      (setq fcounter (1+ fcounter))
      (goto-char (point-min)))))    
       
(defun w3-fix-render-hints ()
  "Parse out the RENDER hints ala the HTML+ specification."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let (x tag sty)
    (while (re-search-forward "<RENDER\\([^>]+\\)>" nil t)
      (setq x (buffer-substring (match-beginning 1) (match-end 1)))
      (replace-match "")
      (if (string-match "TAG=\"\\([^\"]+\\)\"" x)
	  (setq tag (substring x (match-beginning 1) (match-end 1))))
      (if (string-match "STYLE=\"\\([^\"]+\\)\"" x)
	  (setq sty (substring x (match-beginning 1) (match-end 1))))
      (w3-replace-regexp (format "<%s>" tag) (format "<%s>" sty))
      (w3-replace-regexp (format "</%s>" tag) (format "</%s>" sty)))))

(defun w3-handle-arbitrary-tags ()
  "Find occurences of <!ENTITY ...> and replace them correctly."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (while (re-search-forward
	  "<!ENTITY[ \\\t]+\\([^ ]*\\)[ \\\t]+\"\\([^\"]*\\)\">" nil t)
    (let ((entity (buffer-substring (match-beginning 1) (match-end 1)))
	  (defn   (buffer-substring (match-beginning 2) (match-end 2))))
      (replace-match "")
      (w3-replace-regexp (regexp-quote (format "&%s;" entity)) defn))))

(defun w3-balance-xmp ()
  "This function will attempt to balance embedded plaintext elements
<XMP> tags.  This is necessary or the parser will fail some critical
regular expression matches."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let* ((st (w3-count-occurences "<XMP>"))
	 (nd (w3-count-occurences "</XMP>"))
	 (df (- st nd)))
    (goto-char (point-max))
    (while (> df 0)
      (setq df (1- df))
      (w3-insert "</XMP>\n"))))

(defun w3-balance-pre ()
  "This function will attempt to balance embedded plaintext elements
(<PRE> tags).  This is necessary or the parser will fail some critical
regular expression matches."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let* ((st (w3-count-occurences "<PRE[^>]*>"))
	 (nd (w3-count-occurences "</PRE>"))
	 (df (- st nd)))
    (goto-char (point-max))
    (while (> df 0)
      (setq df (1- df))
      (w3-insert "</PRE>\n"))))

(defun w3-fix-extras ()
  "Replace <B>, <I>, etc tags in the buffer.  Appropriate zones will be
created, and highlighting will be added when possible."
  (if (not w3-setup-done) (w3-do-setup))
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (while (re-search-forward w3-style-regexp nil t)
    (let* ((st (match-beginning 0))
	   (dastyle (upcase (buffer-substring (match-beginning 1)
					      (match-end 1))))
	   (nd (progn
		 (replace-match "")
		 (if (re-search-forward (concat "</" dastyle ">") nil t)
		     (prog1
			 (match-beginning 0)
		       (replace-match ""))
		   (point))))
	   (sty (w3-lookup-style dastyle))
	   (ltrs (cdr (assoc dastyle w3-style-chars-assoc))))
      (w3-add-zone st nd sty '(w3style))
      (if (and ltrs w3-delimit-emphasis)
	  (progn
	    (goto-char nd)
	    (w3-insert (cdr ltrs))
	    (goto-char st)
	    (w3-insert (car ltrs)))))))

(defun w3-fix-ampersands ()
  "Replace &#XXX with ASCII character XXX."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (let ((tmp w3-html-entities)
	(case-fold-search nil))
    (while (re-search-forward "&#\\([0-9]+\\);*" nil t)
      (replace-match (char-to-string 
		      (string-to-int (buffer-substring (match-beginning 1)
						       (match-end 1))))))
    (while tmp
      (w3-replace-regexp (car (car tmp)) (cdr (car tmp)))
      (setq tmp (cdr tmp))))
  (goto-char (point-min)))

(defun w3-fix-pre ()
  "Extract <PRE> fields, and put them back in later."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (setq w3-pre-data nil
	w3-pre-data-count 0)
  (while (re-search-forward "<PRE[^>]*>" nil t)
    (let* ((start (prog1 (match-beginning 0) (replace-match "")))
	   (end (prog2
		    (re-search-forward "</PRE>" nil t)
		    (match-beginning 0)
		  (replace-match "")))
	   (repl (not (string= "" (w3-eat-trailing-space
				   (buffer-substring start end))))))
      (cond
       (repl 
	(setq w3-pre-data-count (1+ w3-pre-data-count)
	      w3-pre-data (cons (list w3-pre-data-count
				      (buffer-substring start end))
				w3-pre-data))
	(delete-region start end)
	(goto-char start)
	(w3-insert "***PREDATA" (int-to-string w3-pre-data-count)))
       (t (delete-region start end))))))

(defun w3-restore-pre (&optional done)
  "Restore the <PRE> fields"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (while (> w3-pre-data-count 0)
    (re-search-forward (concat "***PREDATA" (int-to-string w3-pre-data-count))
		       nil t)
    (replace-match (concat (if (not done) "<PRE>" "\n")
			   (car (cdr (assoc w3-pre-data-count w3-pre-data)))
			   (if (not done) "</PRE>" "\n")) t t)
    (goto-char (point-min))
    (setq w3-pre-data-count (1- w3-pre-data-count))))

(defun w3-fix-xmp ()
  "Extract <XMP> fields, and put them back in later."
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (setq w3-xmp-data nil
	w3-xmp-data-count 0)
  (while (re-search-forward "<XMP>" nil t)
    (let* ((start (match-beginning 0))
	   (end (progn (re-search-forward "</XMP>" nil t)
		       (point))))
      (setq w3-xmp-data-count (1+ w3-xmp-data-count)
	    w3-xmp-data (cons (list w3-xmp-data-count
				    (buffer-substring start end)) w3-xmp-data))
      (delete-region start end)
      (goto-char start)
      (w3-insert "***XMPDATA" (int-to-string w3-xmp-data-count)))))

(defun w3-restore-xmp ()
  "Restore the <XMP> fields"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (while (> w3-xmp-data-count 0)
    (goto-char (point-min))
    (re-search-forward (concat "***XMPDATA" (int-to-string w3-xmp-data-count))
		       nil t)
    (replace-match (concat "\n"
			   (substring
			    (car (cdr (assoc w3-xmp-data-count w3-xmp-data)))
			    5 -6) "\n") t t)
    (setq w3-xmp-data-count (1- w3-xmp-data-count))))

(defun w3-check-index ()
  "Check to see if this is an indexed file.  If it is, update the mode line"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (if (re-search-forward "\\\n*<ISINDEX>\\\n*" nil t)
      (progn
	(setq w3-current-isindex t)
	(replace-match
	 (if (and w3-use-forms-index
		  (equal w3-current-type "http"))
	     (concat
	      "<FORM>\nThis is a searchable index.  Search for:"
	      " <INPUT NAME=\"isindex\"><P></FORM>")
	   "") t))
    (setq w3-current-isindex nil)))

(defun w3-handle-whitespace ()
  "Fix newlines, tabs, and spaces"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (w3-replace-regexp "<P>\\\n*" "<P>")
  (w3-replace-regexp "\\\n" " ")
  (w3-replace-regexp "\t" " ")
  (w3-replace-regexp "  +" " ")
  (w3-replace-regexp "\\\. +" ".  "))

(defun w3-handle-headers ()
  "Do the headers"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (while (re-search-forward
	  "[ \\\t\\\n]*\\(<P[^>]*>\\)*[ \\\t\\\n]*<H\\([0-9]+\\)\\([^>]*\\)>"
	  nil t)
    (let* ((siz (buffer-substring (match-beginning 2) (match-end 2)))
	   (tags (buffer-substring (match-beginning 3) (match-end 3)))
	   x y
	   (st (prog1
		   (match-beginning 0)
		 (setq x 
		       (if (match-beginning 1)
			   (w3-eat-trailing-space
			    (w3-strip-leading-spaces
			     (buffer-substring (match-beginning 1)
					       (match-end 1)))) "<P>"))
		 (replace-match "")))
	   (end (progn
		  (if (re-search-forward
		       (concat 
		       (format "</H%s>" siz)
		       "[ \\\t\\\n]*\\(<P[^>]*>\\)*") nil t)
		      (prog1
			  (match-beginning 0)
			(setq y
			      (if (match-beginning 1)
				  (w3-eat-trailing-space
				   (w3-strip-leading-spaces
				    (buffer-substring
				     (match-beginning 1) (match-end 1))))
				"<P>"))
			(replace-match ""))
		    (progn (end-of-line) (setq y "<P>") (point)))))
	   (forms (cdr (assoc siz w3-header-chars-assoc))))
      (w3-add-zone st end w3-header-style
		   (cons 'w3header
			 (list
			  (if (string-match "ID=\"\\([^\"]+\\)\"" tags)
			      (substring tags (match-beginning 1)
					 (match-end 1))
			    nil) nil nil nil nil nil nil nil)))
      (if (and forms w3-delimit-emphasis)
	  (progn
	    (if (nth 2 forms) (funcall (nth 2 forms) st end))
	    (goto-char end)
	    (w3-insert (nth 1 forms) y)
	    (goto-char st)
	    (w3-insert x (nth 0 forms)))
	(progn
	  (goto-char end) (w3-insert y)
	  (goto-char st) (w3-insert x)))))
  (goto-char (point-min)))

(defun w3-fix-horizontal-rules ()
  "Replace all the <HR> tags"
  (goto-char (point-min))
  (if (and (not w3-hrule-pixmap) w3-running-epoch)
      (setq w3-hrule-pixmap (w3-create-hrule)))
  (while (re-search-forward "<[Hh][rR]>" nil t)
;    (if (fboundp 'add-graphic-zone)
;	(progn
;	  (replace-match "-\n")
;	  (add-graphic-zone
;	   w3-hrule-pixmap (match-beginning 0)
;	   (1+ (match-beginning 0)) 50 nil))
      (replace-match (format "<P>%s<P>"
			     (make-string (- (or w3-strict-width
						 (window-width))
					     w3-right-border)
					  w3-horizontal-rule-char)))))

(defun w3-fix-unknown-tags (&optional pt recur)
  "Remove unknown tags in a buffer"
  (set-buffer w3-working-buffer)
  (goto-char (point-min))
  (if (re-search-forward "<\\(PRE\\|XMP\\)>" nil t)
      (let ((st (if pt pt (point-min)))
	    (nd (- (point) 5))
	    (tp (buffer-substring (match-beginning 1) (match-end 1))))
	(replace-match "")
	(save-restriction
	  (narrow-to-region st nd)
	  (w3-replace-regexp "<[^>]*>" ""))
	(re-search-forward (format "</%s>" tp) nil t)
	(replace-match "")
	(w3-fix-unknown-tags (point) t))
    (narrow-to-region (point) (point-max))
    (w3-replace-regexp "<[^>]*>" "")
    (widen)))

(defun w3-fix-paragraphs-in-region ()
  "Fill paragraphs in the visible part of the buffer"
  (set-buffer w3-working-buffer)
  (w3-fix-horizontal-rules)
  (goto-char (point-min))
  (w3-replace-regexp "<[bB][Rr]> *" "<X>\n<BR>")
  (w3-replace-regexp "\\\n\\\n\\\t" "\n\t")
  (goto-char (point-min))
  (let (ptag st)
    (while (re-search-forward "<P\\([^>]*\\)>" nil t)
      (setq ptag (buffer-substring (match-beginning 1) (match-end 1)))
      (setq st (match-beginning 0))
      (replace-match (concat "\n\n" fill-prefix))
      (if (string-match "ID=\"\\([^\"]+\\)\"" ptag)
	  (w3-add-zone st (progn (end-of-line) (point))
		       w3-default-style
		       (cons 'w3par
			     (list (substring ptag (match-beginning 1)
					      (match-end 1))
				   nil nil nil nil nil nil nil))))))
  (w3-replace-regexp "^ +" "")
  (goto-char (point-min))
  (while (re-search-forward "^[^\\\n\\\t]" nil t)
    (fill-region (progn (beginning-of-line) (point))
		 (progn (end-of-line) (point))))
  (w3-replace-regexp "<X>\\\n+<BR>" "\n")
  (w3-replace-regexp "\\\n\\\n+" "\n\n")
  (w3-replace-regexp "<[sS][Pp]>" " "))

(defun w3-fix-paragraphs (&optional pt recur)
  "Fix filling of paragraphs in a new buffer"
  (set-buffer w3-working-buffer)
  (goto-char (if pt pt (point-min)))
  (if (re-search-forward "<\\(PRE\\|XMP\\)>" nil t)
      (let ((st (if pt pt (point-min)))
	    (nd (- (point) 5))
	    (tp (buffer-substring (match-beginning 1) (match-end 1))))
	(save-restriction
	  (narrow-to-region st nd)
	  (w3-fix-paragraphs-in-region))
	(re-search-forward (format "</%s>" tp) nil t)
	(w3-fix-paragraphs (point) t))
    (narrow-to-region (point) (point-max))
    (w3-fix-paragraphs-in-region)
    (widen)))

(defun w3-handle-graphics ()
  "A function to parse out IMG tags.  In epoch, this will actually
insert the picture into the buffer.  The ALT attribute is displayed
when not in epoch (or when epoch fails to read in the graphic
correctly."
  (set-buffer w3-working-buffer)
  (if (get-buffer "Conversion errors") (kill-buffer "Conversion errors"))
  (goto-char (point-min))
  (if (fboundp 'w3-insert-graphic)
      (while (re-search-forward "<IMG[ \\\t]+\\([^>]+\\)>" nil t)
	(let ((img (buffer-substring (match-beginning 1) (match-end 1)))
	      (src "")
	      (align 'center)
	      (alt nil)
	      (st (match-beginning 0)))
	  (replace-match "")
	  (if (string-match "SRC[ \\\t]*=[ \\\t]*\"*\\([^\"]+\\)\"" img)
	      (setq src (substring img (match-beginning 1) (match-end 1)))
	    (setq src ""))
	  (if (string-match "ALT[ \\\t\\\n]*=[ \\\t\\\n]*\"\\([^\"]*\\)\""
			    img)
	      (setq alt (substring img (match-beginning 1) (match-end 1)))
	    (setq alt (concat "&lt;IMAGE(" (w3-basepath src t) ")&gt;")))
	  (if (string-match "ALIGN[ \\\t]*=[ \\\t]*\"*\\([^\"]+\\)\"" img)
	      (setq align (read (downcase (substring img
						     (match-beginning 1)
						     (match-end 1))))))
	  (if w3-delay-image-loads
	      (setq st (set-marker (make-marker) st)
		    w3-delayed-images
		    (cons (list src st align alt) w3-delayed-images))
	    (w3-insert-graphic src st align alt))))
    (progn
      (goto-char (point-min))
      (let ((alt "") (img "") (src "") st)
	(while (re-search-forward "<IMG[ \\\t\\\n]*\\([^>]+\\)>" nil t)
	  (setq img (buffer-substring (match-beginning 1) (match-end 1)))
	  (replace-match "")
	  (if (string-match "SRC[ \\\t]*=[ \\\t]*\"*\\([^\"]+\\)\"" img)
	      (setq src (substring img (match-beginning 1) (match-end 1)))
	    (setq src ""))
	  (if (string-match "ALT[ \\\t\\\n]*=[ \\\t\\\n]*\"\\([^\"]*\\)\"" img)
	      (setq alt (substring img (match-beginning 1) (match-end 1)))
	    (setq alt (concat "&lt;IMAGE(" (w3-basepath src t) ")&gt;")))
	  (setq st (point))
	  (w3-insert alt)
	  (w3-add-zone st (point) nil (list 'w3graphic src) t))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Shared graphics routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-convert-graphic-to-useable-format (buf fname xbm)
  "Convert the image data in buffer BUF into a format useable by
lemacs or epoch.  Second arg FNAME is the filename to redirect output
into.  If third arg XBM is t, convert it to an Xbitmap, otherwise
convert it to an XPM (recommended, as they can do color).  Returns a
filename containing the bitmap specification"
  (save-excursion
    (set-buffer buf)
    (let (converter)
      (if (not w3-current-mime-type)
	  (setq w3-current-mime-type (w3-extension-to-mime
				      (w3-file-extension w3-current-file))))
      (setq converter (assoc w3-current-mime-type w3-graphic-converter-alist))
      (if (not converter)
	  (message "Cannot convert %s to www/present!" w3-current-mime-type)
	(progn
	  (message "Converting image %s (%s)..." w3-current-file
		   w3-current-mime-type)
	  (shell-command-on-region
	   (point-min) (point-max)
	   (concat (cdr converter)
		   (if w3-max-colors
		       (format "ppmquant %d |" w3-max-colors) "")
		   (if xbm w3-ppmtoxbm-command w3-ppmtoxpm-command)
		   (if (and (not w3-running-epoch)
			    (w3-member w3-current-mime-type
				       '("image/xbm" "image/x-xbitmap"
					 "image/xbitmap")))
		       (format "|sed 's/c white/c %s/g'|sed 's/c black/c %s/g'"
			       (pixel-name (face-background 'default))
			       (pixel-name (face-foreground 'default))) "")
		   "> " fname) t))))))
  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Cacheing for documents
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-find-in-cache (url)
  "Return the document referenced by URL if it is in the document cache."
  (let ((x (cdr (assoc url w3-document-cache))))
    (set-buffer w3-working-buffer)
    (erase-buffer)
    (w3-insert x)
    (shell-command-on-region (point-min) (point-max) "gunzip -d -c" t)))

(defun w3-store-in-cache (url text)
  "Store the docuemnt in the cache."
  (save-excursion
    (set-buffer (get-buffer-create " *W3TEMPC*"))
    (erase-buffer)
    (w3-insert text)
    (shell-command-on-region (point-min) (point-max) "gzip -c" t)
    (setq text (buffer-string))
    (kill-buffer (current-buffer)))
  (let ((x (length w3-document-cache))
	(y (assoc url w3-document-cache)))
    (cond
     (y (setcdr y text))
     ((< x w3-cache-size)
      (setq w3-document-cache
	    (nconc w3-document-cache (list (cons url text)))))
     (t (setq w3-document-cache
	      (nconc (cdr w3-document-cache) (list (cons url text))))))))

(defun w3-load-flavors ()
  "Load the correct zone/font info for each flavor of emacs"
  (cond
   (w3-running-lemacs (require 'w3-lemacs))
   (w3-running-old-lemacs (require 'w3-old-lemacs))
   (w3-running-epoch  (require 'w3-epoch))
   (w3-running-FSF19  (require 'w3-emacs19))
   (t                 (require 'w3-emacs))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Automatic bug submission.                                               ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-submit-bug ()
  "Function to submit a bug to the programs maintainer"
  (interactive)
  (cond
   ((and w3-mutable-windows (fboundp w3-mail-other-window-command))
    (funcall w3-mail-other-window-command))
   ((fboundp w3-mail-command)
    (funcall w3-mail-command))
   (w3-mutable-windows (mail-other-window))
   (t (mail)))
  (mail-to)
  (w3-insert "wmperry@indiana.edu")
  (mail-subject)
  (w3-insert "Bug found in w3-mode")
  (re-search-forward mail-header-separator nil t)
  (next-line 1)
  (while (< (current-column) 29) (w3-insert "-"))
  (w3-insert "Description of System:")
  (while (< (current-column) 75) (w3-insert "-"))
  (w3-insert "\n")
  (string-match "WWW \\([^ ]*\\) \\(.*\\)" w3-version)
  (w3-insert "WWW Browser Version: "
	  (substring w3-version (match-beginning 1) (match-end 1))
	  ", of "
	  (substring w3-version (match-beginning 2) (match-end 2))
	  "\n"
	  "      Emacs Version: "
	  (substring (emacs-version) 0 (string-match " of" (emacs-version)))
	  (if w3-running-epoch "(Epoch)" "")
	  (if (boundp 'MULE) "(MULE)" "")
	  "\n"
	  (if window-system
	      (concat "      Window System: " (symbol-name window-system)
		      "-" window-system-version "\n") "")
	  "        System Type: "
	  (prin1-to-string system-type) "\n"
	  (if (featurep 'ange-ftp)
	      (concat "           Ange-FTP: " ange-ftp-version "\n") "")
	  (if (featurep 'efs)
	      (concat "                EFS: " efs-version "\n") ""))
  (while (< (current-column) 29) (w3-insert "-"))
  (w3-insert "Description of Problem:")
  (while (< (current-column) 75) (w3-insert "-"))
  (w3-insert "\n\n"))
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Support for searching						    ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-nuke-spaces-in-search (x)
  "Remove spaces from search strings . . ."
  (let ((new ""))
    (while (not (equal x ""))
      (setq new (concat new (if (= (string-to-char x) 32) "+" 
			      (substring x 0 1)))
	    x (substring x 1 nil)))
    new))

(defun w3-search ()
  "Perform a search, if this is a searchable index."
  (interactive)
  (cond
   ((not w3-current-isindex) (message "Not a searchable index!"))
   ((not (equal w3-current-type "http"))
    (message "Sorry, searching is not implemented on local files yet."))
   (t
    (let ((querystring (w3-nuke-spaces-in-search
			(read-string "Search on (+ separates keywords): "))))
      (w3-fetch (concat (w3-view-url t)
			(if (= ?? (string-to-char (substring w3-current-file
							     -1 nil))) "" "?")
			querystring))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Auto documentation, etc                                                 ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-help ()
  "Print documentation on w3 mode."
  (interactive)
  (let* ((funcs w3-doc-functions)
	 (funcstr "")
	 (vars w3-doc-variables)
	 (varstr "")
	 (keys nil))
    (while funcs
      (if (fboundp (car funcs))
	  (setq keys (where-is-internal (car funcs) w3-mode-map t)
		funcstr (format "%s\n<LI>%5s: %s" funcstr
				(if (null keys)
				    (format "M-x %s" (car funcs))
				  (if w3-running-FSF19
				      (key-description (car keys))
				    (key-description keys)))
				(documentation (car funcs)))))
      (setq funcs (cdr funcs)))
    (while vars
      (if (boundp (car vars))
	  (let* ((thevar (prin1-to-string (car vars)))
		 (doc (documentation-property (car vars)
					      'variable-documentation)))
	    (setq varstr
		  (format "%s\n<LI>%20s: %s\n" varstr thevar
			  (if (> (+ (length thevar) (length doc)) 80)
			      (concat "\n" doc)
			    doc)))))
      (setq vars (cdr vars)))
    (set-buffer (get-buffer-create w3-working-buffer))
    (if buffer-read-only (toggle-read-only))
    (erase-buffer)
    (w3-insert (format "<TITLE>Help For W3 V%s</TITLE>\n" w3-version-number))
    (w3-insert "<H1>Current keybindings:</H1>\n<HR><P>\n"
	    "<UL>" funcstr "</UL>")
    (w3-insert "<P><H1>Modifiable variables:</H1>\n<HR><P>\n"
	    "<UL>" varstr "</UL>")
    (setq w3-current-type "www"
	  w3-current-file "help.html")	  
    (goto-char (point-min))
    (w3-sentinel nil nil)))

(defun w3-remove-relative-links-helper (name)
  (cond
   ((string-match "\\\.\\\./" name)
    (let ((tmp (substring name (match-end 0) nil)))
      (if (= 0 (match-beginning 0))
	  (concat (w3-basepath (w3-basepath w3-current-file)) "/" tmp)
	(concat (w3-basepath (substring name 0
					(1- (match-beginning 0)))) "/" tmp))))
   ((string-match "\\\./" name)
    (if (= 0 (match-beginning 0))
	(substring name 2 nil)
      (concat
       (substring name 0 (match-beginning 0))
       (substring name (match-end 0) nil))))))

(defun w3-remove-relative-links (name)
  (while (string-match "\\\.+/" name)
    (setq name (w3-remove-relative-links-helper name)))
  name)

(defun w3-version ()
  "Show the version # of W3 in the minibuffer"
  (interactive)
  (message w3-version))

;;;###autoload
(defun w3 ()
  "Retrieve the default World Wide Web home page.
The World Wide Web is a global hypertext system started by CERN in
Switzerland in 1991.

The home page is specified by the variable w3-default-homepage.  The
document should be specified by its fully specified Uniform Resource
Locator.  The document will be parsed as HTML (if appropriate) and
displayed in a new buffer."
  (interactive)
  (if (not w3-setup-done) (w3-do-setup))
  (if (not (string-match ".*:.*" w3-default-homepage))
      (w3-fetch (concat "file:" w3-default-homepage))
    (w3-fetch w3-default-homepage)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Mode definition							    ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun w3-reload-document ()
  "Reload the current document"
  (interactive)
  (let ((tmp (w3-view-url t))
	(pnt (point)))
    (kill-buffer (current-buffer))
    (w3-fetch tmp)
    (goto-char pnt)))

(defun w3-leave-buffer ()
  "Bury this buffer,but don't kill it"
  (interactive)
  (let ((x w3-current-last-buffer))
    (if (and w3-running-FSF19
	     (or (eq window-system 'x)
		 (eq window-system 'pm)))
	(set-variable 'lucid-menu-bar-dirty-flag t))
    (bury-buffer (current-buffer))
    (if (and (bufferp x) (buffer-name x))
	(if w3-mutable-windows (pop-to-buffer x) (switch-to-buffer x)))))  

(defun w3-quit ()
  "Quit WWW mode"
  (interactive)
  (let ((x w3-current-last-buffer))
    (if (and w3-running-FSF19
	     (or (eq window-system 'x)
		 (eq window-system 'pm)))
	(set-variable 'lucid-menu-bar-dirty-flag t))
    (kill-buffer (current-buffer))
    (if (and (bufferp x) (buffer-name x))
	(if w3-mutable-windows (pop-to-buffer x) (switch-to-buffer x)))))
  
(defun w3-view-url (&optional no-show)
  "View the current document's URL"
  (interactive)
  (let ((url ""))
    (cond
     ((equal w3-current-type "gopher")
      (setq url (format "%s://%s%s/%s"
			w3-current-type w3-current-server
			(if (string= "70" w3-current-port) ""
			  (concat ":" w3-current-port))
			w3-current-file)))
     ((equal w3-current-type "http")
      (setq url (format  "%s://%s%s/%s" w3-current-type w3-current-server
			 (if (string= "80" w3-current-port) ""
			   (concat ":" w3-current-port))
			 (if (= ?/ (string-to-char w3-current-file))
			     (substring w3-current-file 1 nil)
			   w3-current-file))))
     ((equal w3-current-type "ftp")
      (setq url (format "%s://%s%s/%s" w3-current-type
			(if w3-current-user (concat w3-current-user "@") "")
			w3-current-server 
			(if (= 47 (string-to-char w3-current-file))
			    (substring w3-current-file 1 nil)
			  w3-current-file))))
     ((equal w3-current-type nil)
      (setq url (format "file:%s" w3-current-file)))
     ((equal w3-current-type "www")
      (setq url nil)))
    (if (not no-show) (message url) url)))
    
(defun w3-save-url (under-pt)
  "Save current url in the kill ring"
  (interactive "P")
  (let ((x (if under-pt (w3-view-this-url t) (w3-view-url t))))
    (setq kill-ring (cons x kill-ring))
    (setq kill-ring-yank-pointer kill-ring)
    (if (fboundp 'w3-store-in-x-clipboard) (w3-store-in-x-clipboard x))))

(defun w3-end-of-document ()
  "Go to end of document"
  (interactive)
  (goto-char (point-max)))

(defun w3-start-of-document ()
  "Go to start of document"
  (interactive)
  (goto-char (point-min)))

(defun w3-mail-to-author ()
  "Send mail to the author of this document, if possible."
  (interactive)
  (let ((x (assoc "made" w3-current-links)))
    (if x
	(w3-fetch (cdr x))
      (error "Cannot find the 'made' link for this document, sorry."))))

(defun w3-do-setup ()
  "Do setup - this is to avoid conflict with user settings when W3 is
dumped with emacs."
  (w3-load-flavors)
  (w3-setup-version-specifics)
  ; Create the fonts, etc in windowing systems
  (w3-create-faces)
  
  ; Read in the ~/.w3 file if it exists - could set up some of these
  ; defaults.  This file is where I will store configuration information
  ; once I write the auto-editing of variables/info, etc.
  (if (file-exists-p (expand-file-name "~/.w3"))
      (load-file (expand-file-name "~/.w3")))

  ; Set the w3-use-transparent with decent defaults
  (if (or w3-running-lemacs
	  w3-running-old-lemacs
	  w3-running-epoch
	  window-system)
      (setq w3-use-transparent nil))
  (and w3-use-transparent (require 'transparent))
  
  ; Set the password entry funtion based on user defaults or guess
  ; based on which remote-file-access package they are using.
  (cond
   (w3-passwd-entry-func nil)		; Already been set
   ((or (featurep 'efs)			; Using EFS
	(featurep 'efs-auto))		; or autoloading efs
    (setq w3-passwd-entry-func 'efs-read-passwd))
   ((featurep 'ange-ftp)		; Using ange-ftp
    (setq w3-passwd-entry-func 'ange-ftp-read-passwd))
   (t (error "You must use either ange-ftp or efs!")))

  ; Read in extra mime types that may not be defined in w3-mime-viewers
  (if (file-exists-p (expand-file-name "~/.mailcap")) (w3-parse-mailcap))
  (if (file-exists-p (expand-file-name "~/.mime-types")) (w3-parse-mime-types))
  
  ; Load in the hotlist if they haven't set it already
  (or w3-hotlist (w3-parse-hotlist))
  
  ; Load in their personal annotations if they haven't set them already
  (or w3-personal-annotations (w3-parse-personal-annotations))
  
  ; Set up the news service if they haven't done so
  (or w3-news-server (setq w3-news-server  (or (getenv "NNTPSERVER")
					       "news")))
  
  ; Set the default home page, honoring their defaults, then
  ; the standard WWW_HOME, then default to the documentation @ IU
  (or w3-default-homepage
      (setq w3-default-homepage
	    (or (getenv "WWW_HOME")
		"http://cs.indiana.edu/elisp/w3/docs.html")))
  
  ; Set up the MIME accept string if they haven't got it hardcoded yet
  (or w3-mime-accept-string
      (setq w3-mime-accept-string (w3-parse-viewer-types)))
  
  ; This isn't used yet, but just in case we ever need it for the
  ; graphics parsing routines - perhaps use this value to determine
  ; value for w3-max-colors?
  (or w3-color-planes (setq w3-color-planes
			    (cond
			     (w3-running-lemacs (x-display-planes))
			     (w3-running-old-lemacs (x-display-planes))
			     ((and w3-running-FSF19
				   (or (eq window-system 'x)
				       (eq window-system 'pm)))
			      (x-display-planes))
;			     ((and w3-running-FSF19
;				   (eq window-system 'dps))
;			      (dps-display-color-cells))
			     (w3-running-epoch 8)
			     (t nil))))
  
  ; This isn't used yet, but just in case we ever need it for the
  ; graphics parsing routines - perhaps use this value to determine
  ; value for w3-max-colors?
  (or w3-color-display (setq w3-color-display
			     (cond
			      (w3-running-lemacs (x-color-display-p))
			      (w3-running-old-lemacs (x-color-display-p))
			      ((and w3-running-FSF19
				    (or (eq window-system 'x)
					(eq window-system 'pm)))
			       (x-display-color-p))
;			      ((and w3-running-FSF19
;				    (eq window-system 'dps))
;			       (dps-display-color-p))
			      (w3-running-epoch t)
			      (t nil))))
  
  ; Set up the regular expression used to find styles.
  (or w3-style-regexp (setq w3-style-regexp
			    (concat "<\\("
				    (mapconcat 'car w3-style-assoc "\\|")
				    "\\)>")))
  ; Set up the entity definition for PGP and PEM authentication
  (or w3-pgp/pem-entity (setq w3-pgp/pem-entity
			      (format "%s@%s"  (user-real-login-name)
				      (system-name))))
  (run-hooks 'w3-load-hooks)
  (setq w3-setup-done t))

(defun w3-mode ()
  "Mode for viewing HTML documents.  Will try to bring up the document
specified by w3-default-homepage.
Current keymap is:
\\{w3-mode-map}"
  (interactive)
  (or w3-setup-done (w3-do-setup))
  (let ((tmp (list w3-current-file w3-current-server
		   w3-current-type w3-current-port
		   w3-current-isindex
		   w3-zones-list
		   w3-current-last-buffer
		   w3-current-mime-viewer
		   w3-current-mime-headers
		   w3-current-mime-type
		   w3-current-mime-encoding
		   w3-current-links
		   w3-delayed-images
		   w3-current-next-link
		   w3-current-last-link
		   tab-width
		   tab-stop-list
		   w3-current-user))) ; keep some variables
    (kill-all-local-variables)
    (use-local-map w3-mode-map)
    (setq major-mode 'w3-mode)
    (setq mode-name "WWW")
    (run-hooks 'w3-mode-hooks)
    (setq				; restore buffer-local variables
     w3-current-file (nth 0 tmp)
     w3-current-server (nth 1 tmp)
     w3-current-type (nth 2 tmp)
     w3-current-port (nth 3 tmp)
     w3-current-isindex (nth 4 tmp)
     w3-zones-list (nth 5 tmp)
     w3-current-last-buffer (nth 6 tmp)
     w3-current-mime-viewer (nth 7 tmp)
     w3-current-mime-headers (nth 8 tmp)
     w3-current-mime-type (nth 9 tmp)
     w3-current-mime-encoding (nth 10 tmp)
     w3-current-links (nth 11 tmp)
     w3-delayed-images (nth 12 tmp)
     w3-current-next-link (nth 13 tmp)
     w3-current-last-link (nth 14 tmp)
     tab-width (nth 15 tmp)
     tab-stop-list (nth 16 tmp)
     w3-current-user (nth 17 tmp)
     )
    (w3-mode-version-specifics)
    (if (and w3-current-isindex (equal w3-current-type "http"))
	(setq mode-line-process "-Searchable"))))

(provide 'w3)
